//
// Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
// Copyright 2012, 2013 SAP AG. All rights reserved.
// DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
//
// This code is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 2 only, as
// published by the Free Software Foundation.
//
// This code is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// version 2 for more details (a copy is included in the LICENSE file that
// accompanied this code).
//
// You should have received a copy of the GNU General Public License version
// 2 along with this work; if not, write to the Free Software Foundation,
// Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
// or visit www.oracle.com if you need additional information or have any
// questions.
//
//
//
// Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
// Copyright 2012, 2014 SAP AG. All rights reserved.
// DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
//
// This code is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 2 only, as
// published by the Free Software Foundation.
//
// This code is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// version 2 for more details (a copy is included in the LICENSE file that
// accompanied this code).
//
// You should have received a copy of the GNU General Public License version
// 2 along with this work; if not, write to the Free Software Foundation,
// Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
// or visit www.oracle.com if you need additional information or have any
// questions.
//
//

// Machine Generated File.  Do Not Edit!

#include "precompiled.hpp"
#include "adfiles/adGlobals_ppc_64.hpp"
#include "adfiles/ad_ppc_64.hpp"
#include "memory/allocation.inline.hpp"
#include "asm/assembler.hpp"
#include "code/compiledIC.hpp"
#include "code/vmreg.hpp"
#include "gc_interface/collectedHeap.inline.hpp"
#include "oops/compiledICHolderOop.hpp"
#include "oops/markOop.hpp"
#include "oops/methodOop.hpp"
#include "oops/oop.inline.hpp"
#include "oops/oop.inline2.hpp"
#include "opto/cfgnode.hpp"
#include "opto/locknode.hpp"
#include "opto/opcodes.hpp"
#include "opto/regalloc.hpp"
#include "opto/regmask.hpp"
#include "opto/runtime.hpp"
#include "runtime/biasedLocking.hpp"
#include "runtime/sharedRuntime.hpp"
#include "runtime/stubRoutines.hpp"
#include "utilities/growableArray.hpp"
#include "assembler_ppc.inline.hpp"
#include "nativeInst_ppc.hpp"
#include "vmreg_ppc.inline.hpp"

//SourceForm

#line 907 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"


// Optimize load-acquire.
//
// Check if acquire is unnecessary due to following operation that does
// acquire anyways.
// Walk the pattern:
//
//      n: Load.acq
//           |
//      MemBarAcquire
//       |         |
//  Proj(ctrl)  Proj(mem)
//       |         |
//   MemBarRelease/Volatile
//
bool followed_by_acquire(const Node *load) {
  assert(load->is_Load(), "So far implemented only for loads.");

  // Find MemBarAcquire.
  const Node *mba = NULL;
  for (DUIterator_Fast imax, i = load->fast_outs(imax); i < imax; i++) {
    const Node *out = load->fast_out(i);
    if (out->Opcode() == Op_MemBarAcquire) {
      if (out->in(0) == load) continue; // Skip control edge, membar should be found via precedence edge.
      mba = out;
      break;
    }
  }
  if (!mba) return false;

  // Find following MemBar node.
  //
  // The following node must be reachable by control AND memory
  // edge to assure no other operations are in between the two nodes.
  //
  // So first get the Proj node, mem_proj, to use it to iterate forward.
  Node *mem_proj = NULL;
  for (DUIterator_Fast imax, i = mba->fast_outs(imax); i < imax; i++) {
    mem_proj = mba->fast_out(i);      // Throw out-of-bounds if proj not found
    assert(mem_proj->is_Proj(), "only projections here");
    ProjNode *proj = mem_proj->as_Proj();
    if (proj->_con == TypeFunc::Memory &&
        !Compile::current()->node_arena()->contains(mem_proj)) // Unmatched old-space only
      break;
  }
  assert(mem_proj->as_Proj()->_con == TypeFunc::Memory, "Graph broken");

  // Search MemBar behind Proj. If there are other memory operations
  // behind the Proj we lost.
  for (DUIterator_Fast jmax, j = mem_proj->fast_outs(jmax); j < jmax; j++) {
    Node *x = mem_proj->fast_out(j);
    // Proj might have an edge to a store or load node which precedes the membar.
    if (x->is_Mem()) return false;

    // On PPC64 release and volatile are implemented by an instruction
    // that also has acquire semantics. I.e. there is no need for an
    // acquire before these.
    int xop = x->Opcode();
    if (xop == Op_MemBarRelease || xop == Op_MemBarVolatile) {
      // Make sure we're not missing Call/Phi/MergeMem by checking
      // control edges. The control edge must directly lead back
      // to the MemBarAcquire
      Node *ctrl_proj = x->in(0);
      if (ctrl_proj->is_Proj() && ctrl_proj->in(0) == mba) {
        return true;
      }
    }
  }

  return false;
}

#define __ _masm.

// Tertiary op of a LoadP or StoreP encoding.
#define REGP_OP true

static ConditionRegister reg_to_ConditionRegister_object(int register_encoding);
static FloatRegister reg_to_FloatRegister_object(int register_encoding);
static Register reg_to_register_object(int register_encoding);

// ****************************************************************************

// REQUIRED FUNCTIONALITY

// !!!!! Special hack to get all type of calls to specify the byte offset
//       from the start of the call to the point where the return address
//       will point.

// PPC port: Removed use of lazy constant construct.

int MachCallStaticJavaNode::ret_addr_offset() {
  // It's only a single branch-and-link instruction.
  return 4;
}

int MachCallDynamicJavaNode::ret_addr_offset() {
  // Offset is 4 with postalloc expanded calls (bl is one instruction). We use
  // postalloc expanded calls if we use inline caches and do not update method data.
  if (UseInlineCaches)
    return 4;

  int vtable_index = this->_vtable_index;
  if (vtable_index < 0) {
    // Must be invalid_vtable_index, not nonvirtual_vtable_index.
    assert(vtable_index == methodOopDesc::invalid_vtable_index, "correct sentinel value");
    return 12;
  } else {
    assert(!UseInlineCaches, "expect vtable calls only if not using ICs");
    return 24;
  }
}

int MachCallRuntimeNode::ret_addr_offset() {
#if defined(ABI_ELFv2)
  return 28;
#else
  return 40;
#endif
}

//=============================================================================

// condition code conversions

static int cc_to_boint(int cc) {
  return Assembler::bcondCRbiIs0 | (cc & 8);
}

static int cc_to_inverse_boint(int cc) {
  return Assembler::bcondCRbiIs0 | (8-(cc & 8));
}

static int cc_to_biint(int cc, int flags_reg) {
  return (flags_reg << 2) | (cc & 3);
}

//=============================================================================

// Compute padding required for nodes which need alignment. The padding
// is the number of bytes (not instructions) which will be inserted before
// the instruction. The padding must match the size of a NOP instruction.

int string_indexOf_imm1_charNode::compute_padding(int current_offset) const {
  return (3*4-current_offset)&31;
}

int string_indexOf_imm1Node::compute_padding(int current_offset) const {
  return (2*4-current_offset)&31;
}

int string_indexOf_immNode::compute_padding(int current_offset) const {
  return (3*4-current_offset)&31;
}

int string_indexOfNode::compute_padding(int current_offset) const {
  return (1*4-current_offset)&31;
}

int string_compareNode::compute_padding(int current_offset) const {
  return (4*4-current_offset)&31;
}

int string_equals_immNode::compute_padding(int current_offset) const {
  if (opnd_array(3)->constant() < 16) return 0; // Don't insert nops for short version (loop completely unrolled).
  return (2*4-current_offset)&31;
}

int string_equalsNode::compute_padding(int current_offset) const {
  return (7*4-current_offset)&31;
}

int inlineCallClearArrayNode::compute_padding(int current_offset) const {
  return (2*4-current_offset)&31;
}

//=============================================================================

// Indicate if the safepoint node needs the polling page as an input.
bool SafePointNode::needs_polling_address_input() {
  // The address is loaded from thread by a seperate node.
  return true;
}

//=============================================================================

// Emit an interrupt that is caught by the debugger (for debugging compiler).
void emit_break(CodeBuffer &cbuf) {
  MacroAssembler _masm(&cbuf);
  __ illtrap();
}

#ifndef PRODUCT
void MachBreakpointNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  st->print("BREAKPOINT");
}
#endif

void MachBreakpointNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  emit_break(cbuf);
}

uint MachBreakpointNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}

//=============================================================================

void emit_nop(CodeBuffer &cbuf) {
  MacroAssembler _masm(&cbuf);
  __ nop();
}

static inline void emit_long(CodeBuffer &cbuf, int value) {
  *((int*)(cbuf.insts_end())) = value;
  cbuf.set_insts_end(cbuf.insts_end() + BytesPerInstWord);
}

//=============================================================================

// Emit a java_to_interp stub.
//
// The stub is initially created with a -1 target and a NULL oop, the stub
// is fixed up when the corresponding call is converted from calling compiled
// code to calling interpreted code.

// Offset from start of compiled java to interpreter stub to the load
// constant that loads the inline cache (IC) (8 if we have to load
// toc).
const int CompiledStaticCall::comp_to_int_load_offset = 8;

const uint java_to_interp_stub_size = 12 * BytesPerInstWord;

void emit_java_to_interp_stub(MacroAssembler &_masm, const int insts_relocation_offset) {
  // Start the stub.
  address stub = __ start_a_stub(java_to_interp_stub_size);
  if (stub == NULL) {
    Compile::current()->env()->record_out_of_memory_failure();
    return;
  }

  // For java_to_interp stubs we use R11_scratch1 as scratch register
  // and in call trampoline stubs we use R12_scratch2. This way we
  // can distinguish them (see is_NativeCallTrampolineStub_at()).
  Register reg_scratch = R11_scratch1;

  // Create a static stub relocation which relates this stub
  // with the call instruction at insts_call_instruction_offset in the
  // instructions code-section.
  __ relocate(static_stub_Relocation::spec(__ code()->insts()->start() + insts_relocation_offset));
  const int stub_start_offset = __ offset();

  // Now, create the stub's code:
  // - load the TOC
  // - load the inline cache oop from the constant pool
  // - load the call target from the constant pool
  // - call
  __ calculate_address_from_global_toc(reg_scratch, __ method_toc());
  AddressLiteral oop = __ allocate_oop_address(NULL);
  __ load_const_from_method_toc(reg_to_register_object(Matcher::inline_cache_reg_encode()), oop, reg_scratch);

  if (ReoptimizeCallSequences) {
    __ b64_patchable((address)-1, relocInfo::none);
  } else {
    AddressLiteral a((address)-1);
    __ load_const_from_method_toc(reg_scratch, a, reg_scratch);
    __ mtctr(reg_scratch);
    __ bctr();
  }

  // FIXME: Assert that the stub can be identified and patched.

  // Java_to_interp_stub_size should be good.
  assert((uint)(__ offset() - stub_start_offset) <= java_to_interp_stub_size, "should be good size");
  assert(!is_NativeCallTrampolineStub_at(__ addr_at(stub_start_offset)),
         "must not confuse java_to_interp with trampoline stubs");

 // End the stub.
  __ end_a_stub();
}

// Size of java_to_interp stub, this doesn't need to be accurate but it must
// be larger or equal to the real size of the stub.
// Used for optimization in Compile::Shorten_branches.
uint size_java_to_interp() {
  return java_to_interp_stub_size;
}

// Number of relocation entries needed by compiled java to interpreter
// call stub.
// Used for optimization in Compile::Shorten_branches.
uint reloc_java_to_interp() {
  return 5;
}

//=============================================================================

// Emit a trampoline stub for a call to a target which is too far away.
//
// code sequences:
//
// call-site:
//   branch-and-link to <destination> or <trampoline stub>
//
// Related trampoline stub for this call-site in the stub section:
//   load the call target from the constant pool
//   branch via CTR (LR/link still points to the call-site above)

const uint trampoline_stub_size = 6 * BytesPerInstWord;

void emit_trampoline_stub(MacroAssembler &_masm, int destination_toc_offset, int insts_call_instruction_offset) {
  // Start the stub.
  address stub = __ start_a_stub(Compile::MAX_stubs_size/2);
  if (stub == NULL) {
    Compile::current()->env()->record_out_of_memory_failure();
    return;
  }

  // For java_to_interp stubs we use R11_scratch1 as scratch register
  // and in call trampoline stubs we use R12_scratch2. This way we
  // can distinguish them (see is_NativeCallTrampolineStub_at()).
  Register reg_scratch = R12_scratch2;

  // Create a trampoline stub relocation which relates this trampoline stub
  // with the call instruction at insts_call_instruction_offset in the
  // instructions code-section.
  __ relocate(trampoline_stub_Relocation::spec(__ code()->insts()->start() + insts_call_instruction_offset));
  const int stub_start_offset = __ offset();

  // Now, create the trampoline stub's code:
  // - load the TOC
  // - load the call target from the constant pool
  // - call
  __ calculate_address_from_global_toc(reg_scratch, __ method_toc());
  __ ld_largeoffset_unchecked(reg_scratch, destination_toc_offset, reg_scratch, false);
  __ mtctr(reg_scratch);
  __ bctr();

  const address stub_start_addr = __ addr_at(stub_start_offset);

  // FIXME: Assert that the trampoline stub can be identified and patched.

  // Assert that the encoded destination_toc_offset can be identified and that it is correct.
  assert(destination_toc_offset == NativeCallTrampolineStub_at(stub_start_addr)->destination_toc_offset(),
         "encoded offset into the constant pool must match");
  // Trampoline_stub_size should be good.
  assert((uint)(__ offset() - stub_start_offset) <= trampoline_stub_size, "should be good size");
  assert(is_NativeCallTrampolineStub_at(stub_start_addr), "doesn't look like a trampoline");

  // End the stub.
  __ end_a_stub();
}

// Size of trampoline stub, this doesn't need to be accurate but it must
// be larger or equal to the real size of the stub.
// Used for optimization in Compile::Shorten_branches.
uint size_call_trampoline() {
  return trampoline_stub_size;
}

// Number of relocation entries needed by trampoline stub.
// Used for optimization in Compile::Shorten_branches.
uint reloc_call_trampoline() {
  return 5;
}

//=============================================================================

// Emit an inline branch-and-link call and a related trampoline stub.
//
// code sequences:
//
// call-site:
//   branch-and-link to <destination> or <trampoline stub>
//
// Related trampoline stub for this call-site in the stub section:
//   load the call target from the constant pool
//   branch via CTR (LR/link still points to the call-site above)
//

typedef struct {
  int insts_call_instruction_offset;
  int ret_addr_offset;
} EmitCallOffsets;

// Emit a branch-and-link instruction that branches to a trampoline.
// - Remember the offset of the branch-and-link instruction.
// - Add a relocation at the branch-and-link instruction.
// - Emit a branch-and-link.
// - Remember the return pc offset.
EmitCallOffsets emit_call_with_trampoline_stub(MacroAssembler &_masm, address entry_point, relocInfo::relocType rtype) {
  EmitCallOffsets offsets = { -1, -1 };
  const int start_offset = __ offset();
  offsets.insts_call_instruction_offset = __ offset();

  // No entry point given, use the current pc.
  if (entry_point == NULL) entry_point = __ pc();

  if (!Compile::current()->in_scratch_emit_size()) {
    // Put the entry point as a constant into the constant pool.
    const address entry_point_toc_addr   = __ address_constant(entry_point, RelocationHolder::none);
    const int     entry_point_toc_offset = __ offset_to_method_toc(entry_point_toc_addr);

    // Emit the trampoline stub which will be related to the branch-and-link below.
    emit_trampoline_stub(_masm, entry_point_toc_offset, offsets.insts_call_instruction_offset);
    __ relocate(rtype);
  }

  // Note: At this point we do not have the address of the trampoline
  // stub, and the entry point might be too far away for bl, so __ pc()
  // serves as dummy and the bl will be patched later.
  __ bl((address) __ pc());

  offsets.ret_addr_offset = __ offset() - start_offset;

  return offsets;
}

//=============================================================================

// Factory for creating loadConL* nodes for large/small constant pool.

static inline jlong replicate_immF(float con) {
  // Replicate float con 2 times and pack into vector.
  int val = *((int*)&con);
  jlong lval = val;
  lval = (lval << 32) | (lval & 0xFFFFFFFFl);
  return lval;
}

typedef struct {
  loadConL_hiNode *_large_hi;
  loadConL_loNode *_large_lo;
  loadConLNode    *_small;
  MachNode        *_last;
} loadConLNodesTuple;

loadConLNodesTuple loadConLNodesTuple_create(Compile *C, PhaseRegAlloc *ra_, Node *toc, immLOper *immSrc,
                                             bool isoop, OptoReg::Name reg_second, OptoReg::Name reg_first) {
  loadConLNodesTuple nodes;
  MachOper *immIsOop = new (C) immIOper(isoop ? 1 : 0);

  const bool large_constant_pool = true; // TODO: PPC port C->cfg()->_consts_size > 4000;
  if (large_constant_pool) {
    // Create new nodes.
    loadConL_hiNode *m1 = new (C) loadConL_hiNode();
    loadConL_loNode *m2 = new (C) loadConL_loNode();

    // inputs for new nodes
    m1->add_req(NULL, toc);
    m2->add_req(NULL, m1);

    // operands for new nodes
    m1->_opnds[0] = new (C) iRegLdstOper(); // dst
    m1->_opnds[1] = immSrc;                 // src
    m1->_opnds[2] = new (C) iRegPdstOper(); // toc
    m1->_opnds[3] = immIsOop;               // isoop
    m2->_opnds[0] = new (C) iRegLdstOper(); // dst
    m2->_opnds[1] = immSrc;                 // src
    m2->_opnds[2] = new (C) iRegLdstOper(); // base
    m2->_opnds[3] = immIsOop;               // isoop

    // Initialize ins_attrib TOC fields.
    m1->_const_toc_offset = -1;
    m2->_const_toc_offset_hi_node = m1;

    // Initialize ins_attrib instruction offset.
    m1->_cbuf_insts_offset = -1;

    // register allocation for new nodes
    ra_->set_pair(m1->_idx, reg_second, reg_first);
    ra_->set_pair(m2->_idx, reg_second, reg_first);

    // Create result.
    nodes._large_hi = m1;
    nodes._large_lo = m2;
    nodes._small = NULL;
    nodes._last = nodes._large_lo;
    assert(m2->bottom_type()->isa_long(), "must be long");
  } else {
    loadConLNode *m2 = new (C) loadConLNode();

    // inputs for new nodes
    m2->add_req(NULL, toc);

    // operands for new nodes
    m2->_opnds[0] = new (C) iRegLdstOper(); // dst
    m2->_opnds[1] = immSrc;                 // src
    m2->_opnds[2] = new (C) iRegPdstOper(); // toc
    m2->_opnds[3] = immIsOop;               // isoop

    // Initialize ins_attrib instruction offset.
    m2->_cbuf_insts_offset = -1;

    // register allocation for new nodes
    ra_->set_pair(m2->_idx, reg_second, reg_first);

    // Create result.
    nodes._large_hi = NULL;
    nodes._large_lo = NULL;
    nodes._small = m2;
    nodes._last = nodes._small;
    assert(m2->bottom_type()->isa_long(), "must be long");
  }

  return nodes;
}

//=============================================================================

// Factory for creating loadConP* nodes for large/small constant pool.

typedef struct {
  loadConP_hiNode *_large_hi;
  loadConP_loNode *_large_lo;
  loadConPNode    *_small;
  MachNode          *_last;
} loadConPNodesTuple;

loadConPNodesTuple loadConPNodesTuple_create(Compile *C, PhaseRegAlloc *ra_, Node *toc, immPOper *immSrc,
                                             OptoReg::Name reg_second, OptoReg::Name reg_first) {
  loadConPNodesTuple nodes;
  const bool isoop = immSrc->constant_is_oop();
  MachOper *immIsOop = new (C) immIOper(isoop ? 1 : 0);

  const bool large_constant_pool = true; // TODO: PPC port C->cfg()->_consts_size > 4000;
  if (large_constant_pool) {
    // Create new nodes.
    loadConP_hiNode *m1 = new (C) loadConP_hiNode();
    loadConP_loNode *m2 = new (C) loadConP_loNode();

    // inputs for new nodes
    m1->add_req(NULL, toc);
    m2->add_req(NULL, m1);

    // operands for new nodes
    m1->_opnds[0] = new (C) iRegPdstOper(); // dst
    m1->_opnds[1] = immSrc;                 // src
    m1->_opnds[2] = new (C) iRegPdstOper(); // toc
    m1->_opnds[3] = immIsOop;               // isoop
    m2->_opnds[0] = new (C) iRegPdstOper(); // dst
    m2->_opnds[1] = immSrc;                 // src
    m2->_opnds[2] = new (C) iRegLdstOper(); // base
    m2->_opnds[3] = immIsOop;               // isoop

    // Initialize ins_attrib TOC fields.
    m1->_const_toc_offset = -1;
    m2->_const_toc_offset_hi_node = m1;

    // register allocation for new nodes
    ra_->set_pair(m1->_idx, reg_second, reg_first);
    ra_->set_pair(m2->_idx, reg_second, reg_first);

    // Create result.
    nodes._large_hi = m1;
    nodes._large_lo = m2;
    nodes._small = NULL;
    nodes._last = nodes._large_lo;
    assert(m2->bottom_type()->isa_ptr(), "must be ptr");
  } else {
    loadConPNode *m2 = new (C) loadConPNode();

    // inputs for new nodes
    m2->add_req(NULL, toc);

    // operands for new nodes
    m2->_opnds[0] = new (C) iRegPdstOper(); // dst
    m2->_opnds[1] = immSrc;                 // src
    m2->_opnds[2] = new (C) iRegPdstOper(); // toc
    m2->_opnds[3] = immIsOop;               // isoop

    // register allocation for new nodes
    ra_->set_pair(m2->_idx, reg_second, reg_first);

    nodes._large_hi = NULL;
    nodes._large_lo = NULL;
    nodes._small = m2;
    nodes._last = nodes._small;
    assert(m2->bottom_type()->isa_ptr(), "must be ptr");
  }

  return nodes;
}

//=============================================================================

const RegMask& MachConstantBaseNode::_out_RegMask = BITS64_REG_RW_mask();
int Compile::ConstantTable::calculate_table_base_offset() const {
  return 0;  // absolute addressing, no offset
}

bool MachConstantBaseNode::requires_late_expand() const { return true; }
void MachConstantBaseNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  Compile *C = ra_->C;

  iRegPdstOper *op_dst = new (C) iRegPdstOper();
  MachNode *m1 = new (C) loadToc_hiNode();
  MachNode *m2 = new (C) loadToc_loNode();

  m1->add_req(NULL);
  m2->add_req(NULL, m1);
  m1->_opnds[0] = op_dst;
  m2->_opnds[0] = op_dst;
  m2->_opnds[1] = op_dst;
  ra_->set_pair(m1->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
  ra_->set_pair(m2->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
  nodes->push(m1);
  nodes->push(m2);
}

void MachConstantBaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  // Is late expanded.
  ShouldNotReachHere();
}

uint MachConstantBaseNode::size(PhaseRegAlloc* ra_) const {
  return 0;
}

#ifndef PRODUCT
void MachConstantBaseNode::format(PhaseRegAlloc* ra_, outputStream* st) const {
  st->print("-- \t// MachConstantBaseNode (empty encoding)");
}
#endif

//=============================================================================

#ifndef PRODUCT
void MachPrologNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  Compile* C = ra_->C;
  const long framesize = C->frame_slots() << LogBytesPerInt;

  st->print("PROLOG\n\t");
  if (C->need_stack_bang(framesize)) {
    st->print("stack_overflow_check\n\t");
  }

  if (!false /* TODO: PPC port C->is_frameless_method()*/) {
    st->print("save return pc\n\t");
    st->print("push frame %ld\n\t", -framesize);
  }
}
#endif

// Macro used instead of the common __ to emulate the pipes of PPC.
// Instead of e.g. __ ld(...) one hase to write ___(ld) ld(...) This enables the
// micro scheduler to cope with "hand written" assembler like in the prolog. Though
// still no scheduling of this code is possible, the micro scheduler is aware of the
// code and can update its internal data. The following mechanism is used to achieve this:
// The micro scheduler calls size() of each compound node during scheduling. size() does a
// dummy emit and only during this dummy emit C->hb_scheduling() is not NULL.
#if 0 // TODO: PPC port
#define ___(op) if (UsePower6SchedulerPPC64 && C->hb_scheduling())                    \
                  C->hb_scheduling()->_pdScheduling->PdEmulatePipe(ppc64Opcode_##op); \
                _masm.
#define ___stop if (UsePower6SchedulerPPC64 && C->hb_scheduling())                    \
                  C->hb_scheduling()->_pdScheduling->PdEmulatePipe(archOpcode_none)
#define ___advance if (UsePower6SchedulerPPC64 && C->hb_scheduling())                 \
                  C->hb_scheduling()->_pdScheduling->advance_offset
#else
#define ___(op) if (UsePower6SchedulerPPC64)                                          \
                  Unimplemented();                                                    \
                _masm.
#define ___stop if (UsePower6SchedulerPPC64)                                          \
                  Unimplemented()
#define ___advance if (UsePower6SchedulerPPC64)                                       \
                  Unimplemented()
#endif

void MachPrologNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile* C = ra_->C;
  MacroAssembler _masm(&cbuf);

  const long framesize = ((long)C->frame_slots()) << LogBytesPerInt;
  assert(framesize%(2*wordSize) == 0, "must preserve 2*wordSize alignment");

  const bool method_is_frameless      = false /* TODO: PPC port C->is_frameless_method()*/;

  const Register return_pc            = R20; // Must match return_addr() in frame section.
  const Register callers_sp           = R21;
  const Register push_frame_temp      = R22;
  const Register toc_temp             = R23;
  assert_different_registers(R11, return_pc, callers_sp, push_frame_temp, toc_temp);

  if (method_is_frameless) {
    // Add nop at beginning of all frameless methods to prevent any
    // oop instructions from getting overwritten by make_not_entrant
    // (patching attempt would fail).
    ___(nop) nop();
  } else {
    // Get return pc.
    ___(mflr) mflr(return_pc);
  }

  // Calls to C2R adapters often do not accept exceptional returns.
  // We require that their callers must bang for them. But be
  // careful, because some VM calls (such as call site linkage) can
  // use several kilobytes of stack. But the stack safety zone should
  // account for that. See bugs 4446381, 4468289, 4497237.
  if (C->need_stack_bang(framesize) && UseStackBanging) {
    // Unfortunately we cannot use the function provided in
    // assembler.cpp as we have to emulate the pipes. So I had to
    // insert the code of generate_stack_overflow_check(), see
    // assembler.cpp for some illuminative comments.
    const int page_size = os::vm_page_size();
    int bang_end = StackShadowPages*page_size;

    // This is how far the previous frame's stack banging extended.
    const int bang_end_safe = bang_end;

    if (framesize > page_size) {
      bang_end += framesize;
    }

    int bang_offset = bang_end_safe;

    while (bang_offset <= bang_end) {
      // Need at least one stack bang at end of shadow zone.

      // Again I had to copy code, this time from assembler_ppc64.cpp,
      // bang_stack_with_offset - see there for comments.

      // Stack grows down, caller passes positive offset.
      assert(bang_offset > 0, "must bang with positive offset");

      long stdoffset = -bang_offset;

      if (Assembler::is_simm(stdoffset, 16)) {
        // Signed 16 bit offset, a simple std is ok.
        if (UseLoadInstructionsForStackBangingPPC64) {
          ___(ld) ld(R0,  (int)(signed short)stdoffset, R1_SP);
        } else {
          ___(std) std(R0, (int)(signed short)stdoffset, R1_SP);
        }
      } else if (Assembler::is_simm(stdoffset, 31)) {
        // Use largeoffset calculations for addis & ld/std.
        const int hi = MacroAssembler::largeoffset_si16_si16_hi(stdoffset);
        const int lo = MacroAssembler::largeoffset_si16_si16_lo(stdoffset);

        Register tmp = R11;
        ___(addis) addis(tmp, R1_SP, hi);
        if (UseLoadInstructionsForStackBangingPPC64) {
          ___(ld) ld(R0, lo, tmp);
        } else {
          ___(std) std(R0, lo, tmp);
        }
      } else {
        ShouldNotReachHere();
      }

      bang_offset += page_size;
    }
    // R11 trashed
  } // C->need_stack_bang(framesize) && UseStackBanging

  unsigned int bytes = (unsigned int)framesize;
  long offset = Assembler::align_addr(bytes, frame::alignment_in_bytes);
  ciMethod *currMethod = C -> method();

  // Optimized version for most common case.
  if (UsePower6SchedulerPPC64 &&
      !method_is_frameless && Assembler::is_simm((int)(-(_abi(lr) + offset)), 16) &&
      !(false /* ConstantsALot TODO: PPC port*/)) {
    ___(or) mr(callers_sp, R1_SP);
    ___(addi) addi(R1_SP, R1_SP, -offset);
    ___stop; // Emulator won't recognize dependency.
    ___(std) std(return_pc, _abi(lr) + offset, R1_SP);
    ___(std) std(callers_sp, 0, R1_SP);
    return;
  }

  if (!method_is_frameless) {
    // Get callers sp.
    ___(or) mr(callers_sp, R1_SP);

    // Push method's frame, modifies SP.
    assert(Assembler::is_uimm(framesize, 32U), "wrong type");
    // The ABI is already accounted for in 'framesize' via the
    // 'out_preserve' area.
    Register tmp = push_frame_temp;
    // Had to insert code of push_frame((unsigned int)framesize, push_frame_temp).
    if (Assembler::is_simm(-offset, 16)) {
      ___(stdu) stdu(R1_SP, -offset, R1_SP);
    } else {
      long x = -offset;
      // Had to insert load_const(tmp, -offset).
      ___(addis)  lis( tmp, (int)((signed short)(((x >> 32) & 0xffff0000) >> 16)));
      ___(ori)    ori( tmp, tmp, ((x >> 32) & 0x0000ffff));
      ___(rldicr) sldi(tmp, tmp, 32);
      ___(oris)   oris(tmp, tmp, (x & 0xffff0000) >> 16);
      ___(ori)    ori( tmp, tmp, (x & 0x0000ffff));

      ___(stdux) stdux(R1_SP, R1_SP, tmp);
    }
  }
#if 0 // TODO: PPC port
  // For testing large constant pools, emit a lot of constants to constant pool.
  // "Randomize" const_size.
  if (ConstantsALot) {
    const int num_consts = const_size();
    for (int i = 0; i < num_consts; i++) {
      __ long_constant(0xB0B5B00BBABE);
    }
  }
#endif
  if (!method_is_frameless) {
    // Save return pc.
    ___(std) std(return_pc, _abi(lr), callers_sp);
  }
}
#undef ___
#undef ___stop

uint MachPrologNode::size(PhaseRegAlloc *ra_) const {
  // Variable size. determine dynamically.
  return MachNode::size(ra_);
}

int MachPrologNode::reloc() const {
  // Return number of relocatable values contained in this instruction.
  return 1; // 1 reloc entry for load_const(toc).
}

//=============================================================================

#ifndef PRODUCT
void MachEpilogNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  Compile* C = ra_->C;

  st->print("EPILOG\n\t");
  st->print("restore return pc\n\t");
  st->print("pop frame\n\t");

  if (do_polling() && C->is_method_compilation()) {
    st->print("touch polling page\n\t");
  }
}
#endif

void MachEpilogNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile* C = ra_->C;
  MacroAssembler _masm(&cbuf);

  const long framesize = ((long)C->frame_slots()) << LogBytesPerInt;
  assert(framesize >= 0, "negative frame-size?");

  const bool method_needs_polling = do_polling() && C->is_method_compilation();
  const bool method_is_frameless  = false /* TODO: PPC port C->is_frameless_method()*/;
  const Register return_pc        = R11;
  const Register polling_page     = R12;

  if (!method_is_frameless) {
    // Restore return pc relative to callers' sp.
    __ ld(return_pc, ((int)framesize) + _abi(lr), R1_SP);
  }

  if (method_needs_polling) {
    if (LoadPollAddressFromThread) {
      // TODO: PPC port __ ld(polling_page, in_bytes(JavaThread::poll_address_offset()), R16_thread);
      Unimplemented();
    } else {
      __ load_const_optimized(polling_page, (long)(address) os::get_polling_page()); // TODO: PPC port: get_standard_polling_page()
    }
  }

  if (!method_is_frameless) {
    // Move return pc to LR.
    __ mtlr(return_pc);
    // Pop frame (fixed frame-size).
    __ addi(R1_SP, R1_SP, (int)framesize);
  }

  if (method_needs_polling) {
    // We need to mark the code position where the load from the safepoint
    // polling page was emitted as relocInfo::poll_return_type here.
    __ relocate(relocInfo::poll_return_type);
    __ load_from_polling_page(polling_page);
  }
}

uint MachEpilogNode::size(PhaseRegAlloc *ra_) const {
  // Variable size. Determine dynamically.
  return MachNode::size(ra_);
}

int MachEpilogNode::reloc() const {
  // Return number of relocatable values contained in this instruction.
  return 1; // 1 for load_from_polling_page.
}

const Pipeline * MachEpilogNode::pipeline() const {
  return MachNode::pipeline_class();
}

// This method seems to be obsolete. It is declared in machnode.hpp
// and defined in all *.ad files, but it is never called. Should we
// get rid of it?
int MachEpilogNode::safepoint_offset() const {
  assert(do_polling(), "no return for this epilog node");
  return 0;
}

//=============================================================================

#if 0 // TODO: PPC port
// SIGTRAP-based null checks (on AIX).
#ifdef AIX
void MachNullCheckNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  // Emits entries in the null-pointer exception handler table ...
  assert(ImplicitNullChecks, "precondition");
  assert(MachNullCheckNode::uses_checked_address_as_required_input(), "precondition");

  if (!memory_op_does_trap_hint()) {
    // ... and a null-check trap if the memory op is not guaranteed to be a store.
    MacroAssembler _masm(&cbuf);
    // Trap-instruction-based null checks.
    int reg = ra_->get_reg_first(in(MachNullCheckNode::checked_address_in_index()));
    _masm.trap_null_check(as_Register(Matcher::_regEncode[reg]));
  }
}

uint MachNullCheckNode::size(PhaseRegAlloc *ra_) const {
  if (!memory_op_does_trap_hint()) {
    // We emit a trap instruction, 4 bytes.
    return 4;
  } else {
    // The store will handle the implicit null check, nothing here.
    return 0;
  }
}
#endif

void MachLoadPollAddrLateNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  MacroAssembler _masm(&cbuf);
  if (LoadPollAddressFromThread) {
    _masm.ld(R11, in_bytes(JavaThread::poll_address_offset()), R16_thread);
  } else {
    _masm.nop();
  }
}

uint MachLoadPollAddrLateNode::size(PhaseRegAlloc* ra_) const {
  if (LoadPollAddressFromThread) {
    return 4;
  } else {
    return 4;
  }
}

#ifndef PRODUCT
void MachLoadPollAddrLateNode::format(PhaseRegAlloc* ra_, outputStream* st) const {
  st->print_cr(" LD R11, PollAddressOffset, R16_thread \t// LoadPollAddressFromThread");
}
#endif

const RegMask &MachLoadPollAddrLateNode::out_RegMask() const {
  return RSCRATCH1_BITS64_REG_mask();
}
#endif // PPC port

// =============================================================================

// Figure out which register class each belongs in: rc_int, rc_float or
// rc_stack.
enum RC { rc_bad, rc_int, rc_float, rc_stack };

static enum RC rc_class(OptoReg::Name reg) {
  // Return the register class for the given register. The given register
  // reg is a <register>_num value, which is an index into the MachRegisterNumbers
  // enumeration in adGlobals_ppc64.hpp.

  if (reg == OptoReg::Bad) return rc_bad;

  // We have 64 integer register halves, starting at index 0.
  if (reg < 64) return rc_int;

  // We have 64 floating-point register halves, starting at index 64.
  if (reg < 64+64) return rc_float;

  // Between float regs & stack are the flags regs.
  assert(OptoReg::is_stack(reg), "blow up if spilling flags");

  return rc_stack;
}

static int ld_st_helper(CodeBuffer *cbuf, const char *op_str, uint opcode, int reg, int offset,
                            bool do_print, Compile* C, outputStream *st) {

  assert(opcode == Assembler::LD_OPCODE   ||
         opcode == Assembler::STD_OPCODE  ||
         opcode == Assembler::LWZ_OPCODE  ||
         opcode == Assembler::STW_OPCODE  ||
         opcode == Assembler::LFD_OPCODE  ||
         opcode == Assembler::STFD_OPCODE ||
         opcode == Assembler::LFS_OPCODE  ||
         opcode == Assembler::STFS_OPCODE,
         "opcode not supported");

  if (cbuf) {
    int d =
      (Assembler::LD_OPCODE == opcode || Assembler::STD_OPCODE == opcode) ?
        Assembler::ds(offset+0 /* TODO: PPC port C->frame_slots_sp_bias_in_bytes()*/)
      : Assembler::d1(offset+0 /* TODO: PPC port C->frame_slots_sp_bias_in_bytes()*/); // Makes no difference in opt build.
    emit_long(*cbuf, opcode | Assembler::rt(Matcher::_regEncode[reg]) | d | Assembler::ra(R1_SP));
  }
#ifndef PRODUCT
  else if (do_print) {
    st->print("%-7s %s, [R1_SP + #%d+%d] \t// spill copy",
              op_str,
              Matcher::regName[reg],
              offset, 0 /* TODO: PPC port C->frame_slots_sp_bias_in_bytes()*/);
  }
#endif
  return 4; // size
}

uint MachSpillCopyNode::implementation(CodeBuffer *cbuf, PhaseRegAlloc *ra_, bool do_size, outputStream *st) const {
  Compile* C = ra_->C;

  // Get registers to move.
  OptoReg::Name src_hi = ra_->get_reg_second(in(1));
  OptoReg::Name src_lo = ra_->get_reg_first(in(1));
  OptoReg::Name dst_hi = ra_->get_reg_second(this);
  OptoReg::Name dst_lo = ra_->get_reg_first(this);

  enum RC src_hi_rc = rc_class(src_hi);
  enum RC src_lo_rc = rc_class(src_lo);
  enum RC dst_hi_rc = rc_class(dst_hi);
  enum RC dst_lo_rc = rc_class(dst_lo);

  assert(src_lo != OptoReg::Bad && dst_lo != OptoReg::Bad, "must move at least 1 register");
  if (src_hi != OptoReg::Bad)
    assert((src_lo&1)==0 && src_lo+1==src_hi &&
           (dst_lo&1)==0 && dst_lo+1==dst_hi,
           "expected aligned-adjacent pairs");
  // Generate spill code!
  int size = 0;

  if (src_lo == dst_lo && src_hi == dst_hi)
    return size;            // Self copy, no move.

  // --------------------------------------
  // Memory->Memory Spill. Use R0 to hold the value.
  if (src_lo_rc == rc_stack && dst_lo_rc == rc_stack) {
    int src_offset = ra_->reg2offset(src_lo);
    int dst_offset = ra_->reg2offset(dst_lo);
    if (src_hi != OptoReg::Bad) {
      assert(src_hi_rc==rc_stack && dst_hi_rc==rc_stack,
             "expected same type of move for high parts");
      size += ld_st_helper(cbuf, "LD  ", Assembler::LD_OPCODE,  R0_num, src_offset, !do_size, C, st);
      if (!cbuf && !do_size) st->print("\n\t");
      size += ld_st_helper(cbuf, "STD ", Assembler::STD_OPCODE, R0_num, dst_offset, !do_size, C, st);
    } else {
      size += ld_st_helper(cbuf, "LWZ ", Assembler::LWZ_OPCODE, R0_num, src_offset, !do_size, C, st);
      if (!cbuf && !do_size) st->print("\n\t");
      size += ld_st_helper(cbuf, "STW ", Assembler::STW_OPCODE, R0_num, dst_offset, !do_size, C, st);
    }
    return size;
  }

  // --------------------------------------
  // Check for float->int copy; requires a trip through memory.
  if (src_lo_rc == rc_float && dst_lo_rc == rc_int) {
    Unimplemented();
  }

  // --------------------------------------
  // Check for integer reg-reg copy.
  if (src_lo_rc == rc_int && dst_lo_rc == rc_int) {
      Register Rsrc = as_Register(Matcher::_regEncode[src_lo]);
      Register Rdst = as_Register(Matcher::_regEncode[dst_lo]);
      size = (Rsrc != Rdst) ? 4 : 0;

      if (cbuf) {
        MacroAssembler _masm(cbuf);
        if (size) {
          __ mr(Rdst, Rsrc);
        }
      }
#ifndef PRODUCT
      else if (!do_size) {
        if (size) {
          st->print("%-7s %s, %s \t// spill copy", "MR", Matcher::regName[dst_lo], Matcher::regName[src_lo]);
        } else {
          st->print("%-7s %s, %s \t// spill copy", "MR-NOP", Matcher::regName[dst_lo], Matcher::regName[src_lo]);
        }
      }
#endif
      return size;
  }

  // Check for integer store.
  if (src_lo_rc == rc_int && dst_lo_rc == rc_stack) {
    int dst_offset = ra_->reg2offset(dst_lo);
    if (src_hi != OptoReg::Bad) {
      assert(src_hi_rc==rc_int && dst_hi_rc==rc_stack,
             "expected same type of move for high parts");
      size += ld_st_helper(cbuf, "STD ", Assembler::STD_OPCODE, src_lo, dst_offset, !do_size, C, st);
    } else {
      size += ld_st_helper(cbuf, "STW ", Assembler::STW_OPCODE, src_lo, dst_offset, !do_size, C, st);
    }
    return size;
  }

  // Check for integer load.
  if (dst_lo_rc == rc_int && src_lo_rc == rc_stack) {
    int src_offset = ra_->reg2offset(src_lo);
    if (src_hi != OptoReg::Bad) {
      assert(dst_hi_rc==rc_int && src_hi_rc==rc_stack,
             "expected same type of move for high parts");
      size += ld_st_helper(cbuf, "LD  ", Assembler::LD_OPCODE, dst_lo, src_offset, !do_size, C, st);
    } else {
      size += ld_st_helper(cbuf, "LWZ ", Assembler::LWZ_OPCODE, dst_lo, src_offset, !do_size, C, st);
    }
    return size;
  }

  // Check for float reg-reg copy.
  if (src_lo_rc == rc_float && dst_lo_rc == rc_float) {
    if (cbuf) {
      MacroAssembler _masm(cbuf);
      FloatRegister Rsrc = as_FloatRegister(Matcher::_regEncode[src_lo]);
      FloatRegister Rdst = as_FloatRegister(Matcher::_regEncode[dst_lo]);
      __ fmr(Rdst, Rsrc);
    }
#ifndef PRODUCT
    else if (!do_size) {
      st->print("%-7s %s, %s \t// spill copy", "FMR", Matcher::regName[dst_lo], Matcher::regName[src_lo]);
    }
#endif
    return 4;
  }

  // Check for float store.
  if (src_lo_rc == rc_float && dst_lo_rc == rc_stack) {
    int dst_offset = ra_->reg2offset(dst_lo);
    if (src_hi != OptoReg::Bad) {
      assert(src_hi_rc==rc_float && dst_hi_rc==rc_stack,
             "expected same type of move for high parts");
      size += ld_st_helper(cbuf, "STFD", Assembler::STFD_OPCODE, src_lo, dst_offset, !do_size, C, st);
    } else {
      size += ld_st_helper(cbuf, "STFS", Assembler::STFS_OPCODE, src_lo, dst_offset, !do_size, C, st);
    }
    return size;
  }

  // Check for float load.
  if (dst_lo_rc == rc_float && src_lo_rc == rc_stack) {
    int src_offset = ra_->reg2offset(src_lo);
    if (src_hi != OptoReg::Bad) {
      assert(dst_hi_rc==rc_float && src_hi_rc==rc_stack,
             "expected same type of move for high parts");
      size += ld_st_helper(cbuf, "LFD ", Assembler::LFD_OPCODE, dst_lo, src_offset, !do_size, C, st);
    } else {
      size += ld_st_helper(cbuf, "LFS ", Assembler::LFS_OPCODE, dst_lo, src_offset, !do_size, C, st);
    }
    return size;
  }

  // --------------------------------------------------------------------
  // Check for hi bits still needing moving. Only happens for misaligned
  // arguments to native calls.
  if (src_hi == dst_hi)
    return size;               // Self copy; no move.

  assert(src_hi_rc != rc_bad && dst_hi_rc != rc_bad, "src_hi & dst_hi cannot be Bad");
  ShouldNotReachHere(); // Unimplemented
  return 0;
}

#ifndef PRODUCT
void MachSpillCopyNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  if (!ra_)
    st->print("N%d = SpillCopy(N%d)", _idx, in(1)->_idx);
  else
    implementation(NULL, ra_, false, st);
}
#endif

void MachSpillCopyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  implementation(&cbuf, ra_, false, NULL);
}

uint MachSpillCopyNode::size(PhaseRegAlloc *ra_) const {
  return implementation(NULL, ra_, true, NULL);
}

#if 0 // TODO: PPC port
ArchOpcode MachSpillCopyNode_archOpcode(MachSpillCopyNode *n, PhaseRegAlloc *ra_) {
#ifndef PRODUCT
  if (ra_->node_regs_max_index() == 0) return archOpcode_undefined;
#endif
  assert(ra_->node_regs_max_index() != 0, "");

  // Get registers to move.
  OptoReg::Name src_hi = ra_->get_reg_second(n->in(1));
  OptoReg::Name src_lo = ra_->get_reg_first(n->in(1));
  OptoReg::Name dst_hi = ra_->get_reg_second(n);
  OptoReg::Name dst_lo = ra_->get_reg_first(n);

  enum RC src_lo_rc = rc_class(src_lo);
  enum RC dst_lo_rc = rc_class(dst_lo);

  if (src_lo == dst_lo && src_hi == dst_hi)
    return ppc64Opcode_none;            // Self copy, no move.

  // --------------------------------------
  // Memory->Memory Spill. Use R0 to hold the value.
  if (src_lo_rc == rc_stack && dst_lo_rc == rc_stack) {
    return ppc64Opcode_compound;
  }

  // --------------------------------------
  // Check for float->int copy; requires a trip through memory.
  if (src_lo_rc == rc_float && dst_lo_rc == rc_int) {
    Unimplemented();
  }

  // --------------------------------------
  // Check for integer reg-reg copy.
  if (src_lo_rc == rc_int && dst_lo_rc == rc_int) {
    Register Rsrc = as_Register(Matcher::_regEncode[src_lo]);
    Register Rdst = as_Register(Matcher::_regEncode[dst_lo]);
    if (Rsrc == Rdst) {
      return ppc64Opcode_none;
    } else {
      return ppc64Opcode_or;
    }
  }

  // Check for integer store.
  if (src_lo_rc == rc_int && dst_lo_rc == rc_stack) {
    if (src_hi != OptoReg::Bad) {
      return ppc64Opcode_std;
    } else {
      return ppc64Opcode_stw;
    }
  }

  // Check for integer load.
  if (dst_lo_rc == rc_int && src_lo_rc == rc_stack) {
    if (src_hi != OptoReg::Bad) {
      return ppc64Opcode_ld;
    } else {
      return ppc64Opcode_lwz;
    }
  }

  // Check for float reg-reg copy.
  if (src_lo_rc == rc_float && dst_lo_rc == rc_float) {
    return ppc64Opcode_fmr;
  }

  // Check for float store.
  if (src_lo_rc == rc_float && dst_lo_rc == rc_stack) {
    if (src_hi != OptoReg::Bad) {
      return ppc64Opcode_stfd;
    } else {
      return ppc64Opcode_stfs;
    }
  }

  // Check for float load.
  if (dst_lo_rc == rc_float && src_lo_rc == rc_stack) {
    if (src_hi != OptoReg::Bad) {
      return ppc64Opcode_lfd;
    } else {
      return ppc64Opcode_lfs;
    }
  }

  // --------------------------------------------------------------------
  // Check for hi bits still needing moving. Only happens for misaligned
  // arguments to native calls.
  if (src_hi == dst_hi)
    return ppc64Opcode_none;               // Self copy; no move.

  ShouldNotReachHere();
  return ppc64Opcode_undefined;
}
#endif // PPC port

#ifndef PRODUCT
void MachNopNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  st->print("NOP \t// %d nops to pad for loops.", _count);
}
#endif

void MachNopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *) const {
  MacroAssembler _masm(&cbuf);
  // _count contains the number of nops needed for padding.
  for (int i = 0; i < _count; i++) {
    __ nop();
  }
}

uint MachNopNode::size(PhaseRegAlloc *ra_) const {
   return _count * 4;
}

#ifndef PRODUCT
void BoxLockNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_reg_first(this);
  st->print("ADDI %s, SP, %d \t// box node", Matcher::regName[reg], offset);
}
#endif

void BoxLockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  MacroAssembler _masm(&cbuf);

  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg    = ra_->get_encode(this);

  if (Assembler::is_simm(offset, 16)) {
    __ addi(as_Register(reg), R1, offset);
  } else {
    ShouldNotReachHere();
  }
}

uint BoxLockNode::size(PhaseRegAlloc *ra_) const {
  // BoxLockNode is not a MachNode, so we can't just call MachNode::size(ra_).
  return 4;
}

#ifndef PRODUCT
void MachUEPNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  st->print_cr("---- MachUEPNode ----");
  st->print_cr("...");
}
#endif

void MachUEPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  // This is the unverified entry point.
  MacroAssembler _masm(&cbuf);

  // Inline_cache contains a klass.
  Register ic_klass       = as_Register(Matcher::inline_cache_reg_encode());
  Register receiver_klass = R0;  // tmp

  assert_different_registers(ic_klass, receiver_klass, R11_scratch1, R3_ARG1);
  assert(R11_scratch1 == R11, "need prologue scratch register");

  // Check for NULL argument if we don't have implicit null checks.
  if (!ImplicitNullChecks || !os::zero_page_read_protected()) {
    if (TrapBasedNullChecks) {
      __ trap_null_check(R3_ARG1);
    } else {
      Label valid;
      __ cmpdi(CCR0, R3_ARG1, 0);
      __ bne_predict_taken(CCR0, valid);
      // We have a null argument, branch to ic_miss_stub.
      __ b64_patchable((address)SharedRuntime::get_ic_miss_stub(),
                           relocInfo::runtime_call_type);
      __ bind(valid);
    }
  }
  // Assume argument is not NULL, load klass from receiver.
  __ load_klass(receiver_klass, R3_ARG1);

  if (TrapBasedICMissChecks) {
    __ trap_ic_miss_check(receiver_klass, ic_klass);
  } else {
    Label valid;
    __ cmpd(CCR0, receiver_klass, ic_klass);
    __ beq_predict_taken(CCR0, valid);
    // We have an unexpected klass, branch to ic_miss_stub.
    __ b64_patchable((address)SharedRuntime::get_ic_miss_stub(),
                         relocInfo::runtime_call_type);
    __ bind(valid);
  }

  // Argument is valid and klass is as expected, continue.
}

#if 0 // TODO: PPC port
// Optimize UEP code on z (save a load_const() call in main path).
int MachUEPNode::ep_offset() {
  return 0;
}
#endif

uint MachUEPNode::size(PhaseRegAlloc *ra_) const {
  // Variable size. Determine dynamically.
  return MachNode::size(ra_);
}

//=============================================================================

uint size_exception_handler() {
  // The exception_handler is a b64_patchable.
  return MacroAssembler::b64_patchable_size;
}

uint size_deopt_handler() {
  // The deopt_handler is a bl64_patchable.
  return MacroAssembler::bl64_patchable_size;
}

int emit_exception_handler(CodeBuffer &cbuf) {
  MacroAssembler _masm(&cbuf);

  address base = __ start_a_stub(size_exception_handler());
  if (base == NULL) return 0; // CodeBuffer::expand failed

  int offset = __ offset();
  __ b64_patchable((address)OptoRuntime::exception_blob()->content_begin(),
                       relocInfo::runtime_call_type);
  assert(__ offset() - offset == (int)size_exception_handler(), "must be fixed size");
  __ end_a_stub();

  return offset;
}

// The deopt_handler is like the exception handler, but it calls to
// the deoptimization blob instead of jumping to the exception blob.
int emit_deopt_handler(CodeBuffer& cbuf) {
  MacroAssembler _masm(&cbuf);

  address base = __ start_a_stub(size_deopt_handler());
  if (base == NULL) return 0; // CodeBuffer::expand failed

  int offset = __ offset();
  __ bl64_patchable((address)SharedRuntime::deopt_blob()->unpack(),
                        relocInfo::runtime_call_type);
  assert(__ offset() - offset == (int) size_deopt_handler(), "must be fixed size");
  __ end_a_stub();

  return offset;
}

//=============================================================================

// Given a register encoding, produce an Integer Register object.
static Register reg_to_register_object(int register_encoding) {
  // Verify ppc encoding by example.
  assert(R12->encoding() == R12_enc, "wrong coding");
  return as_Register(register_encoding);
}

// Given a register encoding, produce a Float Register object.
static FloatRegister reg_to_FloatRegister_object(int register_encoding) {
  // Verify ppc encoding by example.
  assert(F5->encoding() == F5_enc, "wrong coding");
  return as_FloatRegister(register_encoding);
}
// Given a register encoding, produce a ConditionRegister object.
static ConditionRegister reg_to_ConditionRegister_object(int register_encoding) {
  // Verify ppc encoding by example.
  assert(CCR5->encoding() == CCR5_enc, "wrong coding");
  return as_ConditionRegister(register_encoding);
}

const bool Matcher::match_rule_supported(int opcode) {
  if (!has_match_rule(opcode))
    return false;

  switch (opcode) {
  case Op_SqrtD:
    return VM_Version::has_fsqrt();
  case Op_CountLeadingZerosI:
  case Op_CountLeadingZerosL:
  case Op_CountTrailingZerosI:
  case Op_CountTrailingZerosL:
    if (!UseCountLeadingZerosInstructionsPPC64)
      return false;
    break;

  case Op_PopCountI:
  case Op_PopCountL:
    return (UsePopCountInstruction && VM_Version::has_popcntw());

  case Op_StrComp:
    return SpecialStringCompareTo;
  case Op_StrEquals:
    return SpecialStringEquals;
  case Op_StrIndexOf:
    return SpecialStringIndexOf;
  }

  return true;  // Per default match rules are supported.
}

int Matcher::regnum_to_fpu_offset(int regnum) {
  // No user for this method?
  Unimplemented();
  return 999;
}

const bool Matcher::convL2FSupported(void) {
  // fcfids can do the conversion (>= Power7).
  // fcfid + frsp showed rounding problem when result should be 0x3f800001.
  return VM_Version::has_fcfids(); // False means that conversion is done by runtime call.
}

// Map Types to machine register types
const int Matcher::base2reg[Type::lastype] = {
  Node::NotAMachineReg,0,0, Op_RegI, Op_RegL, 0, Op_RegN,
  Node::NotAMachineReg, Node::NotAMachineReg, /* tuple, array */
  0, Op_RegL, 0, 0, /* Vectors */
  Op_RegP, Op_RegP, Op_RegP, Op_RegP, Op_RegP, Op_RegP, /* the pointers */
  0, 0/*abio*/,
  Op_RegP /* Return address */, 0, /* the memories */
  Op_RegF, Op_RegF, Op_RegF, Op_RegD, Op_RegD, Op_RegD,
  0  /*bottom*/
};

// Vector width in bytes.
const int Matcher::vector_width_in_bytes(BasicType bt) {
  assert(MaxVectorSize == 8, "");
  return 8;
}

// Vector ideal reg.
const int Matcher::vector_ideal_reg(int size) {
  assert(MaxVectorSize == 8 && size == 8, "");
  return Op_RegL;
}

const int Matcher::vector_shift_count_ideal_reg(int size) {
  fatal("vector shift is not supported");
  return Node::NotAMachineReg;
}

// Limits on vector size (number of elements) loaded into vector.
const int Matcher::max_vector_size(const BasicType bt) {
  assert(is_java_primitive(bt), "only primitive type vectors");
  return vector_width_in_bytes(bt)/type2aelembytes(bt);
}

const int Matcher::min_vector_size(const BasicType bt) {
  return max_vector_size(bt); // Same as max.
}

// PPC doesn't support misaligned vectors store/load.
const bool Matcher::misaligned_vectors_ok() {
  return false;
}

// PPC AES support not yet implemented
const bool Matcher::pass_original_key_for_aes() {
  return false;
}

// RETURNS: whether this branch offset is short enough that a short
// branch can be used.
//
// If the platform does not provide any short branch variants, then
// this method should return `false' for offset 0.
//
// `Compile::Fill_buffer' will decide on basis of this information
// whether to do the pass `Compile::Shorten_branches' at all.
//
// And `Compile::Shorten_branches' will decide on basis of this
// information whether to replace particular branch sites by short
// ones.
bool Matcher::is_short_branch_offset(int rule, int br_size, int offset) {
  // Is the offset within the range of a ppc64 pc relative branch?
  bool b;

  const int safety_zone = 3 * BytesPerInstWord;
  b = Assembler::is_simm((offset<0 ? offset-safety_zone : offset+safety_zone),
                         29 - 16 + 1 + 2);
  return b;
}

const bool Matcher::isSimpleConstant64(jlong value) {
  // Probably always true, even if a temp register is required.
  return true;
}
/* TODO: PPC port
// Make a new machine dependent decode node (with its operands).
MachTypeNode *Matcher::make_decode_node(Compile *C) {
  assert(Universe::narrow_oop_base() == NULL && Universe::narrow_oop_shift() == 0,
         "This method is only implemented for unscaled cOops mode so far");
  MachTypeNode *decode = new (C) decodeN_unscaledNode();
  decode->set_opnd_array(0, new (C) iRegPdstOper());
  decode->set_opnd_array(1, new (C) iRegNsrcOper());
  return decode;
}
*/
// Threshold size for cleararray.
const int Matcher::init_array_short_size = 8 * BytesPerLong;

// false => size gets scaled to BytesPerLong, ok.
const bool Matcher::init_array_count_is_in_bytes = false;

// Use conditional move (CMOVL) on Power7.
const int Matcher::long_cmove_cost() { return 0; } // this only makes long cmoves more expensive than int cmoves

// Suppress CMOVF. Conditional move available (sort of) on PPC64 only from P7 onwards. Not exploited yet.
// fsel doesn't accept a condition register as input, so this would be slightly different.
const int Matcher::float_cmove_cost() { return ConditionalMoveLimit; }

// Power6 requires late expand (see block.cpp for description of late expand).
const bool Matcher::require_late_expand = true;

// Should the Matcher clone shifts on addressing modes, expecting them to
// be subsumed into complex addressing expressions or compute them into
// registers? True for Intel but false for most RISCs.
const bool Matcher::clone_shift_expressions = false;

// Do we need to mask the count passed to shift instructions or does
// the cpu only look at the lower 5/6 bits anyway?
// PowerPC requires masked shift counts.
const bool Matcher::need_masked_shift_count = true;

// This affects two different things:
//  - how Decode nodes are matched
//  - how ImplicitNullCheck opportunities are recognized
// If true, the matcher will try to remove all Decodes and match them
// (as operands) into nodes. NullChecks are not prepared to deal with
// Decodes by final_graph_reshaping().
// If false, final_graph_reshaping() forces the decode behind the Cmp
// for a NullCheck. The matcher matches the Decode node into a register.
// Implicit_null_check optimization moves the Decode along with the
// memory operation back up before the NullCheck.
bool Matcher::narrow_oop_use_complex_address() {
  // TODO: PPC port if (MatchDecodeNodes) return true;
  return false;
}

// Is it better to copy float constants, or load them directly from memory?
// Intel can load a float constant from a direct address, requiring no
// extra registers. Most RISCs will have to materialize an address into a
// register first, so they would do better to copy the constant from stack.
const bool Matcher::rematerialize_float_constants = false;

// If CPU can load and store mis-aligned doubles directly then no fixup is
// needed. Else we split the double into 2 integer pieces and move it
// piece-by-piece. Only happens when passing doubles into C code as the
// Java calling convention forces doubles to be aligned.
const bool Matcher::misaligned_doubles_ok = true;

void Matcher::pd_implicit_null_fixup(MachNode *node, uint idx) {
 Unimplemented();
}

// Advertise here if the CPU requires explicit rounding operations
// to implement the UseStrictFP mode.
const bool Matcher::strict_fp_requires_explicit_rounding = false;

// Do floats take an entire double register or just half?
//
// A float occupies a ppc64 double register. For the allocator, a
// ppc64 double register appears as a pair of float registers.
bool Matcher::float_in_double() { return true; }

// Do ints take an entire long register or just half?
// The relevant question is how the int is callee-saved:
// the whole long is written but de-opt'ing will have to extract
// the relevant 32 bits.
const bool Matcher::int_in_long = true;

// Constants for c2c and c calling conventions.

const MachRegisterNumbers iarg_reg[8] = {
  R3_num, R4_num, R5_num, R6_num,
  R7_num, R8_num, R9_num, R10_num
};

const MachRegisterNumbers farg_reg[13] = {
  F1_num, F2_num, F3_num, F4_num,
  F5_num, F6_num, F7_num, F8_num,
  F9_num, F10_num, F11_num, F12_num,
  F13_num
};

const int num_iarg_registers = sizeof(iarg_reg) / sizeof(iarg_reg[0]);

const int num_farg_registers = sizeof(farg_reg) / sizeof(farg_reg[0]);

// Return whether or not this register is ever used as an argument. This
// function is used on startup to build the trampoline stubs in generateOptoStub.
// Registers not mentioned will be killed by the VM call in the trampoline, and
// arguments in those registers not be available to the callee.
bool Matcher::can_be_java_arg(int reg) {
  // We return true for all registers contained in iarg_reg[] and
  // farg_reg[] and their virtual halves.
  // We must include the virtual halves in order to get STDs and LDs
  // instead of STWs and LWs in the trampoline stubs.

  if (   reg == R3_num  || reg == R3_H_num
      || reg == R4_num  || reg == R4_H_num
      || reg == R5_num  || reg == R5_H_num
      || reg == R6_num  || reg == R6_H_num
      || reg == R7_num  || reg == R7_H_num
      || reg == R8_num  || reg == R8_H_num
      || reg == R9_num  || reg == R9_H_num
      || reg == R10_num || reg == R10_H_num)
    return true;

  if (   reg == F1_num  || reg == F1_H_num
      || reg == F2_num  || reg == F2_H_num
      || reg == F3_num  || reg == F3_H_num
      || reg == F4_num  || reg == F4_H_num
      || reg == F5_num  || reg == F5_H_num
      || reg == F6_num  || reg == F6_H_num
      || reg == F7_num  || reg == F7_H_num
      || reg == F8_num  || reg == F8_H_num
      || reg == F9_num  || reg == F9_H_num
      || reg == F10_num || reg == F10_H_num
      || reg == F11_num || reg == F11_H_num
      || reg == F12_num || reg == F12_H_num
      || reg == F13_num || reg == F13_H_num)
    return true;

  return false;
}

bool Matcher::is_spillable_arg(int reg) {
  return can_be_java_arg(reg);
}

bool Matcher::use_asm_for_ldiv_by_con(jlong divisor) {
  return false;
}

// Register for DIVI projection of divmodI.
RegMask Matcher::divI_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODI projection of divmodI.
RegMask Matcher::modI_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for DIVL projection of divmodL.
RegMask Matcher::divL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODL projection of divmodL.
RegMask Matcher::modL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

const RegMask Matcher::method_handle_invoke_SP_save_mask() {
  return RegMask();
}


#line 1832 "../generated/adfiles/ad_ppc_64.cpp"


#ifndef PRODUCT
void Compile::adlc_verification() {

  // Following assertions generated from definition section
  assert( DEFAULT_COST_LOW == 30, "Expect (30) to equal 30");
  assert( DEFAULT_COST == 100, "Expect (100) to equal 100");
  assert( HUGE_COST == 1000000, "Expect (1000000) to equal 1000000");
  assert( MEMORY_REF_COST_LOW == 200, "Expect (DEFAULT_COST * 2) to equal 200");
  assert( MEMORY_REF_COST == 300, "Expect (DEFAULT_COST * 3) to equal 300");
  assert( BRANCH_COST == 900, "Expect (DEFAULT_COST * 9) to equal 900");
  assert( CALL_COST == 1300, "Expect (DEFAULT_COST * 13) to equal 1300");
}
#endif

// Map from machine-independent register number to register_save_policy
const        char register_save_policy[] = {
  'C', // R14
  'C', // R14_H
  'C', // R15
  'C', // R15_H
  'C', // R17
  'C', // R17_H
  'C', // R18
  'C', // R18_H
  'C', // R19
  'C', // R19_H
  'C', // R20
  'C', // R20_H
  'C', // R21
  'C', // R21_H
  'C', // R22
  'C', // R22_H
  'C', // R23
  'C', // R23_H
  'C', // R24
  'C', // R24_H
  'C', // R25
  'C', // R25_H
  'C', // R26
  'C', // R26_H
  'C', // R27
  'C', // R27_H
  'C', // R28
  'C', // R28_H
  'C', // R29
  'C', // R29_H
  'C', // R30
  'C', // R30_H
  'C', // R31
  'C', // R31_H
  'C', // R11
  'C', // R11_H
  'C', // R12
  'C', // R12_H
  'C', // R10
  'C', // R10_H
  'C', // R9
  'C', // R9_H
  'C', // R8
  'C', // R8_H
  'C', // R7
  'C', // R7_H
  'C', // R6
  'C', // R6_H
  'C', // R5
  'C', // R5_H
  'C', // R4
  'C', // R4_H
  'C', // R3
  'C', // R3_H
  'C', // R16
  'C', // R16_H
  'N', // R13
  'N', // R13_H
  'C', // R2
  'C', // R2_H
  'N', // R1
  'N', // R1_H
  'C', // R0
  'C', // R0_H
  'C', // F0
  'C', // F0_H
  'C', // F13
  'C', // F13_H
  'C', // F12
  'C', // F12_H
  'C', // F11
  'C', // F11_H
  'C', // F10
  'C', // F10_H
  'C', // F9
  'C', // F9_H
  'C', // F8
  'C', // F8_H
  'C', // F7
  'C', // F7_H
  'C', // F6
  'C', // F6_H
  'C', // F5
  'C', // F5_H
  'C', // F4
  'C', // F4_H
  'C', // F3
  'C', // F3_H
  'C', // F2
  'C', // F2_H
  'C', // F1
  'C', // F1_H
  'C', // F14
  'C', // F14_H
  'C', // F15
  'C', // F15_H
  'C', // F16
  'C', // F16_H
  'C', // F17
  'C', // F17_H
  'C', // F18
  'C', // F18_H
  'C', // F19
  'C', // F19_H
  'C', // F20
  'C', // F20_H
  'C', // F21
  'C', // F21_H
  'C', // F22
  'C', // F22_H
  'C', // F23
  'C', // F23_H
  'C', // F24
  'C', // F24_H
  'C', // F25
  'C', // F25_H
  'C', // F26
  'C', // F26_H
  'C', // F27
  'C', // F27_H
  'C', // F28
  'C', // F28_H
  'C', // F29
  'C', // F29_H
  'C', // F30
  'C', // F30_H
  'C', // F31
  'C', // F31_H
  'C', // CCR0
  'C', // CCR1
  'C', // CCR2
  'C', // CCR3
  'C', // CCR4
  'C', // CCR5
  'C', // CCR6
  'C', // CCR7
  'C', // SR_XER
  'C', // SR_LR
  'C', // SR_CTR
  'C', // SR_VRSAVE
  'C', // SR_SPEFSCR
  'C' // no trailing comma // SR_PPR
};

// Map from machine-independent register number to c_reg_save_policy
const        char c_reg_save_policy[] = {
  'E', // R14
  'E', // R14_H
  'E', // R15
  'E', // R15_H
  'E', // R17
  'E', // R17_H
  'E', // R18
  'E', // R18_H
  'E', // R19
  'E', // R19_H
  'E', // R20
  'E', // R20_H
  'E', // R21
  'E', // R21_H
  'E', // R22
  'E', // R22_H
  'E', // R23
  'E', // R23_H
  'E', // R24
  'E', // R24_H
  'E', // R25
  'E', // R25_H
  'E', // R26
  'E', // R26_H
  'E', // R27
  'E', // R27_H
  'E', // R28
  'E', // R28_H
  'E', // R29
  'E', // R29_H
  'E', // R30
  'E', // R30_H
  'E', // R31
  'E', // R31_H
  'C', // R11
  'C', // R11_H
  'C', // R12
  'C', // R12_H
  'C', // R10
  'C', // R10_H
  'C', // R9
  'C', // R9_H
  'C', // R8
  'C', // R8_H
  'C', // R7
  'C', // R7_H
  'C', // R6
  'C', // R6_H
  'C', // R5
  'C', // R5_H
  'C', // R4
  'C', // R4_H
  'C', // R3
  'C', // R3_H
  'E', // R16
  'E', // R16_H
  'N', // R13
  'N', // R13_H
  'C', // R2
  'C', // R2_H
  'N', // R1
  'N', // R1_H
  'C', // R0
  'C', // R0_H
  'C', // F0
  'C', // F0_H
  'C', // F13
  'C', // F13_H
  'C', // F12
  'C', // F12_H
  'C', // F11
  'C', // F11_H
  'C', // F10
  'C', // F10_H
  'C', // F9
  'C', // F9_H
  'C', // F8
  'C', // F8_H
  'C', // F7
  'C', // F7_H
  'C', // F6
  'C', // F6_H
  'C', // F5
  'C', // F5_H
  'C', // F4
  'C', // F4_H
  'C', // F3
  'C', // F3_H
  'C', // F2
  'C', // F2_H
  'C', // F1
  'C', // F1_H
  'E', // F14
  'E', // F14_H
  'E', // F15
  'E', // F15_H
  'E', // F16
  'E', // F16_H
  'E', // F17
  'E', // F17_H
  'E', // F18
  'E', // F18_H
  'E', // F19
  'E', // F19_H
  'E', // F20
  'E', // F20_H
  'E', // F21
  'E', // F21_H
  'E', // F22
  'E', // F22_H
  'E', // F23
  'E', // F23_H
  'E', // F24
  'E', // F24_H
  'E', // F25
  'E', // F25_H
  'E', // F26
  'E', // F26_H
  'E', // F27
  'E', // F27_H
  'E', // F28
  'E', // F28_H
  'E', // F29
  'E', // F29_H
  'E', // F30
  'E', // F30_H
  'E', // F31
  'E', // F31_H
  'C', // CCR0
  'C', // CCR1
  'C', // CCR2
  'C', // CCR3
  'C', // CCR4
  'C', // CCR5
  'C', // CCR6
  'C', // CCR7
  'C', // SR_XER
  'C', // SR_LR
  'C', // SR_CTR
  'C', // SR_VRSAVE
  'C', // SR_SPEFSCR
  'C' // no trailing comma // SR_PPR
};

// Map from machine-independent register number to register_save_type
const        int register_save_type[] = {
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegFlags,
  Op_RegFlags,
  Op_RegFlags,
  Op_RegFlags,
  Op_RegFlags,
  Op_RegFlags,
  Op_RegFlags,
  Op_RegFlags,
  Op_RegP,
  Op_RegP,
  Op_RegP,
  Op_RegP,
  Op_RegP,
  Op_RegP // no trailing comma
};


const        int   reduceOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  immI_rule,
  /*    9 */  immI8_rule,
  /*   10 */  immI16_rule,
  /*   11 */  immIhi16_rule,
  /*   12 */  immInegpow2_rule,
  /*   13 */  immIpow2minus1_rule,
  /*   14 */  immIpowerOf2_rule,
  /*   15 */  uimmI5_rule,
  /*   16 */  uimmI6_rule,
  /*   17 */  uimmI6_ge32_rule,
  /*   18 */  uimmI15_rule,
  /*   19 */  uimmI16_rule,
  /*   20 */  immI_0_rule,
  /*   21 */  immI_1_rule,
  /*   22 */  immI_minus1_rule,
  /*   23 */  immI_16_rule,
  /*   24 */  immI_24_rule,
  /*   25 */  immN_rule,
  /*   26 */  immN_0_rule,
  /*   27 */  immP_rule,
  /*   28 */  immP_NM_rule,
  /*   29 */  immP_0_rule,
  /*   30 */  immP_0or1_rule,
  /*   31 */  immP_poll_rule,
  /*   32 */  immL_rule,
  /*   33 */  immL16_rule,
  /*   34 */  immL16Alg4_rule,
  /*   35 */  immL32hi16_rule,
  /*   36 */  immL32_rule,
  /*   37 */  immLhighest16_rule,
  /*   38 */  immLnegpow2_rule,
  /*   39 */  immLpow2minus1_rule,
  /*   40 */  immL_0_rule,
  /*   41 */  immL_minus1_rule,
  /*   42 */  immL_32bits_rule,
  /*   43 */  uimmL16_rule,
  /*   44 */  immF_rule,
  /*   45 */  immF_0_rule,
  /*   46 */  immD_rule,
  /*   47 */  iRegIdst_rule,
  /*   48 */  iRegIsrc_rule,
  /*   49 */  rscratch1RegI_rule,
  /*   50 */  rscratch2RegI_rule,
  /*   51 */  rarg1RegI_rule,
  /*   52 */  rarg2RegI_rule,
  /*   53 */  rarg3RegI_rule,
  /*   54 */  rarg4RegI_rule,
  /*   55 */  rarg1RegL_rule,
  /*   56 */  rarg2RegL_rule,
  /*   57 */  rarg3RegL_rule,
  /*   58 */  rarg4RegL_rule,
  /*   59 */  iRegPdst_rule,
  /*   60 */  iRegPdstNoScratch_rule,
  /*   61 */  iRegPsrc_rule,
  /*   62 */  threadRegP_rule,
  /*   63 */  rscratch1RegP_rule,
  /*   64 */  rscratch2RegP_rule,
  /*   65 */  rarg1RegP_rule,
  /*   66 */  rarg2RegP_rule,
  /*   67 */  rarg3RegP_rule,
  /*   68 */  rarg4RegP_rule,
  /*   69 */  iRegNsrc_rule,
  /*   70 */  iRegNdst_rule,
  /*   71 */  iRegLdst_rule,
  /*   72 */  iRegLsrc_rule,
  /*   73 */  iRegL2Isrc_rule,
  /*   74 */  rscratch1RegL_rule,
  /*   75 */  rscratch2RegL_rule,
  /*   76 */  flagsReg_rule,
  /*   77 */  flagsRegCR0_rule,
  /*   78 */  flagsRegCR1_rule,
  /*   79 */  flagsRegCR6_rule,
  /*   80 */  regCTR_rule,
  /*   81 */  regD_rule,
  /*   82 */  regF_rule,
  /*   83 */  inline_cache_regP_rule,
  /*   84 */  compiler_method_oop_regP_rule,
  /*   85 */  interpreter_method_oop_regP_rule,
  /*   86 */  iRegP2N_rule,
  /*   87 */  iRegN2P_rule,
  /*   88 */  indirect_rule,
  /*   89 */  indOffset16_rule,
  /*   90 */  indOffset16Alg4_rule,
  /*   91 */  indirectNarrow_rule,
  /*   92 */  indOffset16Narrow_rule,
  /*   93 */  indOffset16NarrowAlg4_rule,
  /*   94 */  stackSlotI_rule,
  /*   95 */  stackSlotL_rule,
  /*   96 */  stackSlotP_rule,
  /*   97 */  stackSlotF_rule,
  /*   98 */  stackSlotD_rule,
  /*   99 */  cmpOp_rule,
  // last operand
  /*  100 */  memory_rule,
  /*  101 */  memoryAlg4_rule,
  /*  102 */  indirectMemory_rule,
  /*  103 */  iRegIsrc_iRegL2Isrc_rule,
  /*  104 */  iRegN_P2N_rule,
  /*  105 */  iRegP_N2P_rule,
  // last operand class
  /*  106 */  _DecodeN_iRegNsrc__rule,
  /*  107 */  _LoadUB_memory__rule,
  /*  108 */  _LoadUS_memory__rule,
  /*  109 */  _LoadI_memory__rule,
  /*  110 */  _ConvI2L__LoadI_memory___rule,
  /*  111 */  _LoadI_memoryAlg4__rule,
  /*  112 */  _LoadN_memory__rule,
  /*  113 */  _LoadP_memoryAlg4__rule,
  /*  114 */  _AddP_indirectMemory_iRegLsrc_rule,
  /*  115 */  _ConvL2I_iRegLsrc__rule,
  /*  116 */  _Binary_flagsReg_iRegPsrc_rule,
  /*  117 */  _CastP2X__DecodeN_iRegNsrc___rule,
  /*  118 */  _Binary_cmpOp_flagsReg_rule,
  /*  119 */  _Binary_iRegIdst_iRegIsrc_rule,
  /*  120 */  _Binary_iRegIdst_immI16_rule,
  /*  121 */  _Binary_iRegLdst_iRegLsrc_rule,
  /*  122 */  _Binary_iRegLdst_immL16_rule,
  /*  123 */  _Binary_iRegNdst_iRegNsrc_rule,
  /*  124 */  _Binary_iRegNdst_immN_0_rule,
  /*  125 */  _Binary_iRegPdst_iRegPsrc_rule,
  /*  126 */  _Binary_iRegPdst_iRegP_N2P_rule,
  /*  127 */  _Binary_iRegPdst_immP_0_rule,
  /*  128 */  _Binary_regF_regF_rule,
  /*  129 */  _Binary_regD_regD_rule,
  /*  130 */  _Binary_iRegLsrc_iRegLsrc_rule,
  /*  131 */  _Binary_iRegPsrc_iRegPsrc_rule,
  /*  132 */  _Binary_iRegIsrc_iRegIsrc_rule,
  /*  133 */  _Binary_iRegNsrc_iRegNsrc_rule,
  /*  134 */  _AddI_iRegIsrc_iRegIsrc_rule,
  /*  135 */  _AddI__AddI_iRegIsrc_iRegIsrc_iRegIsrc_rule,
  /*  136 */  _AddI_iRegIsrc__AddI_iRegIsrc_iRegIsrc_rule,
  /*  137 */  _AddL_iRegLsrc_iRegLsrc_rule,
  /*  138 */  _AddL__AddL_iRegLsrc_iRegLsrc_iRegLsrc_rule,
  /*  139 */  _AddL_iRegLsrc__AddL_iRegLsrc_iRegLsrc_rule,
  /*  140 */  _SubL_iRegLsrc_iRegLsrc_rule,
  /*  141 */  _SubL_immL_0_iRegLsrc_rule,
  /*  142 */  _AndI_iRegIsrc_immInegpow2_rule,
  /*  143 */  _RShiftI_iRegIsrc_uimmI5_rule,
  /*  144 */  _AndI__RShiftI_iRegIsrc_uimmI5_immInegpow2_rule,
  /*  145 */  _ConvI2L_iRegIsrc__rule,
  /*  146 */  _RShiftL_iRegLsrc_immI_rule,
  /*  147 */  _URShiftL_iRegLsrc_immI_rule,
  /*  148 */  _CastP2X_iRegP_N2P__rule,
  /*  149 */  _LShiftI_iRegIsrc_immI8_rule,
  /*  150 */  _URShiftI_iRegIsrc_immI8_rule,
  /*  151 */  _AbsF_regF__rule,
  /*  152 */  _AbsD_regD__rule,
  /*  153 */  _ConvF2D_regF__rule,
  /*  154 */  _SqrtD__ConvF2D_regF___rule,
  /*  155 */  _AndL_iRegLsrc_immLpow2minus1_rule,
  /*  156 */  _OrI_iRegIsrc_iRegIsrc_rule,
  /*  157 */  _OrI__OrI_iRegIsrc_iRegIsrc_iRegIsrc_rule,
  /*  158 */  _OrI_iRegIsrc__OrI_iRegIsrc_iRegIsrc_rule,
  /*  159 */  _OrL_iRegLsrc_iRegLsrc_rule,
  /*  160 */  _XorI_iRegIsrc_iRegIsrc_rule,
  /*  161 */  _XorI__XorI_iRegIsrc_iRegIsrc_iRegIsrc_rule,
  /*  162 */  _XorI_iRegIsrc__XorI_iRegIsrc_iRegIsrc_rule,
  /*  163 */  _XorL_iRegLsrc_iRegLsrc_rule,
  /*  164 */  _XorI_iRegIsrc_immI_minus1_rule,
  /*  165 */  _Conv2B_iRegIsrc__rule,
  /*  166 */  _AndI_iRegIsrc_immIpowerOf2_rule,
  /*  167 */  _Conv2B_iRegP_N2P__rule,
  /*  168 */  _LShiftI_iRegIsrc_immI_24_rule,
  /*  169 */  _LShiftI_iRegIsrc_immI_16_rule,
  /*  170 */  _AndI_iRegIsrc_uimmI16_rule,
  /*  171 */  _AndL_iRegLsrc_iRegLsrc_rule,
  /*  172 */  _AndL_iRegLsrc_uimmL16_rule,
  /*  173 */  _CmpU_iRegIsrc_uimmI15_rule,
  /*  174 */  _CmpU_iRegIsrc_iRegIsrc_rule,
  /*  175 */  _CmpN_iRegNsrc_immN_0_rule,
  /*  176 */  _CmpP_iRegP_N2P_immP_0_rule,
  /*  177 */  _CastP2X_iRegPsrc__rule,
  /*  178 */  _AndL__CastP2X_iRegPsrc__immLnegpow2_rule,
  /*  179 */  _Binary_iRegPsrc_iRegIsrc_rule,
  /*  180 */  _AddP_immP_immL_rule,
  /*  181 */  _Binary__AddP_immP_immL_immI_1_rule,
  /*  182 */  _Binary_rscratch2RegP_immI_1_rule,
  /*  183 */  _Binary_iRegPsrc_rscratch1RegI_rule,
  /*  184 */  _Binary_iRegPsrc_uimmI15_rule,
  /*  185 */  _Binary_iRegPsrc_rscratch2RegI_rule,
  /*  186 */  _Binary_rarg1RegP_rarg3RegI_rule,
  /*  187 */  _Binary_rarg2RegP_rarg4RegI_rule,
  // last internally defined operand
  /*  188 */  iRegPdst_rule,
  /*  189 */  regF_rule,
  /*  190 */  regD_rule,
  /*  191 */  stackSlotI_rule,
  /*  192 */  stackSlotL_rule,
  /*  193 */  iRegIdst_rule,
  /*  194 */  iRegIdst_rule,
  /*  195 */  iRegIdst_rule,
  /*  196 */  iRegLdst_rule,
  /*  197 */  iRegLdst_rule,
  /*  198 */  iRegLdst_rule,
  /*  199 */  iRegLdst_rule,
  /*  200 */  iRegLdst_rule,
  /*  201 */  iRegNdst_rule,
  /*  202 */  iRegNdst_rule,
  /*  203 */  iRegPdst_rule,
  /*  204 */  iRegPdst_rule,
  /*  205 */  iRegPdst_rule,
  /*  206 */  iRegIdst_rule,
  /*  207 */  iRegLdst_rule,
  /*  208 */  rscratch2RegP_rule,
  /*  209 */  0,
  /*  210 */  iRegLdst_rule,
  /*  211 */  0,
  /*  212 */  0,
  /*  213 */  0,
  /*  214 */  flagsReg_rule,
  /*  215 */  flagsReg_rule,
  /*  216 */  0,
  /*  217 */  iRegIdst_rule,
  /*  218 */  0,
  /*  219 */  0,
  /*  220 */  regF_rule,
  /*  221 */  regD_rule,
  /*  222 */  regF_rule,
  /*  223 */  regD_rule,
  /*  224 */  regF_rule,
  /*  225 */  regD_rule,
  /*  226 */  regD_rule,
  /*  227 */  regF_rule,
  /*  228 */  iRegIdst_rule,
  /*  229 */  regF_rule,
  /*  230 */  iRegLdst_rule,
  /*  231 */  regD_rule,
  /*  232 */  0,
  /*  233 */  iRegPdst_rule,
  /*  234 */  iRegLdst_rule,
  /*  235 */  iRegPdst_rule,
  /*  236 */  iRegIdst_rule,
  /*  237 */  iRegPdst_rule,
  /*  238 */  iRegIdst_rule,
  /*  239 */  iRegIdst_rule,
  /*  240 */  0,
  /*  241 */  0,
  /*  242 */  0,
  /*  243 */  0,
  /*  244 */  0,
  /*  245 */  0,
  /*  246 */  flagsReg_rule,
  /*  247 */  flagsReg_rule,
  /*  248 */  flagsRegCR0_rule,
  /*  249 */  flagsReg_rule,
  /*  250 */  flagsReg_rule,
  /*  251 */  flagsReg_rule,
  /*  252 */  flagsReg_rule,
  /*  253 */  flagsRegCR0_rule,
  /*  254 */  flagsRegCR0_rule,
  /*  255 */  0,
  /*  256 */  0,
  /*  257 */  flagsReg_rule,
  /*  258 */  flagsReg_rule,
  /*  259 */  flagsReg_rule,
  /*  260 */  flagsReg_rule,
  /*  261 */  flagsReg_rule,
  /*  262 */  flagsReg_rule,
  /*  263 */  0,
  /*  264 */  0,
  /*  265 */  flagsReg_rule,
  /*  266 */  0,
  /*  267 */  flagsReg_rule,
  /*  268 */  flagsReg_rule,
  /*  269 */  flagsReg_rule,
  /*  270 */  iRegIdst_rule,
  /*  271 */  iRegIdst_rule,
  /*  272 */  iRegIdst_rule,
  /*  273 */  iRegIdst_rule,
  /*  274 */  0,
  /*  275 */  iRegIdst_rule,
  /*  276 */  iRegIdst_rule,
  /*  277 */  0,
  /*  278 */  0,
  /*  279 */  0,
  /*  280 */  0,
  /*  281 */  threadRegP_rule,
  /*  282 */  iRegIdst_rule,
  /*  283 */  iRegIdst_rule,
  /*  284 */  iRegIdst_rule,
  /*  285 */  iRegIdst_rule,
  /*  286 */  iRegIdst_rule,
  /*  287 */  iRegIdst_rule,
  /*  288 */  iRegIdst_rule,
  /*  289 */  iRegIdst_rule,
  /*  290 */  iRegIdst_rule,
  /*  291 */  iRegIdst_rule,
  /*  292 */  iRegLdst_rule,
  /*  293 */  iRegLdst_rule,
  /*  294 */  iRegIdst_rule,
  /*  295 */  iRegIdst_rule,
  /*  296 */  iRegIdst_rule,
  /*  297 */  iRegIdst_rule,
  /*  298 */  iRegLdst_rule,
  /*  299 */  iRegLdst_rule,
  /*  300 */  iRegIdst_rule,
  /*  301 */  iRegIdst_rule,
  /*  302 */  iRegLdst_rule,
  /*  303 */  iRegLdst_rule,
  /*  304 */  iRegLdst_rule,
  /*  305 */  iRegLdst_rule,
  /*  306 */  iRegLdst_rule,
  /*  307 */  iRegLdst_rule,
  /*  308 */  iRegLdst_rule,
  /*  309 */  iRegIdst_rule,
  /*  310 */  iRegNdst_rule,
  /*  311 */  iRegNdst_rule,
  /*  312 */  iRegPdst_rule,
  /*  313 */  iRegPdst_rule,
  /*  314 */  iRegPdst_rule,
  /*  315 */  iRegNdst_rule,
  /*  316 */  iRegPdst_rule,
  /*  317 */  regF_rule,
  /*  318 */  regF_rule,
  /*  319 */  regD_rule,
  /*  320 */  regD_rule,
  /*  321 */  regD_rule,
  /*  322 */  0,
  /*  323 */  0,
  /*  324 */  0,
  /*  325 */  0,
  /*  326 */  0,
  /*  327 */  0,
  /*  328 */  0,
  /*  329 */  0,
  /*  330 */  0,
  /*  331 */  0,
  /*  332 */  0,
  /*  333 */  0,
  /*  334 */  Universe_rule,
  /*  335 */  Universe_rule,
  /*  336 */  Universe_rule,
  /*  337 */  Universe_rule,
  /*  338 */  Universe_rule,
  /*  339 */  Universe_rule,
  /*  340 */  Universe_rule,
  /*  341 */  Universe_rule,
  /*  342 */  Universe_rule,
  /*  343 */  Universe_rule,
  /*  344 */  Universe_rule,
  /*  345 */  Universe_rule,
  /*  346 */  Universe_rule,
  /*  347 */  Universe_rule,
  /*  348 */  Universe_rule,
  /*  349 */  Universe_rule,
  /*  350 */  Universe_rule,
  /*  351 */  Universe_rule,
  /*  352 */  Universe_rule,
  /*  353 */  Universe_rule,
  /*  354 */  Universe_rule,
  /*  355 */  iRegNdst_rule,
  /*  356 */  iRegPdst_rule,
  /*  357 */  iRegNdst_rule,
  /*  358 */  iRegNdst_rule,
  /*  359 */  iRegNdst_rule,
  /*  360 */  iRegNdst_rule,
  /*  361 */  iRegNdst_rule,
  /*  362 */  iRegNdst_rule,
  /*  363 */  iRegNdst_rule,
  /*  364 */  iRegPdst_rule,
  /*  365 */  iRegPdst_rule,
  /*  366 */  iRegPdst_rule,
  /*  367 */  iRegPdst_rule,
  /*  368 */  iRegPdst_rule,
  /*  369 */  iRegPdst_rule,
  /*  370 */  iRegPdst_rule,
  /*  371 */  iRegPdst_rule,
  /*  372 */  iRegIdst_rule,
  /*  373 */  Universe_rule,
  /*  374 */  Universe_rule,
  /*  375 */  Universe_rule,
  /*  376 */  Universe_rule,
  /*  377 */  Universe_rule,
  /*  378 */  Universe_rule,
  /*  379 */  Universe_rule,
  /*  380 */  iRegIdst_rule,
  /*  381 */  iRegIdst_rule,
  /*  382 */  iRegIdst_rule,
  /*  383 */  iRegLdst_rule,
  /*  384 */  iRegLdst_rule,
  /*  385 */  iRegLdst_rule,
  /*  386 */  iRegNdst_rule,
  /*  387 */  iRegNdst_rule,
  /*  388 */  iRegNdst_rule,
  /*  389 */  iRegPdst_rule,
  /*  390 */  iRegPdst_rule,
  /*  391 */  iRegPdst_rule,
  /*  392 */  regF_rule,
  /*  393 */  regD_rule,
  /*  394 */  iRegPdst_rule,
  /*  395 */  iRegIdst_rule,
  /*  396 */  iRegIdst_rule,
  /*  397 */  iRegIdst_rule,
  /*  398 */  iRegIdst_rule,
  /*  399 */  iRegIdst_rule,
  /*  400 */  iRegLdst_rule,
  /*  401 */  iRegIdst_rule,
  /*  402 */  iRegLdst_rule,
  /*  403 */  iRegPdst_rule,
  /*  404 */  iRegNdst_rule,
  /*  405 */  iRegIdst_rule,
  /*  406 */  0,
  /*  407 */  iRegIdst_rule,
  /*  408 */  iRegIdst_rule,
  /*  409 */  iRegIdst_rule,
  /*  410 */  iRegIdst_rule,
  /*  411 */  iRegIdst_rule,
  /*  412 */  iRegIdst_rule,
  /*  413 */  iRegLdst_rule,
  /*  414 */  0,
  /*  415 */  iRegLdst_rule,
  /*  416 */  iRegLdst_rule,
  /*  417 */  iRegLdst_rule,
  /*  418 */  iRegLdst_rule,
  /*  419 */  iRegIdst_rule,
  /*  420 */  iRegLdst_rule,
  /*  421 */  iRegLdst_rule,
  /*  422 */  iRegPdst_rule,
  /*  423 */  iRegPdst_rule,
  /*  424 */  iRegPdst_rule,
  /*  425 */  iRegIdst_rule,
  /*  426 */  iRegIdst_rule,
  /*  427 */  iRegIdst_rule,
  /*  428 */  iRegIdst_rule,
  /*  429 */  iRegLdst_rule,
  /*  430 */  iRegIdst_rule,
  /*  431 */  iRegLdst_rule,
  /*  432 */  iRegLdst_rule,
  /*  433 */  iRegIdst_rule,
  /*  434 */  iRegIdst_rule,
  /*  435 */  iRegIdst_rule,
  /*  436 */  iRegLdst_rule,
  /*  437 */  iRegLdst_rule,
  /*  438 */  iRegLdst_rule,
  /*  439 */  iRegIdst_rule,
  /*  440 */  iRegIdst_rule,
  /*  441 */  0,
  /*  442 */  iRegIdst_rule,
  /*  443 */  iRegLdst_rule,
  /*  444 */  iRegLdst_rule,
  /*  445 */  0,
  /*  446 */  iRegLdst_rule,
  /*  447 */  iRegIdst_rule,
  /*  448 */  iRegLdst_rule,
  /*  449 */  0,
  /*  450 */  0,
  /*  451 */  iRegIdst_rule,
  /*  452 */  iRegIdst_rule,
  /*  453 */  iRegIdst_rule,
  /*  454 */  iRegIdst_rule,
  /*  455 */  0,
  /*  456 */  iRegLdst_rule,
  /*  457 */  iRegLdst_rule,
  /*  458 */  iRegLdst_rule,
  /*  459 */  iRegLdst_rule,
  /*  460 */  0,
  /*  461 */  iRegIdst_rule,
  /*  462 */  iRegIdst_rule,
  /*  463 */  0,
  /*  464 */  iRegLdst_rule,
  /*  465 */  iRegLdst_rule,
  /*  466 */  iRegIdst_rule,
  /*  467 */  0,
  /*  468 */  iRegIdst_rule,
  /*  469 */  iRegIdst_rule,
  /*  470 */  0,
  /*  471 */  iRegLdst_rule,
  /*  472 */  iRegLdst_rule,
  /*  473 */  iRegIdst_rule,
  /*  474 */  iRegLdst_rule,
  /*  475 */  iRegIdst_rule,
  /*  476 */  iRegIdst_rule,
  /*  477 */  iRegIdst_rule,
  /*  478 */  iRegIdst_rule,
  /*  479 */  iRegIdst_rule,
  /*  480 */  regF_rule,
  /*  481 */  regD_rule,
  /*  482 */  regF_rule,
  /*  483 */  regD_rule,
  /*  484 */  regF_rule,
  /*  485 */  regD_rule,
  /*  486 */  regF_rule,
  /*  487 */  regD_rule,
  /*  488 */  regD_rule,
  /*  489 */  regF_rule,
  /*  490 */  iRegIdst_rule,
  /*  491 */  iRegIdst_rule,
  /*  492 */  iRegIdst_rule,
  /*  493 */  iRegIdst_rule,
  /*  494 */  iRegIdst_rule,
  /*  495 */  iRegLdst_rule,
  /*  496 */  iRegLdst_rule,
  /*  497 */  iRegLdst_rule,
  /*  498 */  iRegLdst_rule,
  /*  499 */  iRegIdst_rule,
  /*  500 */  iRegIdst_rule,
  /*  501 */  0,
  /*  502 */  iRegIdst_rule,
  /*  503 */  iRegIdst_rule,
  /*  504 */  iRegIdst_rule,
  /*  505 */  iRegIdst_rule,
  /*  506 */  iRegIdst_rule,
  /*  507 */  iRegLdst_rule,
  /*  508 */  iRegIdst_rule,
  /*  509 */  iRegLdst_rule,
  /*  510 */  iRegIdst_rule,
  /*  511 */  0,
  /*  512 */  iRegIdst_rule,
  /*  513 */  iRegIdst_rule,
  /*  514 */  iRegIdst_rule,
  /*  515 */  iRegIdst_rule,
  /*  516 */  iRegIdst_rule,
  /*  517 */  iRegLdst_rule,
  /*  518 */  iRegIdst_rule,
  /*  519 */  iRegLdst_rule,
  /*  520 */  iRegIdst_rule,
  /*  521 */  iRegLdst_rule,
  /*  522 */  iRegIdst_rule,
  /*  523 */  iRegIdst_rule,
  /*  524 */  0,
  /*  525 */  stackSlotI_rule,
  /*  526 */  stackSlotF_rule,
  /*  527 */  0,
  /*  528 */  stackSlotL_rule,
  /*  529 */  stackSlotD_rule,
  /*  530 */  iRegIdst_rule,
  /*  531 */  iRegIdst_rule,
  /*  532 */  iRegIdst_rule,
  /*  533 */  iRegIdst_rule,
  /*  534 */  iRegIdst_rule,
  /*  535 */  iRegIdst_rule,
  /*  536 */  iRegIdst_rule,
  /*  537 */  iRegIdst_rule,
  /*  538 */  iRegIdst_rule,
  /*  539 */  iRegIdst_rule,
  /*  540 */  iRegIdst_rule,
  /*  541 */  iRegLdst_rule,
  /*  542 */  iRegIdst_rule,
  /*  543 */  0,
  /*  544 */  0,
  /*  545 */  iRegIdst_rule,
  /*  546 */  iRegIdst_rule,
  /*  547 */  iRegLdst_rule,
  /*  548 */  iRegLdst_rule,
  /*  549 */  iRegLdst_rule,
  /*  550 */  0,
  /*  551 */  0,
  /*  552 */  iRegLdst_rule,
  /*  553 */  iRegLdst_rule,
  /*  554 */  regF_rule,
  /*  555 */  regF_rule,
  /*  556 */  regF_rule,
  /*  557 */  regF_rule,
  /*  558 */  regD_rule,
  /*  559 */  regD_rule,
  /*  560 */  regD_rule,
  /*  561 */  iRegIdst_rule,
  /*  562 */  Universe_rule,
  /*  563 */  Universe_rule,
  /*  564 */  Universe_rule,
  /*  565 */  Universe_rule,
  /*  566 */  Universe_rule,
  /*  567 */  iRegIdst_rule,
  /*  568 */  iRegIdst_rule,
  /*  569 */  Universe_rule,
  /*  570 */  Universe_rule,
  /*  571 */  Universe_rule,
  /*  572 */  Universe_rule,
  /*  573 */  Universe_rule,
  /*  574 */  Universe_rule,
  /*  575 */  Universe_rule,
  /*  576 */  iRegPdst_rule,
  /*  577 */  iRegPdst_rule,
  /*  578 */  iRegLdst_rule,
  /*  579 */  Universe_rule,
  /*  580 */  iRegIdst_rule,
  /*  581 */  iRegIdst_rule,
  /*  582 */  iRegIdst_rule,
  /*  583 */  iRegIdst_rule,
  /*  584 */  iRegIdst_rule,
  /*  585 */  iRegIdst_rule,
  /*  586 */  iRegIdst_rule,
  /*  587 */  iRegIdst_rule,
  /*  588 */  iRegIdst_rule,
  /*  589 */  0,
  /*  590 */  0,
  /*  591 */  iRegIdst_rule,
  /*  592 */  iRegLdst_rule,
  /*  593 */  iRegLdst_rule,
  /*  594 */  iRegLdst_rule,
  /*  595 */  iRegLdst_rule,
  /*  596 */  iRegLdst_rule,
  /*  597 */  iRegLdst_rule,
  /*  598 */  iRegLdst_rule,
  /*  599 */  iRegLdst_rule,
  /*  600 */  iRegLdst_rule,
  /*  601 */  iRegLdst_rule,
  /*  602 */  iRegLdst_rule,
  /*  603 */  iRegLdst_rule,
  /*  604 */  Universe_rule,
  /*  605 */  Universe_rule,
  /*  606 */  Universe_rule,
  /*  607 */  Universe_rule,
  /*  608 */  Universe_rule,
  /*  609 */  Universe_rule,
  /*  610 */  Universe_rule,
  /*  611 */  Universe_rule,
  /*  612 */  Universe_rule,
  /*  613 */  Universe_rule,
  /*  614 */  Universe_rule,
  /*  615 */  Universe_rule,
  /*  616 */  Universe_rule,
  /*  617 */  Universe_rule,
  /*  618 */  rarg1RegP_rule,
  /*  619 */  Universe_rule,
  /*  620 */  Universe_rule,
  /*  621 */  0,
  /*  622 */  0,
  /*  623 */  0,
  /*  624 */  0,
  /*  625 */  0,
  /*  626 */  0,
  /*  627 */  0,
  // last instruction
  0 // no trailing comma
};

const        int   leftOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  iRegLsrc_rule,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  0,
  /*   78 */  0,
  /*   79 */  0,
  /*   80 */  0,
  /*   81 */  0,
  /*   82 */  0,
  /*   83 */  0,
  /*   84 */  0,
  /*   85 */  0,
  /*   86 */  iRegPsrc_rule,
  /*   87 */  iRegNsrc_rule,
  /*   88 */  0,
  /*   89 */  iRegPsrc_rule,
  /*   90 */  iRegPsrc_rule,
  /*   91 */  iRegNsrc_rule,
  /*   92 */  _DecodeN_iRegNsrc__rule,
  /*   93 */  _DecodeN_iRegNsrc__rule,
  /*   94 */  0,
  /*   95 */  0,
  /*   96 */  0,
  /*   97 */  0,
  /*   98 */  0,
  /*   99 */  0,
  // last operand
  /*  100 */  0,
  /*  101 */  0,
  /*  102 */  0,
  /*  103 */  0,
  /*  104 */  0,
  /*  105 */  0,
  // last operand class
  /*  106 */  iRegNsrc_rule,
  /*  107 */  memory_rule,
  /*  108 */  memory_rule,
  /*  109 */  memory_rule,
  /*  110 */  _LoadI_memory__rule,
  /*  111 */  memoryAlg4_rule,
  /*  112 */  memory_rule,
  /*  113 */  memoryAlg4_rule,
  /*  114 */  indirectMemory_rule,
  /*  115 */  iRegLsrc_rule,
  /*  116 */  flagsReg_rule,
  /*  117 */  _DecodeN_iRegNsrc__rule,
  /*  118 */  cmpOp_rule,
  /*  119 */  iRegIdst_rule,
  /*  120 */  iRegIdst_rule,
  /*  121 */  iRegLdst_rule,
  /*  122 */  iRegLdst_rule,
  /*  123 */  iRegNdst_rule,
  /*  124 */  iRegNdst_rule,
  /*  125 */  iRegPdst_rule,
  /*  126 */  iRegPdst_rule,
  /*  127 */  iRegPdst_rule,
  /*  128 */  regF_rule,
  /*  129 */  regD_rule,
  /*  130 */  iRegLsrc_rule,
  /*  131 */  iRegPsrc_rule,
  /*  132 */  iRegIsrc_rule,
  /*  133 */  iRegNsrc_rule,
  /*  134 */  iRegIsrc_rule,
  /*  135 */  _AddI_iRegIsrc_iRegIsrc_rule,
  /*  136 */  iRegIsrc_rule,
  /*  137 */  iRegLsrc_rule,
  /*  138 */  _AddL_iRegLsrc_iRegLsrc_rule,
  /*  139 */  iRegLsrc_rule,
  /*  140 */  iRegLsrc_rule,
  /*  141 */  immL_0_rule,
  /*  142 */  iRegIsrc_rule,
  /*  143 */  iRegIsrc_rule,
  /*  144 */  _RShiftI_iRegIsrc_uimmI5_rule,
  /*  145 */  iRegIsrc_rule,
  /*  146 */  iRegLsrc_rule,
  /*  147 */  iRegLsrc_rule,
  /*  148 */  iRegP_N2P_rule,
  /*  149 */  iRegIsrc_rule,
  /*  150 */  iRegIsrc_rule,
  /*  151 */  regF_rule,
  /*  152 */  regD_rule,
  /*  153 */  regF_rule,
  /*  154 */  _ConvF2D_regF__rule,
  /*  155 */  iRegLsrc_rule,
  /*  156 */  iRegIsrc_rule,
  /*  157 */  _OrI_iRegIsrc_iRegIsrc_rule,
  /*  158 */  iRegIsrc_rule,
  /*  159 */  iRegLsrc_rule,
  /*  160 */  iRegIsrc_rule,
  /*  161 */  _XorI_iRegIsrc_iRegIsrc_rule,
  /*  162 */  iRegIsrc_rule,
  /*  163 */  iRegLsrc_rule,
  /*  164 */  iRegIsrc_rule,
  /*  165 */  iRegIsrc_rule,
  /*  166 */  iRegIsrc_rule,
  /*  167 */  iRegP_N2P_rule,
  /*  168 */  iRegIsrc_rule,
  /*  169 */  iRegIsrc_rule,
  /*  170 */  iRegIsrc_rule,
  /*  171 */  iRegLsrc_rule,
  /*  172 */  iRegLsrc_rule,
  /*  173 */  iRegIsrc_rule,
  /*  174 */  iRegIsrc_rule,
  /*  175 */  iRegNsrc_rule,
  /*  176 */  iRegP_N2P_rule,
  /*  177 */  iRegPsrc_rule,
  /*  178 */  _CastP2X_iRegPsrc__rule,
  /*  179 */  iRegPsrc_rule,
  /*  180 */  immP_rule,
  /*  181 */  _AddP_immP_immL_rule,
  /*  182 */  rscratch2RegP_rule,
  /*  183 */  iRegPsrc_rule,
  /*  184 */  iRegPsrc_rule,
  /*  185 */  iRegPsrc_rule,
  /*  186 */  rarg1RegP_rule,
  /*  187 */  rarg2RegP_rule,
  // last internally defined operand
  /*  188 */  immP_rule,
  /*  189 */  immF_rule,
  /*  190 */  immD_rule,
  /*  191 */  iRegIsrc_rule,
  /*  192 */  iRegLsrc_rule,
  /*  193 */  immI16_rule,
  /*  194 */  immIhi16_rule,
  /*  195 */  immI_rule,
  /*  196 */  immL16_rule,
  /*  197 */  immL32hi16_rule,
  /*  198 */  immL32_rule,
  /*  199 */  immLhighest16_rule,
  /*  200 */  immL_rule,
  /*  201 */  immN_0_rule,
  /*  202 */  immN_rule,
  /*  203 */  immP_0or1_rule,
  /*  204 */  immP_NM_rule,
  /*  205 */  immP_NM_rule,
  /*  206 */  stackSlotI_rule,
  /*  207 */  stackSlotL_rule,
  /*  208 */  immP_NM_rule,
  /*  209 */  0,
  /*  210 */  _LoadP_memoryAlg4__rule,
  /*  211 */  0,
  /*  212 */  0,
  /*  213 */  0,
  /*  214 */  indirect_rule,
  /*  215 */  indirect_rule,
  /*  216 */  0,
  /*  217 */  iRegIsrc_rule,
  /*  218 */  0,
  /*  219 */  0,
  /*  220 */  regF_rule,
  /*  221 */  regD_rule,
  /*  222 */  regF_rule,
  /*  223 */  regD_rule,
  /*  224 */  _AbsF_regF__rule,
  /*  225 */  _AbsD_regD__rule,
  /*  226 */  regD_rule,
  /*  227 */  regF_rule,
  /*  228 */  stackSlotF_rule,
  /*  229 */  stackSlotI_rule,
  /*  230 */  stackSlotD_rule,
  /*  231 */  stackSlotL_rule,
  /*  232 */  0,
  /*  233 */  iRegLsrc_rule,
  /*  234 */  iRegP_N2P_rule,
  /*  235 */  iRegPdst_rule,
  /*  236 */  iRegIdst_rule,
  /*  237 */  iRegPdst_rule,
  /*  238 */  iRegIsrc_rule,
  /*  239 */  iRegP_N2P_rule,
  /*  240 */  0,
  /*  241 */  0,
  /*  242 */  0,
  /*  243 */  0,
  /*  244 */  0,
  /*  245 */  0,
  /*  246 */  iRegIsrc_rule,
  /*  247 */  iRegIsrc_rule,
  /*  248 */  _AndI_iRegIsrc_uimmI16_rule,
  /*  249 */  iRegLsrc_rule,
  /*  250 */  iRegLsrc_rule,
  /*  251 */  iRegLsrc_rule,
  /*  252 */  iRegLsrc_rule,
  /*  253 */  _AndL_iRegLsrc_iRegLsrc_rule,
  /*  254 */  _AndL_iRegLsrc_uimmL16_rule,
  /*  255 */  0,
  /*  256 */  0,
  /*  257 */  iRegIsrc_rule,
  /*  258 */  iRegIsrc_rule,
  /*  259 */  iRegNsrc_rule,
  /*  260 */  iRegNsrc_rule,
  /*  261 */  iRegP_N2P_rule,
  /*  262 */  iRegPsrc_rule,
  /*  263 */  0,
  /*  264 */  0,
  /*  265 */  regF_rule,
  /*  266 */  0,
  /*  267 */  regD_rule,
  /*  268 */  iRegPdst_rule,
  /*  269 */  iRegPdst_rule,
  /*  270 */  iRegIsrc_rule,
  /*  271 */  iRegLsrc_rule,
  /*  272 */  iRegIsrc_rule,
  /*  273 */  iRegLsrc_rule,
  /*  274 */  0,
  /*  275 */  iRegIsrc_rule,
  /*  276 */  iRegLsrc_rule,
  /*  277 */  0,
  /*  278 */  0,
  /*  279 */  0,
  /*  280 */  0,
  /*  281 */  0,
  /*  282 */  indirectMemory_rule,
  /*  283 */  indirectMemory_rule,
  /*  284 */  indirectMemory_rule,
  /*  285 */  indirectMemory_rule,
  /*  286 */  indOffset16_rule,
  /*  287 */  indOffset16_rule,
  /*  288 */  indOffset16_rule,
  /*  289 */  indOffset16_rule,
  /*  290 */  memory_rule,
  /*  291 */  memory_rule,
  /*  292 */  _LoadUB_memory__rule,
  /*  293 */  _LoadUB_memory__rule,
  /*  294 */  memory_rule,
  /*  295 */  memory_rule,
  /*  296 */  memory_rule,
  /*  297 */  memory_rule,
  /*  298 */  _LoadUS_memory__rule,
  /*  299 */  _LoadUS_memory__rule,
  /*  300 */  memory_rule,
  /*  301 */  memory_rule,
  /*  302 */  _ConvI2L__LoadI_memory___rule,
  /*  303 */  _LoadI_memoryAlg4__rule,
  /*  304 */  _LoadI_memoryAlg4__rule,
  /*  305 */  memoryAlg4_rule,
  /*  306 */  memoryAlg4_rule,
  /*  307 */  memoryAlg4_rule,
  /*  308 */  memoryAlg4_rule,
  /*  309 */  memory_rule,
  /*  310 */  memory_rule,
  /*  311 */  memory_rule,
  /*  312 */  _LoadN_memory__rule,
  /*  313 */  memoryAlg4_rule,
  /*  314 */  memoryAlg4_rule,
  /*  315 */  memory_rule,
  /*  316 */  memoryAlg4_rule,
  /*  317 */  memory_rule,
  /*  318 */  memory_rule,
  /*  319 */  memory_rule,
  /*  320 */  memory_rule,
  /*  321 */  memory_rule,
  /*  322 */  0,
  /*  323 */  0,
  /*  324 */  0,
  /*  325 */  0,
  /*  326 */  0,
  /*  327 */  0,
  /*  328 */  0,
  /*  329 */  0,
  /*  330 */  0,
  /*  331 */  0,
  /*  332 */  0,
  /*  333 */  0,
  /*  334 */  _AddP_indirectMemory_iRegLsrc_rule,
  /*  335 */  indirectMemory_rule,
  /*  336 */  _AddP_indirectMemory_iRegLsrc_rule,
  /*  337 */  indirectMemory_rule,
  /*  338 */  _AddP_indirectMemory_iRegLsrc_rule,
  /*  339 */  indirectMemory_rule,
  /*  340 */  _AddP_indirectMemory_iRegLsrc_rule,
  /*  341 */  indirectMemory_rule,
  /*  342 */  memory_rule,
  /*  343 */  memory_rule,
  /*  344 */  memory_rule,
  /*  345 */  memory_rule,
  /*  346 */  memoryAlg4_rule,
  /*  347 */  memoryAlg4_rule,
  /*  348 */  memory_rule,
  /*  349 */  memoryAlg4_rule,
  /*  350 */  memory_rule,
  /*  351 */  memory_rule,
  /*  352 */  memory_rule,
  /*  353 */  memory_rule,
  /*  354 */  memory_rule,
  /*  355 */  iRegNsrc_rule,
  /*  356 */  iRegPdst_rule,
  /*  357 */  _Binary_flagsReg_iRegPsrc_rule,
  /*  358 */  _Binary_flagsReg_iRegPsrc_rule,
  /*  359 */  iRegPsrc_rule,
  /*  360 */  iRegPsrc_rule,
  /*  361 */  iRegPsrc_rule,
  /*  362 */  iRegPsrc_rule,
  /*  363 */  iRegPsrc_rule,
  /*  364 */  iRegPsrc_rule,
  /*  365 */  iRegPdst_rule,
  /*  366 */  _Binary_flagsReg_iRegPsrc_rule,
  /*  367 */  _Binary_flagsReg_iRegPsrc_rule,
  /*  368 */  iRegNsrc_rule,
  /*  369 */  iRegNsrc_rule,
  /*  370 */  iRegNsrc_rule,
  /*  371 */  iRegNsrc_rule,
  /*  372 */  _CastP2X__DecodeN_iRegNsrc___rule,
  /*  373 */  0,
  /*  374 */  0,
  /*  375 */  0,
  /*  376 */  0,
  /*  377 */  0,
  /*  378 */  0,
  /*  379 */  0,
  /*  380 */  _Binary_cmpOp_flagsReg_rule,
  /*  381 */  _Binary_cmpOp_flagsReg_rule,
  /*  382 */  _Binary_cmpOp_flagsReg_rule,
  /*  383 */  _Binary_cmpOp_flagsReg_rule,
  /*  384 */  _Binary_cmpOp_flagsReg_rule,
  /*  385 */  _Binary_cmpOp_flagsReg_rule,
  /*  386 */  _Binary_cmpOp_flagsReg_rule,
  /*  387 */  _Binary_cmpOp_flagsReg_rule,
  /*  388 */  _Binary_cmpOp_flagsReg_rule,
  /*  389 */  _Binary_cmpOp_flagsReg_rule,
  /*  390 */  _Binary_cmpOp_flagsReg_rule,
  /*  391 */  _Binary_cmpOp_flagsReg_rule,
  /*  392 */  _Binary_cmpOp_flagsReg_rule,
  /*  393 */  _Binary_cmpOp_flagsReg_rule,
  /*  394 */  memory_rule,
  /*  395 */  iRegPdst_rule,
  /*  396 */  iRegPdst_rule,
  /*  397 */  iRegPdst_rule,
  /*  398 */  iRegPdst_rule,
  /*  399 */  iRegPdst_rule,
  /*  400 */  iRegPdst_rule,
  /*  401 */  iRegPdst_rule,
  /*  402 */  iRegPdst_rule,
  /*  403 */  iRegPdst_rule,
  /*  404 */  iRegPdst_rule,
  /*  405 */  iRegIsrc_iRegL2Isrc_rule,
  /*  406 */  0,
  /*  407 */  _AddI__AddI_iRegIsrc_iRegIsrc_iRegIsrc_rule,
  /*  408 */  iRegIsrc_rule,
  /*  409 */  _AddI_iRegIsrc__AddI_iRegIsrc_iRegIsrc_rule,
  /*  410 */  iRegIsrc_rule,
  /*  411 */  iRegIsrc_rule,
  /*  412 */  iRegIsrc_rule,
  /*  413 */  iRegLsrc_rule,
  /*  414 */  0,
  /*  415 */  _AddL__AddL_iRegLsrc_iRegLsrc_iRegLsrc_rule,
  /*  416 */  iRegLsrc_rule,
  /*  417 */  _AddL_iRegLsrc__AddL_iRegLsrc_iRegLsrc_rule,
  /*  418 */  iRegLsrc_rule,
  /*  419 */  _AddL_iRegLsrc_iRegLsrc_rule,
  /*  420 */  iRegLsrc_rule,
  /*  421 */  iRegLsrc_rule,
  /*  422 */  iRegP_N2P_rule,
  /*  423 */  iRegP_N2P_rule,
  /*  424 */  iRegP_N2P_rule,
  /*  425 */  iRegIsrc_rule,
  /*  426 */  iRegIsrc_rule,
  /*  427 */  immI16_rule,
  /*  428 */  immI_0_rule,
  /*  429 */  iRegLsrc_rule,
  /*  430 */  _SubL_iRegLsrc_iRegLsrc_rule,
  /*  431 */  iRegLsrc_rule,
  /*  432 */  immL_0_rule,
  /*  433 */  _SubL_immL_0_iRegLsrc_rule,
  /*  434 */  iRegIsrc_rule,
  /*  435 */  iRegIsrc_rule,
  /*  436 */  iRegLsrc_rule,
  /*  437 */  iRegLsrc_rule,
  /*  438 */  iRegLsrc_rule,
  /*  439 */  iRegIsrc_rule,
  /*  440 */  iRegIsrc_rule,
  /*  441 */  0,
  /*  442 */  iRegIsrc_rule,
  /*  443 */  iRegLsrc_rule,
  /*  444 */  iRegLsrc_rule,
  /*  445 */  0,
  /*  446 */  iRegLsrc_rule,
  /*  447 */  iRegIsrc_rule,
  /*  448 */  iRegLsrc_rule,
  /*  449 */  0,
  /*  450 */  0,
  /*  451 */  iRegIsrc_rule,
  /*  452 */  iRegIsrc_rule,
  /*  453 */  _AndI_iRegIsrc_immInegpow2_rule,
  /*  454 */  _AndI__RShiftI_iRegIsrc_uimmI5_immInegpow2_rule,
  /*  455 */  0,
  /*  456 */  iRegLsrc_rule,
  /*  457 */  iRegLsrc_rule,
  /*  458 */  _ConvI2L_iRegIsrc__rule,
  /*  459 */  _ConvI2L_iRegIsrc__rule,
  /*  460 */  0,
  /*  461 */  iRegIsrc_rule,
  /*  462 */  iRegIsrc_rule,
  /*  463 */  0,
  /*  464 */  iRegLsrc_rule,
  /*  465 */  iRegLsrc_rule,
  /*  466 */  _RShiftL_iRegLsrc_immI_rule,
  /*  467 */  0,
  /*  468 */  iRegIsrc_rule,
  /*  469 */  iRegIsrc_rule,
  /*  470 */  0,
  /*  471 */  iRegLsrc_rule,
  /*  472 */  iRegLsrc_rule,
  /*  473 */  _URShiftL_iRegLsrc_immI_rule,
  /*  474 */  _CastP2X_iRegP_N2P__rule,
  /*  475 */  _ConvI2L_iRegIsrc__rule,
  /*  476 */  _LShiftI_iRegIsrc_immI8_rule,
  /*  477 */  _URShiftI_iRegIsrc_immI8_rule,
  /*  478 */  _URShiftI_iRegIsrc_immI8_rule,
  /*  479 */  _LShiftI_iRegIsrc_immI8_rule,
  /*  480 */  regF_rule,
  /*  481 */  regD_rule,
  /*  482 */  regF_rule,
  /*  483 */  regD_rule,
  /*  484 */  regF_rule,
  /*  485 */  regD_rule,
  /*  486 */  regF_rule,
  /*  487 */  regD_rule,
  /*  488 */  regD_rule,
  /*  489 */  _SqrtD__ConvF2D_regF___rule,
  /*  490 */  iRegIsrc_rule,
  /*  491 */  iRegIsrc_rule,
  /*  492 */  iRegIsrc_rule,
  /*  493 */  iRegIsrc_rule,
  /*  494 */  iRegIsrc_rule,
  /*  495 */  iRegLsrc_rule,
  /*  496 */  iRegLsrc_rule,
  /*  497 */  iRegLsrc_rule,
  /*  498 */  iRegLsrc_rule,
  /*  499 */  _AndL_iRegLsrc_immLpow2minus1_rule,
  /*  500 */  iRegIsrc_rule,
  /*  501 */  0,
  /*  502 */  _OrI__OrI_iRegIsrc_iRegIsrc_iRegIsrc_rule,
  /*  503 */  iRegIsrc_rule,
  /*  504 */  _OrI_iRegIsrc__OrI_iRegIsrc_iRegIsrc_rule,
  /*  505 */  iRegIsrc_rule,
  /*  506 */  iRegIsrc_rule,
  /*  507 */  iRegLsrc_rule,
  /*  508 */  _OrL_iRegLsrc_iRegLsrc_rule,
  /*  509 */  iRegLsrc_rule,
  /*  510 */  iRegIsrc_rule,
  /*  511 */  0,
  /*  512 */  _XorI__XorI_iRegIsrc_iRegIsrc_iRegIsrc_rule,
  /*  513 */  iRegIsrc_rule,
  /*  514 */  _XorI_iRegIsrc__XorI_iRegIsrc_iRegIsrc_rule,
  /*  515 */  iRegIsrc_rule,
  /*  516 */  iRegIsrc_rule,
  /*  517 */  iRegLsrc_rule,
  /*  518 */  _XorL_iRegLsrc_iRegLsrc_rule,
  /*  519 */  iRegLsrc_rule,
  /*  520 */  iRegIsrc_rule,
  /*  521 */  iRegLsrc_rule,
  /*  522 */  _XorI_iRegIsrc_immI_minus1_rule,
  /*  523 */  iRegIsrc_rule,
  /*  524 */  0,
  /*  525 */  regF_rule,
  /*  526 */  iRegIsrc_rule,
  /*  527 */  0,
  /*  528 */  regD_rule,
  /*  529 */  iRegLsrc_rule,
  /*  530 */  iRegIsrc_rule,
  /*  531 */  _Conv2B_iRegIsrc__rule,
  /*  532 */  _Conv2B_iRegIsrc__rule,
  /*  533 */  _AndI_iRegIsrc_immIpowerOf2_rule,
  /*  534 */  iRegP_N2P_rule,
  /*  535 */  _Conv2B_iRegP_N2P__rule,
  /*  536 */  _Conv2B_iRegP_N2P__rule,
  /*  537 */  iRegIsrc_rule,
  /*  538 */  iRegIsrc_rule,
  /*  539 */  _LShiftI_iRegIsrc_immI_24_rule,
  /*  540 */  _LShiftI_iRegIsrc_immI_16_rule,
  /*  541 */  _ConvL2I_iRegLsrc__rule,
  /*  542 */  iRegLsrc_rule,
  /*  543 */  0,
  /*  544 */  0,
  /*  545 */  regD_rule,
  /*  546 */  regF_rule,
  /*  547 */  iRegIsrc_rule,
  /*  548 */  _ConvI2L_iRegIsrc__rule,
  /*  549 */  iRegLsrc_rule,
  /*  550 */  0,
  /*  551 */  0,
  /*  552 */  regF_rule,
  /*  553 */  regD_rule,
  /*  554 */  regD_rule,
  /*  555 */  iRegIsrc_rule,
  /*  556 */  iRegIsrc_rule,
  /*  557 */  iRegLsrc_rule,
  /*  558 */  iRegIsrc_rule,
  /*  559 */  stackSlotL_rule,
  /*  560 */  regF_rule,
  /*  561 */  iRegLsrc_rule,
  /*  562 */  cmpOp_rule,
  /*  563 */  cmpOp_rule,
  /*  564 */  cmpOp_rule,
  /*  565 */  cmpOp_rule,
  /*  566 */  cmpOp_rule,
  /*  567 */  regF_rule,
  /*  568 */  regD_rule,
  /*  569 */  0,
  /*  570 */  cmpOp_rule,
  /*  571 */  cmpOp_rule,
  /*  572 */  cmpOp_rule,
  /*  573 */  cmpOp_rule,
  /*  574 */  cmpOp_rule,
  /*  575 */  cmpOp_rule,
  /*  576 */  iRegP_N2P_rule,
  /*  577 */  _AndL__CastP2X_iRegPsrc__immLnegpow2_rule,
  /*  578 */  _CastP2X_iRegPsrc__rule,
  /*  579 */  rarg1RegL_rule,
  /*  580 */  _Binary_iRegPsrc_iRegIsrc_rule,
  /*  581 */  _Binary_iRegPsrc_iRegIsrc_rule,
  /*  582 */  _Binary_iRegPsrc_rscratch1RegI_rule,
  /*  583 */  _Binary_iRegPsrc_rscratch1RegI_rule,
  /*  584 */  _Binary_iRegPsrc_iRegPsrc_rule,
  /*  585 */  _Binary_iRegPsrc_iRegPsrc_rule,
  /*  586 */  _Binary_rarg1RegP_rarg3RegI_rule,
  /*  587 */  iRegIsrc_rule,
  /*  588 */  iRegIsrc_rule,
  /*  589 */  0,
  /*  590 */  0,
  /*  591 */  iRegIsrc_rule,
  /*  592 */  iRegIsrc_rule,
  /*  593 */  immI_0_rule,
  /*  594 */  immI_minus1_rule,
  /*  595 */  iRegIsrc_rule,
  /*  596 */  immI_0_rule,
  /*  597 */  immI_minus1_rule,
  /*  598 */  iRegIsrc_rule,
  /*  599 */  immI_0_rule,
  /*  600 */  immI_minus1_rule,
  /*  601 */  regF_rule,
  /*  602 */  immF_rule,
  /*  603 */  immF_0_rule,
  /*  604 */  iRegPdst_rule,
  /*  605 */  rscratch2RegP_rule,
  /*  606 */  0,
  /*  607 */  0,
  /*  608 */  0,
  /*  609 */  0,
  /*  610 */  0,
  /*  611 */  0,
  /*  612 */  0,
  /*  613 */  0,
  /*  614 */  0,
  /*  615 */  iRegPdstNoScratch_rule,
  /*  616 */  0,
  /*  617 */  iRegPdstNoScratch_rule,
  /*  618 */  0,
  /*  619 */  0,
  /*  620 */  0,
  /*  621 */  0,
  /*  622 */  0,
  /*  623 */  0,
  /*  624 */  0,
  /*  625 */  0,
  /*  626 */  0,
  /*  627 */  0,
  // last instruction
  0 // no trailing comma
};

const        int   rightOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  0,
  /*   78 */  0,
  /*   79 */  0,
  /*   80 */  0,
  /*   81 */  0,
  /*   82 */  0,
  /*   83 */  0,
  /*   84 */  0,
  /*   85 */  0,
  /*   86 */  0,
  /*   87 */  0,
  /*   88 */  0,
  /*   89 */  immL16_rule,
  /*   90 */  immL16Alg4_rule,
  /*   91 */  0,
  /*   92 */  immL16_rule,
  /*   93 */  immL16Alg4_rule,
  /*   94 */  0,
  /*   95 */  0,
  /*   96 */  0,
  /*   97 */  0,
  /*   98 */  0,
  /*   99 */  0,
  // last operand
  /*  100 */  0,
  /*  101 */  0,
  /*  102 */  0,
  /*  103 */  0,
  /*  104 */  0,
  /*  105 */  0,
  // last operand class
  /*  106 */  0,
  /*  107 */  0,
  /*  108 */  0,
  /*  109 */  0,
  /*  110 */  0,
  /*  111 */  0,
  /*  112 */  0,
  /*  113 */  0,
  /*  114 */  iRegLsrc_rule,
  /*  115 */  0,
  /*  116 */  iRegPsrc_rule,
  /*  117 */  0,
  /*  118 */  flagsReg_rule,
  /*  119 */  iRegIsrc_rule,
  /*  120 */  immI16_rule,
  /*  121 */  iRegLsrc_rule,
  /*  122 */  immL16_rule,
  /*  123 */  iRegNsrc_rule,
  /*  124 */  immN_0_rule,
  /*  125 */  iRegPsrc_rule,
  /*  126 */  iRegP_N2P_rule,
  /*  127 */  immP_0_rule,
  /*  128 */  regF_rule,
  /*  129 */  regD_rule,
  /*  130 */  iRegLsrc_rule,
  /*  131 */  iRegPsrc_rule,
  /*  132 */  iRegIsrc_rule,
  /*  133 */  iRegNsrc_rule,
  /*  134 */  iRegIsrc_rule,
  /*  135 */  iRegIsrc_rule,
  /*  136 */  _AddI_iRegIsrc_iRegIsrc_rule,
  /*  137 */  iRegLsrc_rule,
  /*  138 */  iRegLsrc_rule,
  /*  139 */  _AddL_iRegLsrc_iRegLsrc_rule,
  /*  140 */  iRegLsrc_rule,
  /*  141 */  iRegLsrc_rule,
  /*  142 */  immInegpow2_rule,
  /*  143 */  uimmI5_rule,
  /*  144 */  immInegpow2_rule,
  /*  145 */  0,
  /*  146 */  immI_rule,
  /*  147 */  immI_rule,
  /*  148 */  0,
  /*  149 */  immI8_rule,
  /*  150 */  immI8_rule,
  /*  151 */  0,
  /*  152 */  0,
  /*  153 */  0,
  /*  154 */  0,
  /*  155 */  immLpow2minus1_rule,
  /*  156 */  iRegIsrc_rule,
  /*  157 */  iRegIsrc_rule,
  /*  158 */  _OrI_iRegIsrc_iRegIsrc_rule,
  /*  159 */  iRegLsrc_rule,
  /*  160 */  iRegIsrc_rule,
  /*  161 */  iRegIsrc_rule,
  /*  162 */  _XorI_iRegIsrc_iRegIsrc_rule,
  /*  163 */  iRegLsrc_rule,
  /*  164 */  immI_minus1_rule,
  /*  165 */  0,
  /*  166 */  immIpowerOf2_rule,
  /*  167 */  0,
  /*  168 */  immI_24_rule,
  /*  169 */  immI_16_rule,
  /*  170 */  uimmI16_rule,
  /*  171 */  iRegLsrc_rule,
  /*  172 */  uimmL16_rule,
  /*  173 */  uimmI15_rule,
  /*  174 */  iRegIsrc_rule,
  /*  175 */  immN_0_rule,
  /*  176 */  immP_0_rule,
  /*  177 */  0,
  /*  178 */  immLnegpow2_rule,
  /*  179 */  iRegIsrc_rule,
  /*  180 */  immL_rule,
  /*  181 */  immI_1_rule,
  /*  182 */  immI_1_rule,
  /*  183 */  rscratch1RegI_rule,
  /*  184 */  uimmI15_rule,
  /*  185 */  rscratch2RegI_rule,
  /*  186 */  rarg3RegI_rule,
  /*  187 */  rarg4RegI_rule,
  // last internally defined operand
  /*  188 */  0,
  /*  189 */  0,
  /*  190 */  0,
  /*  191 */  0,
  /*  192 */  0,
  /*  193 */  0,
  /*  194 */  0,
  /*  195 */  0,
  /*  196 */  0,
  /*  197 */  0,
  /*  198 */  0,
  /*  199 */  0,
  /*  200 */  0,
  /*  201 */  0,
  /*  202 */  0,
  /*  203 */  0,
  /*  204 */  0,
  /*  205 */  0,
  /*  206 */  0,
  /*  207 */  0,
  /*  208 */  0,
  /*  209 */  0,
  /*  210 */  0,
  /*  211 */  0,
  /*  212 */  0,
  /*  213 */  0,
  /*  214 */  _Binary_iRegLsrc_iRegLsrc_rule,
  /*  215 */  _Binary_iRegPsrc_iRegPsrc_rule,
  /*  216 */  0,
  /*  217 */  0,
  /*  218 */  0,
  /*  219 */  0,
  /*  220 */  0,
  /*  221 */  0,
  /*  222 */  0,
  /*  223 */  0,
  /*  224 */  0,
  /*  225 */  0,
  /*  226 */  0,
  /*  227 */  0,
  /*  228 */  0,
  /*  229 */  0,
  /*  230 */  0,
  /*  231 */  0,
  /*  232 */  0,
  /*  233 */  0,
  /*  234 */  0,
  /*  235 */  0,
  /*  236 */  0,
  /*  237 */  0,
  /*  238 */  0,
  /*  239 */  0,
  /*  240 */  0,
  /*  241 */  0,
  /*  242 */  0,
  /*  243 */  0,
  /*  244 */  0,
  /*  245 */  0,
  /*  246 */  iRegIsrc_rule,
  /*  247 */  immI16_rule,
  /*  248 */  immI_0_rule,
  /*  249 */  iRegLsrc_rule,
  /*  250 */  immL16_rule,
  /*  251 */  iRegLsrc_rule,
  /*  252 */  uimmL16_rule,
  /*  253 */  immL_0_rule,
  /*  254 */  immL_0_rule,
  /*  255 */  0,
  /*  256 */  0,
  /*  257 */  iRegIsrc_rule,
  /*  258 */  uimmI16_rule,
  /*  259 */  iRegNsrc_rule,
  /*  260 */  immN_0_rule,
  /*  261 */  iRegP_N2P_rule,
  /*  262 */  immL16_rule,
  /*  263 */  0,
  /*  264 */  0,
  /*  265 */  regF_rule,
  /*  266 */  0,
  /*  267 */  regD_rule,
  /*  268 */  iRegPdst_rule,
  /*  269 */  iRegPdst_rule,
  /*  270 */  0,
  /*  271 */  0,
  /*  272 */  0,
  /*  273 */  0,
  /*  274 */  0,
  /*  275 */  0,
  /*  276 */  0,
  /*  277 */  0,
  /*  278 */  0,
  /*  279 */  0,
  /*  280 */  0,
  /*  281 */  0,
  /*  282 */  0,
  /*  283 */  0,
  /*  284 */  0,
  /*  285 */  0,
  /*  286 */  0,
  /*  287 */  0,
  /*  288 */  0,
  /*  289 */  0,
  /*  290 */  0,
  /*  291 */  0,
  /*  292 */  0,
  /*  293 */  0,
  /*  294 */  0,
  /*  295 */  0,
  /*  296 */  0,
  /*  297 */  0,
  /*  298 */  0,
  /*  299 */  0,
  /*  300 */  0,
  /*  301 */  0,
  /*  302 */  immL_32bits_rule,
  /*  303 */  0,
  /*  304 */  0,
  /*  305 */  0,
  /*  306 */  0,
  /*  307 */  0,
  /*  308 */  0,
  /*  309 */  0,
  /*  310 */  0,
  /*  311 */  0,
  /*  312 */  0,
  /*  313 */  0,
  /*  314 */  0,
  /*  315 */  0,
  /*  316 */  0,
  /*  317 */  0,
  /*  318 */  0,
  /*  319 */  0,
  /*  320 */  0,
  /*  321 */  0,
  /*  322 */  0,
  /*  323 */  0,
  /*  324 */  0,
  /*  325 */  0,
  /*  326 */  0,
  /*  327 */  0,
  /*  328 */  0,
  /*  329 */  0,
  /*  330 */  0,
  /*  331 */  0,
  /*  332 */  0,
  /*  333 */  0,
  /*  334 */  0,
  /*  335 */  0,
  /*  336 */  0,
  /*  337 */  0,
  /*  338 */  0,
  /*  339 */  0,
  /*  340 */  0,
  /*  341 */  0,
  /*  342 */  iRegIsrc_rule,
  /*  343 */  iRegIsrc_rule,
  /*  344 */  iRegIsrc_rule,
  /*  345 */  _ConvL2I_iRegLsrc__rule,
  /*  346 */  iRegLsrc_rule,
  /*  347 */  iRegLsrc_rule,
  /*  348 */  iRegN_P2N_rule,
  /*  349 */  iRegPsrc_rule,
  /*  350 */  regF_rule,
  /*  351 */  regD_rule,
  /*  352 */  iRegLdst_rule,
  /*  353 */  immI_0_rule,
  /*  354 */  immI_0_rule,
  /*  355 */  0,
  /*  356 */  0,
  /*  357 */  0,
  /*  358 */  0,
  /*  359 */  0,
  /*  360 */  0,
  /*  361 */  0,
  /*  362 */  0,
  /*  363 */  0,
  /*  364 */  0,
  /*  365 */  0,
  /*  366 */  0,
  /*  367 */  0,
  /*  368 */  0,
  /*  369 */  0,
  /*  370 */  0,
  /*  371 */  0,
  /*  372 */  0,
  /*  373 */  0,
  /*  374 */  0,
  /*  375 */  0,
  /*  376 */  0,
  /*  377 */  0,
  /*  378 */  0,
  /*  379 */  0,
  /*  380 */  _Binary_iRegIdst_iRegIsrc_rule,
  /*  381 */  _Binary_iRegIdst_iRegIsrc_rule,
  /*  382 */  _Binary_iRegIdst_immI16_rule,
  /*  383 */  _Binary_iRegLdst_iRegLsrc_rule,
  /*  384 */  _Binary_iRegLdst_iRegLsrc_rule,
  /*  385 */  _Binary_iRegLdst_immL16_rule,
  /*  386 */  _Binary_iRegNdst_iRegNsrc_rule,
  /*  387 */  _Binary_iRegNdst_iRegNsrc_rule,
  /*  388 */  _Binary_iRegNdst_immN_0_rule,
  /*  389 */  _Binary_iRegPdst_iRegPsrc_rule,
  /*  390 */  _Binary_iRegPdst_iRegP_N2P_rule,
  /*  391 */  _Binary_iRegPdst_immP_0_rule,
  /*  392 */  _Binary_regF_regF_rule,
  /*  393 */  _Binary_regD_regD_rule,
  /*  394 */  0,
  /*  395 */  _Binary_iRegIsrc_iRegIsrc_rule,
  /*  396 */  _Binary_iRegNsrc_iRegNsrc_rule,
  /*  397 */  _Binary_iRegLsrc_iRegLsrc_rule,
  /*  398 */  _Binary_iRegPsrc_iRegPsrc_rule,
  /*  399 */  iRegIsrc_rule,
  /*  400 */  iRegLsrc_rule,
  /*  401 */  iRegIsrc_rule,
  /*  402 */  iRegLsrc_rule,
  /*  403 */  iRegPsrc_rule,
  /*  404 */  iRegNsrc_rule,
  /*  405 */  iRegIsrc_iRegL2Isrc_rule,
  /*  406 */  0,
  /*  407 */  iRegIsrc_rule,
  /*  408 */  _AddI__AddI_iRegIsrc_iRegIsrc_iRegIsrc_rule,
  /*  409 */  iRegIsrc_rule,
  /*  410 */  _AddI_iRegIsrc__AddI_iRegIsrc_iRegIsrc_rule,
  /*  411 */  immI16_rule,
  /*  412 */  immIhi16_rule,
  /*  413 */  iRegLsrc_rule,
  /*  414 */  0,
  /*  415 */  iRegLsrc_rule,
  /*  416 */  _AddL__AddL_iRegLsrc_iRegLsrc_iRegLsrc_rule,
  /*  417 */  iRegLsrc_rule,
  /*  418 */  _AddL_iRegLsrc__AddL_iRegLsrc_iRegLsrc_rule,
  /*  419 */  0,
  /*  420 */  immL16_rule,
  /*  421 */  immL32hi16_rule,
  /*  422 */  iRegLsrc_rule,
  /*  423 */  immL16_rule,
  /*  424 */  immL32hi16_rule,
  /*  425 */  iRegIsrc_rule,
  /*  426 */  immI16_rule,
  /*  427 */  iRegIsrc_rule,
  /*  428 */  iRegIsrc_rule,
  /*  429 */  iRegLsrc_rule,
  /*  430 */  0,
  /*  431 */  immL16_rule,
  /*  432 */  iRegLsrc_rule,
  /*  433 */  0,
  /*  434 */  iRegIsrc_rule,
  /*  435 */  immI16_rule,
  /*  436 */  iRegLsrc_rule,
  /*  437 */  iRegLsrc_rule,
  /*  438 */  immL16_rule,
  /*  439 */  immI_minus1_rule,
  /*  440 */  iRegIsrc_rule,
  /*  441 */  0,
  /*  442 */  iRegIsrc_rule,
  /*  443 */  immL_minus1_rule,
  /*  444 */  iRegLsrc_rule,
  /*  445 */  0,
  /*  446 */  iRegLsrc_rule,
  /*  447 */  iRegIsrc_rule,
  /*  448 */  iRegLsrc_rule,
  /*  449 */  0,
  /*  450 */  0,
  /*  451 */  iRegIsrc_rule,
  /*  452 */  immI_rule,
  /*  453 */  uimmI5_rule,
  /*  454 */  uimmI5_rule,
  /*  455 */  0,
  /*  456 */  iRegIsrc_rule,
  /*  457 */  immI_rule,
  /*  458 */  uimmI6_ge32_rule,
  /*  459 */  uimmI6_rule,
  /*  460 */  0,
  /*  461 */  iRegIsrc_rule,
  /*  462 */  immI_rule,
  /*  463 */  0,
  /*  464 */  iRegIsrc_rule,
  /*  465 */  immI_rule,
  /*  466 */  0,
  /*  467 */  0,
  /*  468 */  iRegIsrc_rule,
  /*  469 */  immI_rule,
  /*  470 */  0,
  /*  471 */  iRegIsrc_rule,
  /*  472 */  immI_rule,
  /*  473 */  0,
  /*  474 */  uimmI6_rule,
  /*  475 */  0,
  /*  476 */  _URShiftI_iRegIsrc_immI8_rule,
  /*  477 */  _LShiftI_iRegIsrc_immI8_rule,
  /*  478 */  _LShiftI_iRegIsrc_immI8_rule,
  /*  479 */  _URShiftI_iRegIsrc_immI8_rule,
  /*  480 */  regF_rule,
  /*  481 */  regD_rule,
  /*  482 */  regF_rule,
  /*  483 */  regD_rule,
  /*  484 */  regF_rule,
  /*  485 */  regD_rule,
  /*  486 */  regF_rule,
  /*  487 */  regD_rule,
  /*  488 */  0,
  /*  489 */  0,
  /*  490 */  iRegIsrc_rule,
  /*  491 */  uimmI16_rule,
  /*  492 */  immInegpow2_rule,
  /*  493 */  immIpow2minus1_rule,
  /*  494 */  immIpowerOf2_rule,
  /*  495 */  iRegLsrc_rule,
  /*  496 */  uimmL16_rule,
  /*  497 */  immLnegpow2_rule,
  /*  498 */  immLpow2minus1_rule,
  /*  499 */  0,
  /*  500 */  iRegIsrc_rule,
  /*  501 */  0,
  /*  502 */  iRegIsrc_rule,
  /*  503 */  _OrI__OrI_iRegIsrc_iRegIsrc_iRegIsrc_rule,
  /*  504 */  iRegIsrc_rule,
  /*  505 */  _OrI_iRegIsrc__OrI_iRegIsrc_iRegIsrc_rule,
  /*  506 */  uimmI16_rule,
  /*  507 */  iRegLsrc_rule,
  /*  508 */  0,
  /*  509 */  uimmL16_rule,
  /*  510 */  iRegIsrc_rule,
  /*  511 */  0,
  /*  512 */  iRegIsrc_rule,
  /*  513 */  _XorI__XorI_iRegIsrc_iRegIsrc_iRegIsrc_rule,
  /*  514 */  iRegIsrc_rule,
  /*  515 */  _XorI_iRegIsrc__XorI_iRegIsrc_iRegIsrc_rule,
  /*  516 */  uimmI16_rule,
  /*  517 */  iRegLsrc_rule,
  /*  518 */  0,
  /*  519 */  uimmL16_rule,
  /*  520 */  immI_minus1_rule,
  /*  521 */  immL_minus1_rule,
  /*  522 */  iRegIsrc_rule,
  /*  523 */  _XorI_iRegIsrc_immI_minus1_rule,
  /*  524 */  0,
  /*  525 */  0,
  /*  526 */  0,
  /*  527 */  0,
  /*  528 */  0,
  /*  529 */  0,
  /*  530 */  0,
  /*  531 */  immI_1_rule,
  /*  532 */  immI_1_rule,
  /*  533 */  0,
  /*  534 */  0,
  /*  535 */  immI_1_rule,
  /*  536 */  immI_1_rule,
  /*  537 */  iRegIsrc_rule,
  /*  538 */  immI_0_rule,
  /*  539 */  immI_24_rule,
  /*  540 */  immI_16_rule,
  /*  541 */  0,
  /*  542 */  0,
  /*  543 */  0,
  /*  544 */  0,
  /*  545 */  0,
  /*  546 */  0,
  /*  547 */  0,
  /*  548 */  immL_32bits_rule,
  /*  549 */  immL_32bits_rule,
  /*  550 */  0,
  /*  551 */  0,
  /*  552 */  0,
  /*  553 */  0,
  /*  554 */  0,
  /*  555 */  0,
  /*  556 */  0,
  /*  557 */  0,
  /*  558 */  0,
  /*  559 */  0,
  /*  560 */  0,
  /*  561 */  iRegLsrc_rule,
  /*  562 */  _CmpU_iRegIsrc_uimmI15_rule,
  /*  563 */  _CmpU_iRegIsrc_iRegIsrc_rule,
  /*  564 */  _CmpU_iRegIsrc_uimmI15_rule,
  /*  565 */  _CmpN_iRegNsrc_immN_0_rule,
  /*  566 */  _CmpP_iRegP_N2P_immP_0_rule,
  /*  567 */  regF_rule,
  /*  568 */  regD_rule,
  /*  569 */  0,
  /*  570 */  flagsReg_rule,
  /*  571 */  flagsReg_rule,
  /*  572 */  flagsReg_rule,
  /*  573 */  flagsReg_rule,
  /*  574 */  flagsReg_rule,
  /*  575 */  flagsReg_rule,
  /*  576 */  iRegP_N2P_rule,
  /*  577 */  0,
  /*  578 */  _CastP2X_iRegPsrc__rule,
  /*  579 */  rarg2RegP_rule,
  /*  580 */  _Binary__AddP_immP_immL_immI_1_rule,
  /*  581 */  _Binary_rscratch2RegP_immI_1_rule,
  /*  582 */  _Binary_iRegPsrc_uimmI15_rule,
  /*  583 */  _Binary_iRegPsrc_rscratch2RegI_rule,
  /*  584 */  uimmI15_rule,
  /*  585 */  iRegIsrc_rule,
  /*  586 */  _Binary_rarg2RegP_rarg4RegI_rule,
  /*  587 */  iRegIsrc_rule,
  /*  588 */  iRegIsrc_rule,
  /*  589 */  0,
  /*  590 */  0,
  /*  591 */  0,
  /*  592 */  0,
  /*  593 */  0,
  /*  594 */  0,
  /*  595 */  0,
  /*  596 */  0,
  /*  597 */  0,
  /*  598 */  0,
  /*  599 */  0,
  /*  600 */  0,
  /*  601 */  0,
  /*  602 */  0,
  /*  603 */  0,
  /*  604 */  0,
  /*  605 */  0,
  /*  606 */  0,
  /*  607 */  0,
  /*  608 */  0,
  /*  609 */  0,
  /*  610 */  0,
  /*  611 */  0,
  /*  612 */  0,
  /*  613 */  0,
  /*  614 */  0,
  /*  615 */  inline_cache_regP_rule,
  /*  616 */  0,
  /*  617 */  rarg1RegP_rule,
  /*  618 */  0,
  /*  619 */  0,
  /*  620 */  0,
  /*  621 */  0,
  /*  622 */  0,
  /*  623 */  0,
  /*  624 */  0,
  /*  625 */  0,
  /*  626 */  0,
  /*  627 */  0,
  // last instruction
  0 // no trailing comma
};

const char        *ruleName[] = {
  /*    0 */  "UNIVERSE",
  /*    1 */  "LABEL",
  /*    2 */  "SREGI",
  /*    3 */  "SREGP",
  /*    4 */  "SREGF",
  /*    5 */  "SREGD",
  /*    6 */  "SREGL",
  /*    7 */  "METHOD",
  /*    8 */  "IMMI",
  /*    9 */  "IMMI8",
  /*   10 */  "IMMI16",
  /*   11 */  "IMMIHI16",
  /*   12 */  "IMMINEGPOW2",
  /*   13 */  "IMMIPOW2MINUS1",
  /*   14 */  "IMMIPOWEROF2",
  /*   15 */  "UIMMI5",
  /*   16 */  "UIMMI6",
  /*   17 */  "UIMMI6_GE32",
  /*   18 */  "UIMMI15",
  /*   19 */  "UIMMI16",
  /*   20 */  "IMMI_0",
  /*   21 */  "IMMI_1",
  /*   22 */  "IMMI_MINUS1",
  /*   23 */  "IMMI_16",
  /*   24 */  "IMMI_24",
  /*   25 */  "IMMN",
  /*   26 */  "IMMN_0",
  /*   27 */  "IMMP",
  /*   28 */  "IMMP_NM",
  /*   29 */  "IMMP_0",
  /*   30 */  "IMMP_0OR1",
  /*   31 */  "IMMP_POLL",
  /*   32 */  "IMML",
  /*   33 */  "IMML16",
  /*   34 */  "IMML16ALG4",
  /*   35 */  "IMML32HI16",
  /*   36 */  "IMML32",
  /*   37 */  "IMMLHIGHEST16",
  /*   38 */  "IMMLNEGPOW2",
  /*   39 */  "IMMLPOW2MINUS1",
  /*   40 */  "IMML_0",
  /*   41 */  "IMML_MINUS1",
  /*   42 */  "IMML_32BITS",
  /*   43 */  "UIMML16",
  /*   44 */  "IMMF",
  /*   45 */  "IMMF_0",
  /*   46 */  "IMMD",
  /*   47 */  "IREGIDST",
  /*   48 */  "IREGISRC",
  /*   49 */  "RSCRATCH1REGI",
  /*   50 */  "RSCRATCH2REGI",
  /*   51 */  "RARG1REGI",
  /*   52 */  "RARG2REGI",
  /*   53 */  "RARG3REGI",
  /*   54 */  "RARG4REGI",
  /*   55 */  "RARG1REGL",
  /*   56 */  "RARG2REGL",
  /*   57 */  "RARG3REGL",
  /*   58 */  "RARG4REGL",
  /*   59 */  "IREGPDST",
  /*   60 */  "IREGPDSTNOSCRATCH",
  /*   61 */  "IREGPSRC",
  /*   62 */  "THREADREGP",
  /*   63 */  "RSCRATCH1REGP",
  /*   64 */  "RSCRATCH2REGP",
  /*   65 */  "RARG1REGP",
  /*   66 */  "RARG2REGP",
  /*   67 */  "RARG3REGP",
  /*   68 */  "RARG4REGP",
  /*   69 */  "IREGNSRC",
  /*   70 */  "IREGNDST",
  /*   71 */  "IREGLDST",
  /*   72 */  "IREGLSRC",
  /*   73 */  "IREGL2ISRC",
  /*   74 */  "RSCRATCH1REGL",
  /*   75 */  "RSCRATCH2REGL",
  /*   76 */  "FLAGSREG",
  /*   77 */  "FLAGSREGCR0",
  /*   78 */  "FLAGSREGCR1",
  /*   79 */  "FLAGSREGCR6",
  /*   80 */  "REGCTR",
  /*   81 */  "REGD",
  /*   82 */  "REGF",
  /*   83 */  "INLINE_CACHE_REGP",
  /*   84 */  "COMPILER_METHOD_OOP_REGP",
  /*   85 */  "INTERPRETER_METHOD_OOP_REGP",
  /*   86 */  "IREGP2N",
  /*   87 */  "IREGN2P",
  /*   88 */  "INDIRECT",
  /*   89 */  "INDOFFSET16",
  /*   90 */  "INDOFFSET16ALG4",
  /*   91 */  "INDIRECTNARROW",
  /*   92 */  "INDOFFSET16NARROW",
  /*   93 */  "INDOFFSET16NARROWALG4",
  /*   94 */  "STACKSLOTI",
  /*   95 */  "STACKSLOTL",
  /*   96 */  "STACKSLOTP",
  /*   97 */  "STACKSLOTF",
  /*   98 */  "STACKSLOTD",
  /*   99 */  "CMPOP",
  // last operand
  /*  100 */  "MEMORY",
  /*  101 */  "MEMORYALG4",
  /*  102 */  "INDIRECTMEMORY",
  /*  103 */  "IREGISRC_IREGL2ISRC",
  /*  104 */  "IREGN_P2N",
  /*  105 */  "IREGP_N2P",
  // last operand class
  /*  106 */  "_DecodeN_iRegNsrc_",
  /*  107 */  "_LoadUB_memory_",
  /*  108 */  "_LoadUS_memory_",
  /*  109 */  "_LoadI_memory_",
  /*  110 */  "_ConvI2L__LoadI_memory__",
  /*  111 */  "_LoadI_memoryAlg4_",
  /*  112 */  "_LoadN_memory_",
  /*  113 */  "_LoadP_memoryAlg4_",
  /*  114 */  "_AddP_indirectMemory_iRegLsrc",
  /*  115 */  "_ConvL2I_iRegLsrc_",
  /*  116 */  "_Binary_flagsReg_iRegPsrc",
  /*  117 */  "_CastP2X__DecodeN_iRegNsrc__",
  /*  118 */  "_Binary_cmpOp_flagsReg",
  /*  119 */  "_Binary_iRegIdst_iRegIsrc",
  /*  120 */  "_Binary_iRegIdst_immI16",
  /*  121 */  "_Binary_iRegLdst_iRegLsrc",
  /*  122 */  "_Binary_iRegLdst_immL16",
  /*  123 */  "_Binary_iRegNdst_iRegNsrc",
  /*  124 */  "_Binary_iRegNdst_immN_0",
  /*  125 */  "_Binary_iRegPdst_iRegPsrc",
  /*  126 */  "_Binary_iRegPdst_iRegP_N2P",
  /*  127 */  "_Binary_iRegPdst_immP_0",
  /*  128 */  "_Binary_regF_regF",
  /*  129 */  "_Binary_regD_regD",
  /*  130 */  "_Binary_iRegLsrc_iRegLsrc",
  /*  131 */  "_Binary_iRegPsrc_iRegPsrc",
  /*  132 */  "_Binary_iRegIsrc_iRegIsrc",
  /*  133 */  "_Binary_iRegNsrc_iRegNsrc",
  /*  134 */  "_AddI_iRegIsrc_iRegIsrc",
  /*  135 */  "_AddI__AddI_iRegIsrc_iRegIsrc_iRegIsrc",
  /*  136 */  "_AddI_iRegIsrc__AddI_iRegIsrc_iRegIsrc",
  /*  137 */  "_AddL_iRegLsrc_iRegLsrc",
  /*  138 */  "_AddL__AddL_iRegLsrc_iRegLsrc_iRegLsrc",
  /*  139 */  "_AddL_iRegLsrc__AddL_iRegLsrc_iRegLsrc",
  /*  140 */  "_SubL_iRegLsrc_iRegLsrc",
  /*  141 */  "_SubL_immL_0_iRegLsrc",
  /*  142 */  "_AndI_iRegIsrc_immInegpow2",
  /*  143 */  "_RShiftI_iRegIsrc_uimmI5",
  /*  144 */  "_AndI__RShiftI_iRegIsrc_uimmI5_immInegpow2",
  /*  145 */  "_ConvI2L_iRegIsrc_",
  /*  146 */  "_RShiftL_iRegLsrc_immI",
  /*  147 */  "_URShiftL_iRegLsrc_immI",
  /*  148 */  "_CastP2X_iRegP_N2P_",
  /*  149 */  "_LShiftI_iRegIsrc_immI8",
  /*  150 */  "_URShiftI_iRegIsrc_immI8",
  /*  151 */  "_AbsF_regF_",
  /*  152 */  "_AbsD_regD_",
  /*  153 */  "_ConvF2D_regF_",
  /*  154 */  "_SqrtD__ConvF2D_regF__",
  /*  155 */  "_AndL_iRegLsrc_immLpow2minus1",
  /*  156 */  "_OrI_iRegIsrc_iRegIsrc",
  /*  157 */  "_OrI__OrI_iRegIsrc_iRegIsrc_iRegIsrc",
  /*  158 */  "_OrI_iRegIsrc__OrI_iRegIsrc_iRegIsrc",
  /*  159 */  "_OrL_iRegLsrc_iRegLsrc",
  /*  160 */  "_XorI_iRegIsrc_iRegIsrc",
  /*  161 */  "_XorI__XorI_iRegIsrc_iRegIsrc_iRegIsrc",
  /*  162 */  "_XorI_iRegIsrc__XorI_iRegIsrc_iRegIsrc",
  /*  163 */  "_XorL_iRegLsrc_iRegLsrc",
  /*  164 */  "_XorI_iRegIsrc_immI_minus1",
  /*  165 */  "_Conv2B_iRegIsrc_",
  /*  166 */  "_AndI_iRegIsrc_immIpowerOf2",
  /*  167 */  "_Conv2B_iRegP_N2P_",
  /*  168 */  "_LShiftI_iRegIsrc_immI_24",
  /*  169 */  "_LShiftI_iRegIsrc_immI_16",
  /*  170 */  "_AndI_iRegIsrc_uimmI16",
  /*  171 */  "_AndL_iRegLsrc_iRegLsrc",
  /*  172 */  "_AndL_iRegLsrc_uimmL16",
  /*  173 */  "_CmpU_iRegIsrc_uimmI15",
  /*  174 */  "_CmpU_iRegIsrc_iRegIsrc",
  /*  175 */  "_CmpN_iRegNsrc_immN_0",
  /*  176 */  "_CmpP_iRegP_N2P_immP_0",
  /*  177 */  "_CastP2X_iRegPsrc_",
  /*  178 */  "_AndL__CastP2X_iRegPsrc__immLnegpow2",
  /*  179 */  "_Binary_iRegPsrc_iRegIsrc",
  /*  180 */  "_AddP_immP_immL",
  /*  181 */  "_Binary__AddP_immP_immL_immI_1",
  /*  182 */  "_Binary_rscratch2RegP_immI_1",
  /*  183 */  "_Binary_iRegPsrc_rscratch1RegI",
  /*  184 */  "_Binary_iRegPsrc_uimmI15",
  /*  185 */  "_Binary_iRegPsrc_rscratch2RegI",
  /*  186 */  "_Binary_rarg1RegP_rarg3RegI",
  /*  187 */  "_Binary_rarg2RegP_rarg4RegI",
  // last internally defined operand
  /*  188 */  "loadConP_Ex",
  /*  189 */  "loadConF_Ex",
  /*  190 */  "loadConD_Ex",
  /*  191 */  "regI_to_stkI",
  /*  192 */  "regL_to_stkL",
  /*  193 */  "loadConI16",
  /*  194 */  "loadConIhi16",
  /*  195 */  "loadConI_Ex",
  /*  196 */  "loadConL16",
  /*  197 */  "loadConL32hi16",
  /*  198 */  "loadConL32_Ex",
  /*  199 */  "loadConLhighest16_Ex",
  /*  200 */  "loadConL_Ex",
  /*  201 */  "loadConN0",
  /*  202 */  "loadConN_Ex",
  /*  203 */  "loadConP0or1",
  /*  204 */  "loadConP",
  /*  205 */  "loadConP_lo",
  /*  206 */  "stkI_to_regI",
  /*  207 */  "stkL_to_regL",
  /*  208 */  "loadConPollAddr",
  /*  209 */  "convB2I_reg_2",
  /*  210 */  "loadP2X",
  /*  211 */  "loadToc_lo",
  /*  212 */  "loadConN1",
  /*  213 */  "clearMs32b",
  /*  214 */  "storeLConditional_regP_regL_regL",
  /*  215 */  "storePConditional_regP_regP_regP",
  /*  216 */  "signmask32I_regI",
  /*  217 */  "absI_reg_Ex",
  /*  218 */  "signmask64I_regL",
  /*  219 */  "signmask64L_regL",
  /*  220 */  "absF_reg",
  /*  221 */  "absD_reg",
  /*  222 */  "negF_reg",
  /*  223 */  "negD_reg",
  /*  224 */  "negF_absF_reg",
  /*  225 */  "negD_absD_reg",
  /*  226 */  "roundDouble_nop",
  /*  227 */  "roundFloat_nop",
  /*  228 */  "moveF2I_stack_reg",
  /*  229 */  "moveI2F_stack_reg",
  /*  230 */  "moveD2L_stack_reg",
  /*  231 */  "moveL2D_stack_reg",
  /*  232 */  "moveReg",
  /*  233 */  "castX2P",
  /*  234 */  "castP2X",
  /*  235 */  "castPP",
  /*  236 */  "castII",
  /*  237 */  "checkCastPP",
  /*  238 */  "convI2Bool_reg__cntlz_Ex",
  /*  239 */  "convP2Bool_reg__cntlz_Ex",
  /*  240 */  "convD2IRaw_regD",
  /*  241 */  "convF2IRaw_regF",
  /*  242 */  "convF2LRaw_regF",
  /*  243 */  "convD2LRaw_regD",
  /*  244 */  "convL2DRaw_regD",
  /*  245 */  "convL2FRaw_regF",
  /*  246 */  "cmpI_reg_reg",
  /*  247 */  "cmpI_reg_imm16",
  /*  248 */  "testI_reg_imm",
  /*  249 */  "cmpL_reg_reg",
  /*  250 */  "cmpL_reg_imm16",
  /*  251 */  "cmpUL_reg_reg",
  /*  252 */  "cmpUL_reg_imm16",
  /*  253 */  "testL_reg_reg",
  /*  254 */  "testL_reg_imm",
  /*  255 */  "cmovI_conIvalueMinus1_conIvalue1",
  /*  256 */  "cmovI_conIvalueMinus1_conIvalue0_conIvalue1_Ex",
  /*  257 */  "compU_reg_reg",
  /*  258 */  "compU_reg_uimm16",
  /*  259 */  "cmpN_reg_reg",
  /*  260 */  "cmpN_reg_imm0",
  /*  261 */  "cmpP_reg_reg",
  /*  262 */  "cmpP_reg_imm16",
  /*  263 */  "cmpFUnordered_reg_reg",
  /*  264 */  "cmov_bns_less",
  /*  265 */  "cmpF_reg_reg_Ex",
  /*  266 */  "cmpDUnordered_reg_reg",
  /*  267 */  "cmpD_reg_reg_Ex",
  /*  268 */  "cmpFastLock",
  /*  269 */  "cmpFastUnlock",
  /*  270 */  "popCountI",
  /*  271 */  "popCountL",
  /*  272 */  "countLeadingZerosI",
  /*  273 */  "countLeadingZerosL",
  /*  274 */  "countLeadingZerosP",
  /*  275 */  "countTrailingZerosI_Ex",
  /*  276 */  "countTrailingZerosL_Ex",
  /*  277 */  "repl32",
  /*  278 */  "repl48",
  /*  279 */  "repl56",
  /*  280 */  "CallLeafDirect_mtctr",
  /*  281 */  "tlsLoadP",
  /*  282 */  "loadUB_indirect",
  /*  283 */  "loadUB_indirect_ac",
  /*  284 */  "loadB_indirect_Ex",
  /*  285 */  "loadB_indirect_ac_Ex",
  /*  286 */  "loadUB_indOffset16",
  /*  287 */  "loadUB_indOffset16_ac",
  /*  288 */  "loadB_indOffset16_Ex",
  /*  289 */  "loadB_indOffset16_ac_Ex",
  /*  290 */  "loadUB",
  /*  291 */  "loadUB_ac",
  /*  292 */  "loadUB2L",
  /*  293 */  "loadUB2L_ac",
  /*  294 */  "loadS",
  /*  295 */  "loadS_ac",
  /*  296 */  "loadUS",
  /*  297 */  "loadUS_ac",
  /*  298 */  "loadUS2L",
  /*  299 */  "loadUS2L_ac",
  /*  300 */  "loadI",
  /*  301 */  "loadI_ac",
  /*  302 */  "loadUI2L",
  /*  303 */  "loadI2L",
  /*  304 */  "loadI2L_ac",
  /*  305 */  "loadL",
  /*  306 */  "loadL_ac",
  /*  307 */  "loadL_unaligned",
  /*  308 */  "loadV8",
  /*  309 */  "loadRange",
  /*  310 */  "loadN",
  /*  311 */  "loadN_ac",
  /*  312 */  "loadN2P_unscaled",
  /*  313 */  "loadP",
  /*  314 */  "loadP_ac",
  /*  315 */  "loadNKlass",
  /*  316 */  "loadKlass",
  /*  317 */  "loadF",
  /*  318 */  "loadF_ac",
  /*  319 */  "loadD",
  /*  320 */  "loadD_ac",
  /*  321 */  "loadD_unaligned",
  /*  322 */  "loadToc_hi",
  /*  323 */  "loadConI32_lo16",
  /*  324 */  "loadConL32_lo16",
  /*  325 */  "loadConL",
  /*  326 */  "loadConL_hi",
  /*  327 */  "loadConL_lo",
  /*  328 */  "loadConN2",
  /*  329 */  "loadConP_hi",
  /*  330 */  "loadConF",
  /*  331 */  "loadConFComp",
  /*  332 */  "loadConD",
  /*  333 */  "loadConDComp",
  /*  334 */  "prefetchr",
  /*  335 */  "prefetchr_no_offset",
  /*  336 */  "prefetchw",
  /*  337 */  "prefetchw_no_offset",
  /*  338 */  "prefetch_alloc_zero",
  /*  339 */  "prefetch_alloc_zero_no_offset",
  /*  340 */  "prefetch_alloc",
  /*  341 */  "prefetch_alloc_no_offset",
  /*  342 */  "storeB",
  /*  343 */  "storeC",
  /*  344 */  "storeI",
  /*  345 */  "storeI_convL2I",
  /*  346 */  "storeL",
  /*  347 */  "storeA8B",
  /*  348 */  "storeN",
  /*  349 */  "storeP",
  /*  350 */  "storeF",
  /*  351 */  "storeD",
  /*  352 */  "storeCM_CMS",
  /*  353 */  "storeCM_CMS_ExEx",
  /*  354 */  "storeCM_G1",
  /*  355 */  "encodeP_shift",
  /*  356 */  "encodeP_sub",
  /*  357 */  "cond_sub_base",
  /*  358 */  "cond_set_0_oop",
  /*  359 */  "encodeP_32GAligned",
  /*  360 */  "encodeP_Ex",
  /*  361 */  "encodeP_not_null_Ex",
  /*  362 */  "encodeP_not_null_base_null",
  /*  363 */  "encodeP_narrow_oop_shift_0",
  /*  364 */  "decodeN_shift",
  /*  365 */  "decodeN_add",
  /*  366 */  "cond_add_base",
  /*  367 */  "cond_set_0_ptr",
  /*  368 */  "decodeN_Ex",
  /*  369 */  "decodeN_nullBase",
  /*  370 */  "decodeN_notNull_addBase_Ex",
  /*  371 */  "decodeN_unscaled",
  /*  372 */  "decodeN2I_unscaled",
  /*  373 */  "unnecessary_membar_acquire",
  /*  374 */  "membar_acquire_lock",
  /*  375 */  "membar_release",
  /*  376 */  "membar_release_lock",
  /*  377 */  "membar_volatile",
  /*  378 */  "membar_CPUOrder",
  /*  379 */  "membar_storestore",
  /*  380 */  "cmovI_reg_isel",
  /*  381 */  "cmovI_reg",
  /*  382 */  "cmovI_imm",
  /*  383 */  "cmovL_reg_isel",
  /*  384 */  "cmovL_reg",
  /*  385 */  "cmovL_imm",
  /*  386 */  "cmovN_reg_isel",
  /*  387 */  "cmovN_reg",
  /*  388 */  "cmovN_imm",
  /*  389 */  "cmovP_reg_isel",
  /*  390 */  "cmovP_reg",
  /*  391 */  "cmovP_imm",
  /*  392 */  "cmovF_reg",
  /*  393 */  "cmovD_reg",
  /*  394 */  "loadPLocked",
  /*  395 */  "compareAndSwapI_regP_regI_regI",
  /*  396 */  "compareAndSwapN_regP_regN_regN",
  /*  397 */  "compareAndSwapL_regP_regL_regL",
  /*  398 */  "compareAndSwapP_regP_regP_regP",
  /*  399 */  "getAndAddI",
  /*  400 */  "getAndAddL",
  /*  401 */  "getAndSetI",
  /*  402 */  "getAndSetL",
  /*  403 */  "getAndSetP",
  /*  404 */  "getAndSetN",
  /*  405 */  "addI_reg_reg",
  /*  406 */  "addI_reg_reg_2",
  /*  407 */  "tree_addI_addI_addI_reg_reg_Ex",
  /*  408 */  "tree_addI_addI_addI_reg_reg_Ex_1",
  /*  409 */  "tree_addI_addI_addI_reg_reg_Ex_0",
  /*  410 */  "tree_addI_addI_addI_reg_reg_Ex_2",
  /*  411 */  "addI_reg_imm16",
  /*  412 */  "addI_reg_immhi16",
  /*  413 */  "addL_reg_reg",
  /*  414 */  "addL_reg_reg_2",
  /*  415 */  "tree_addL_addL_addL_reg_reg_Ex",
  /*  416 */  "tree_addL_addL_addL_reg_reg_Ex_1",
  /*  417 */  "tree_addL_addL_addL_reg_reg_Ex_0",
  /*  418 */  "tree_addL_addL_addL_reg_reg_Ex_2",
  /*  419 */  "addI_regL_regL",
  /*  420 */  "addL_reg_imm16",
  /*  421 */  "addL_reg_immhi16",
  /*  422 */  "addP_reg_reg",
  /*  423 */  "addP_reg_imm16",
  /*  424 */  "addP_reg_immhi16",
  /*  425 */  "subI_reg_reg",
  /*  426 */  "subI_reg_imm16",
  /*  427 */  "subI_imm16_reg",
  /*  428 */  "negI_regI",
  /*  429 */  "subL_reg_reg",
  /*  430 */  "subI_regL_regL",
  /*  431 */  "subL_reg_imm16",
  /*  432 */  "negL_reg_reg",
  /*  433 */  "negI_con0_regL",
  /*  434 */  "mulI_reg_reg",
  /*  435 */  "mulI_reg_imm16",
  /*  436 */  "mulL_reg_reg",
  /*  437 */  "mulHighL_reg_reg",
  /*  438 */  "mulL_reg_imm16",
  /*  439 */  "divI_reg_immIvalueMinus1",
  /*  440 */  "divI_reg_regnotMinus1",
  /*  441 */  "cmovI_bne_negI_reg",
  /*  442 */  "divI_reg_reg_Ex",
  /*  443 */  "divL_reg_immLvalueMinus1",
  /*  444 */  "divL_reg_regnotMinus1",
  /*  445 */  "cmovL_bne_negL_reg",
  /*  446 */  "divL_reg_reg_Ex",
  /*  447 */  "modI_reg_reg_Ex",
  /*  448 */  "modL_reg_reg_Ex",
  /*  449 */  "maskI_reg_imm",
  /*  450 */  "lShiftI_reg_reg",
  /*  451 */  "lShiftI_reg_reg_Ex",
  /*  452 */  "lShiftI_reg_imm",
  /*  453 */  "lShiftI_andI_immInegpow2_imm5",
  /*  454 */  "lShiftI_andI_immInegpow2_rShiftI_imm5",
  /*  455 */  "lShiftL_regL_regI",
  /*  456 */  "lShiftL_regL_regI_Ex",
  /*  457 */  "lshiftL_regL_immI",
  /*  458 */  "lShiftL_regI_immGE32",
  /*  459 */  "scaledPositiveI2L_lShiftL_convI2L_reg_imm6",
  /*  460 */  "arShiftI_reg_reg",
  /*  461 */  "arShiftI_reg_reg_Ex",
  /*  462 */  "arShiftI_reg_imm",
  /*  463 */  "arShiftL_regL_regI",
  /*  464 */  "arShiftL_regL_regI_Ex",
  /*  465 */  "arShiftL_regL_immI",
  /*  466 */  "convL2I_arShiftL_regL_immI",
  /*  467 */  "urShiftI_reg_reg",
  /*  468 */  "urShiftI_reg_reg_Ex",
  /*  469 */  "urShiftI_reg_imm",
  /*  470 */  "urShiftL_regL_regI",
  /*  471 */  "urShiftL_regL_regI_Ex",
  /*  472 */  "urShiftL_regL_immI",
  /*  473 */  "convL2I_urShiftL_regL_immI",
  /*  474 */  "shrP_convP2X_reg_imm6",
  /*  475 */  "sxtI_reg",
  /*  476 */  "rotlI_reg_immi8",
  /*  477 */  "rotlI_reg_immi8_0",
  /*  478 */  "rotrI_reg_immi8",
  /*  479 */  "rotrI_reg_immi8_0",
  /*  480 */  "addF_reg_reg",
  /*  481 */  "addD_reg_reg",
  /*  482 */  "subF_reg_reg",
  /*  483 */  "subD_reg_reg",
  /*  484 */  "mulF_reg_reg",
  /*  485 */  "mulD_reg_reg",
  /*  486 */  "divF_reg_reg",
  /*  487 */  "divD_reg_reg",
  /*  488 */  "sqrtD_reg",
  /*  489 */  "sqrtF_reg",
  /*  490 */  "andI_reg_reg",
  /*  491 */  "andI_reg_uimm16",
  /*  492 */  "andI_reg_immInegpow2",
  /*  493 */  "andI_reg_immIpow2minus1",
  /*  494 */  "andI_reg_immIpowerOf2",
  /*  495 */  "andL_reg_reg",
  /*  496 */  "andL_reg_uimm16",
  /*  497 */  "andL_reg_immLnegpow2",
  /*  498 */  "andL_reg_immLpow2minus1",
  /*  499 */  "convL2I_andL_reg_immLpow2minus1",
  /*  500 */  "orI_reg_reg",
  /*  501 */  "orI_reg_reg_2",
  /*  502 */  "tree_orI_orI_orI_reg_reg_Ex",
  /*  503 */  "tree_orI_orI_orI_reg_reg_Ex_1",
  /*  504 */  "tree_orI_orI_orI_reg_reg_Ex_0",
  /*  505 */  "tree_orI_orI_orI_reg_reg_Ex_2",
  /*  506 */  "orI_reg_uimm16",
  /*  507 */  "orL_reg_reg",
  /*  508 */  "orI_regL_regL",
  /*  509 */  "orL_reg_uimm16",
  /*  510 */  "xorI_reg_reg",
  /*  511 */  "xorI_reg_reg_2",
  /*  512 */  "tree_xorI_xorI_xorI_reg_reg_Ex",
  /*  513 */  "tree_xorI_xorI_xorI_reg_reg_Ex_1",
  /*  514 */  "tree_xorI_xorI_xorI_reg_reg_Ex_0",
  /*  515 */  "tree_xorI_xorI_xorI_reg_reg_Ex_2",
  /*  516 */  "xorI_reg_uimm16",
  /*  517 */  "xorL_reg_reg",
  /*  518 */  "xorI_regL_regL",
  /*  519 */  "xorL_reg_uimm16",
  /*  520 */  "notI_reg",
  /*  521 */  "notL_reg",
  /*  522 */  "andcI_reg_reg",
  /*  523 */  "andcI_reg_reg_0",
  /*  524 */  "andcL_reg_reg",
  /*  525 */  "moveF2I_reg_stack",
  /*  526 */  "moveI2F_reg_stack",
  /*  527 */  "moveF2L_reg_stack",
  /*  528 */  "moveD2L_reg_stack",
  /*  529 */  "moveL2D_reg_stack",
  /*  530 */  "convI2Bool_reg__cmove",
  /*  531 */  "xorI_convI2Bool_reg_immIvalue1__cntlz_Ex",
  /*  532 */  "xorI_convI2Bool_reg_immIvalue1__cmove",
  /*  533 */  "convI2Bool_andI_reg_immIpowerOf2",
  /*  534 */  "convP2Bool_reg__cmove",
  /*  535 */  "xorI_convP2Bool_reg__cntlz_Ex",
  /*  536 */  "xorI_convP2Bool_reg_immIvalue1__cmove",
  /*  537 */  "cmpLTMask_reg_reg_Ex",
  /*  538 */  "cmpLTMask_reg_immI0",
  /*  539 */  "convB2I_reg",
  /*  540 */  "convS2I_reg",
  /*  541 */  "sxtI_L2L_reg",
  /*  542 */  "convL2I_reg",
  /*  543 */  "cmovI_bso_stackSlotL",
  /*  544 */  "cmovI_bso_stackSlotL_conLvalue0_Ex",
  /*  545 */  "convD2I_reg_ExEx",
  /*  546 */  "convF2I_regF_ExEx",
  /*  547 */  "convI2L_reg",
  /*  548 */  "zeroExtendL_regI",
  /*  549 */  "zeroExtendL_regL",
  /*  550 */  "cmovL_bso_stackSlotL",
  /*  551 */  "cmovL_bso_stackSlotL_conLvalue0_Ex",
  /*  552 */  "convF2L_reg_ExEx",
  /*  553 */  "convD2L_reg_ExEx",
  /*  554 */  "convD2F_reg",
  /*  555 */  "convI2F_ireg_Ex",
  /*  556 */  "convI2F_ireg_fcfids_Ex",
  /*  557 */  "convL2F_ireg_fcfids_Ex",
  /*  558 */  "convI2D_reg_Ex",
  /*  559 */  "convL2D_reg_Ex",
  /*  560 */  "convF2D_reg",
  /*  561 */  "cmpL3_reg_reg_ExEx",
  /*  562 */  "rangeCheck_iReg_uimm15",
  /*  563 */  "rangeCheck_iReg_iReg",
  /*  564 */  "rangeCheck_uimm15_iReg",
  /*  565 */  "zeroCheckN_iReg_imm0",
  /*  566 */  "zeroCheckP_reg_imm0",
  /*  567 */  "cmpF3_reg_reg_ExEx",
  /*  568 */  "cmpD3_reg_reg_ExEx",
  /*  569 */  "branch",
  /*  570 */  "branchCon",
  /*  571 */  "branchConFar",
  /*  572 */  "branchConSched",
  /*  573 */  "branchLoopEnd",
  /*  574 */  "branchLoopEndFar",
  /*  575 */  "branchLoopEndSched",
  /*  576 */  "partialSubtypeCheck",
  /*  577 */  "align_addr",
  /*  578 */  "array_size",
  /*  579 */  "inlineCallClearArray",
  /*  580 */  "string_indexOf_imm1_char",
  /*  581 */  "string_indexOf_imm1",
  /*  582 */  "string_indexOf_imm",
  /*  583 */  "string_indexOf",
  /*  584 */  "string_equals_imm",
  /*  585 */  "string_equals",
  /*  586 */  "string_compare",
  /*  587 */  "minI_reg_reg_Ex",
  /*  588 */  "maxI_reg_reg_Ex",
  /*  589 */  "insrwi_a",
  /*  590 */  "insrwi",
  /*  591 */  "bytes_reverse_int_Ex",
  /*  592 */  "repl8B_reg_Ex",
  /*  593 */  "repl8B_immI0",
  /*  594 */  "repl8B_immIminus1",
  /*  595 */  "repl4S_reg_Ex",
  /*  596 */  "repl4S_immI0",
  /*  597 */  "repl4S_immIminus1",
  /*  598 */  "repl2I_reg_Ex",
  /*  599 */  "repl2I_immI0",
  /*  600 */  "repl2I_immIminus1",
  /*  601 */  "repl2F_reg_Ex",
  /*  602 */  "repl2F_immF_Ex",
  /*  603 */  "repl2F_immF0",
  /*  604 */  "safePoint_poll",
  /*  605 */  "safePoint_poll_conPollAddr",
  /*  606 */  "CallStaticJavaDirect",
  /*  607 */  "CallStaticJavaDirectHandle",
  /*  608 */  "CallDynamicJavaDirectSched",
  /*  609 */  "CallDynamicJavaDirectSched_Ex",
  /*  610 */  "CallDynamicJavaDirect",
  /*  611 */  "CallRuntimeDirect",
  /*  612 */  "CallLeafDirect",
  /*  613 */  "CallLeafDirect_Ex",
  /*  614 */  "CallLeafNoFPDirect_Ex",
  /*  615 */  "TailCalljmpInd",
  /*  616 */  "Ret",
  /*  617 */  "tailjmpInd",
  /*  618 */  "CreateException",
  /*  619 */  "RethrowException",
  /*  620 */  "ShouldNotReachHere",
  /*  621 */  "endGroup",
  /*  622 */  "fxNop",
  /*  623 */  "fpNop0",
  /*  624 */  "fpNop1",
  /*  625 */  "brNop0",
  /*  626 */  "brNop1",
  /*  627 */  "brNop2",
  // last instruction
  "invalid rule name" // no trailing comma
};

const        bool  swallowed[] = {
  /*    0 */  false,
  /*    1 */  false,
  /*    2 */  false,
  /*    3 */  false,
  /*    4 */  false,
  /*    5 */  false,
  /*    6 */  false,
  /*    7 */  false,
  /*    8 */  true,
  /*    9 */  true,
  /*   10 */  true,
  /*   11 */  true,
  /*   12 */  true,
  /*   13 */  true,
  /*   14 */  true,
  /*   15 */  true,
  /*   16 */  true,
  /*   17 */  true,
  /*   18 */  true,
  /*   19 */  true,
  /*   20 */  true,
  /*   21 */  true,
  /*   22 */  true,
  /*   23 */  true,
  /*   24 */  true,
  /*   25 */  true,
  /*   26 */  true,
  /*   27 */  true,
  /*   28 */  true,
  /*   29 */  true,
  /*   30 */  true,
  /*   31 */  true,
  /*   32 */  true,
  /*   33 */  true,
  /*   34 */  true,
  /*   35 */  true,
  /*   36 */  true,
  /*   37 */  true,
  /*   38 */  true,
  /*   39 */  true,
  /*   40 */  true,
  /*   41 */  true,
  /*   42 */  true,
  /*   43 */  true,
  /*   44 */  true,
  /*   45 */  true,
  /*   46 */  true,
  /*   47 */  false,
  /*   48 */  false,
  /*   49 */  false,
  /*   50 */  false,
  /*   51 */  false,
  /*   52 */  false,
  /*   53 */  false,
  /*   54 */  false,
  /*   55 */  false,
  /*   56 */  false,
  /*   57 */  false,
  /*   58 */  false,
  /*   59 */  false,
  /*   60 */  false,
  /*   61 */  false,
  /*   62 */  false,
  /*   63 */  false,
  /*   64 */  false,
  /*   65 */  false,
  /*   66 */  false,
  /*   67 */  false,
  /*   68 */  false,
  /*   69 */  false,
  /*   70 */  false,
  /*   71 */  false,
  /*   72 */  false,
  /*   73 */  false,
  /*   74 */  false,
  /*   75 */  false,
  /*   76 */  false,
  /*   77 */  false,
  /*   78 */  false,
  /*   79 */  false,
  /*   80 */  false,
  /*   81 */  false,
  /*   82 */  false,
  /*   83 */  false,
  /*   84 */  false,
  /*   85 */  false,
  /*   86 */  false,
  /*   87 */  false,
  /*   88 */  false,
  /*   89 */  false,
  /*   90 */  false,
  /*   91 */  false,
  /*   92 */  false,
  /*   93 */  false,
  /*   94 */  false,
  /*   95 */  false,
  /*   96 */  false,
  /*   97 */  false,
  /*   98 */  false,
  /*   99 */  true,
  // last operand
  /*  100 */  false,
  /*  101 */  false,
  /*  102 */  false,
  /*  103 */  false,
  /*  104 */  false,
  /*  105 */  false,
  // last operand class
  /*  106 */  false,
  /*  107 */  false,
  /*  108 */  false,
  /*  109 */  false,
  /*  110 */  false,
  /*  111 */  false,
  /*  112 */  false,
  /*  113 */  false,
  /*  114 */  false,
  /*  115 */  false,
  /*  116 */  false,
  /*  117 */  false,
  /*  118 */  false,
  /*  119 */  false,
  /*  120 */  false,
  /*  121 */  false,
  /*  122 */  false,
  /*  123 */  false,
  /*  124 */  false,
  /*  125 */  false,
  /*  126 */  false,
  /*  127 */  false,
  /*  128 */  false,
  /*  129 */  false,
  /*  130 */  false,
  /*  131 */  false,
  /*  132 */  false,
  /*  133 */  false,
  /*  134 */  false,
  /*  135 */  false,
  /*  136 */  false,
  /*  137 */  false,
  /*  138 */  false,
  /*  139 */  false,
  /*  140 */  false,
  /*  141 */  false,
  /*  142 */  false,
  /*  143 */  false,
  /*  144 */  false,
  /*  145 */  false,
  /*  146 */  false,
  /*  147 */  false,
  /*  148 */  false,
  /*  149 */  false,
  /*  150 */  false,
  /*  151 */  false,
  /*  152 */  false,
  /*  153 */  false,
  /*  154 */  false,
  /*  155 */  false,
  /*  156 */  false,
  /*  157 */  false,
  /*  158 */  false,
  /*  159 */  false,
  /*  160 */  false,
  /*  161 */  false,
  /*  162 */  false,
  /*  163 */  false,
  /*  164 */  false,
  /*  165 */  false,
  /*  166 */  false,
  /*  167 */  false,
  /*  168 */  false,
  /*  169 */  false,
  /*  170 */  false,
  /*  171 */  false,
  /*  172 */  false,
  /*  173 */  false,
  /*  174 */  false,
  /*  175 */  false,
  /*  176 */  false,
  /*  177 */  false,
  /*  178 */  false,
  /*  179 */  false,
  /*  180 */  false,
  /*  181 */  false,
  /*  182 */  false,
  /*  183 */  false,
  /*  184 */  false,
  /*  185 */  false,
  /*  186 */  false,
  /*  187 */  false,
  // last internally defined operand
  /*  188 */  false,
  /*  189 */  false,
  /*  190 */  false,
  /*  191 */  false,
  /*  192 */  false,
  /*  193 */  false,
  /*  194 */  false,
  /*  195 */  false,
  /*  196 */  false,
  /*  197 */  false,
  /*  198 */  false,
  /*  199 */  false,
  /*  200 */  false,
  /*  201 */  false,
  /*  202 */  false,
  /*  203 */  false,
  /*  204 */  false,
  /*  205 */  false,
  /*  206 */  false,
  /*  207 */  false,
  /*  208 */  false,
  /*  209 */  false,
  /*  210 */  false,
  /*  211 */  false,
  /*  212 */  false,
  /*  213 */  false,
  /*  214 */  false,
  /*  215 */  false,
  /*  216 */  false,
  /*  217 */  false,
  /*  218 */  false,
  /*  219 */  false,
  /*  220 */  false,
  /*  221 */  false,
  /*  222 */  false,
  /*  223 */  false,
  /*  224 */  false,
  /*  225 */  false,
  /*  226 */  false,
  /*  227 */  false,
  /*  228 */  false,
  /*  229 */  false,
  /*  230 */  false,
  /*  231 */  false,
  /*  232 */  false,
  /*  233 */  false,
  /*  234 */  false,
  /*  235 */  false,
  /*  236 */  false,
  /*  237 */  false,
  /*  238 */  false,
  /*  239 */  false,
  /*  240 */  false,
  /*  241 */  false,
  /*  242 */  false,
  /*  243 */  false,
  /*  244 */  false,
  /*  245 */  false,
  /*  246 */  false,
  /*  247 */  false,
  /*  248 */  false,
  /*  249 */  false,
  /*  250 */  false,
  /*  251 */  false,
  /*  252 */  false,
  /*  253 */  false,
  /*  254 */  false,
  /*  255 */  false,
  /*  256 */  false,
  /*  257 */  false,
  /*  258 */  false,
  /*  259 */  false,
  /*  260 */  false,
  /*  261 */  false,
  /*  262 */  false,
  /*  263 */  false,
  /*  264 */  false,
  /*  265 */  false,
  /*  266 */  false,
  /*  267 */  false,
  /*  268 */  false,
  /*  269 */  false,
  /*  270 */  false,
  /*  271 */  false,
  /*  272 */  false,
  /*  273 */  false,
  /*  274 */  false,
  /*  275 */  false,
  /*  276 */  false,
  /*  277 */  false,
  /*  278 */  false,
  /*  279 */  false,
  /*  280 */  false,
  /*  281 */  false,
  /*  282 */  false,
  /*  283 */  false,
  /*  284 */  false,
  /*  285 */  false,
  /*  286 */  false,
  /*  287 */  false,
  /*  288 */  false,
  /*  289 */  false,
  /*  290 */  false,
  /*  291 */  false,
  /*  292 */  false,
  /*  293 */  false,
  /*  294 */  false,
  /*  295 */  false,
  /*  296 */  false,
  /*  297 */  false,
  /*  298 */  false,
  /*  299 */  false,
  /*  300 */  false,
  /*  301 */  false,
  /*  302 */  false,
  /*  303 */  false,
  /*  304 */  false,
  /*  305 */  false,
  /*  306 */  false,
  /*  307 */  false,
  /*  308 */  false,
  /*  309 */  false,
  /*  310 */  false,
  /*  311 */  false,
  /*  312 */  false,
  /*  313 */  false,
  /*  314 */  false,
  /*  315 */  false,
  /*  316 */  false,
  /*  317 */  false,
  /*  318 */  false,
  /*  319 */  false,
  /*  320 */  false,
  /*  321 */  false,
  /*  322 */  false,
  /*  323 */  false,
  /*  324 */  false,
  /*  325 */  false,
  /*  326 */  false,
  /*  327 */  false,
  /*  328 */  false,
  /*  329 */  false,
  /*  330 */  false,
  /*  331 */  false,
  /*  332 */  false,
  /*  333 */  false,
  /*  334 */  false,
  /*  335 */  false,
  /*  336 */  false,
  /*  337 */  false,
  /*  338 */  false,
  /*  339 */  false,
  /*  340 */  false,
  /*  341 */  false,
  /*  342 */  false,
  /*  343 */  false,
  /*  344 */  false,
  /*  345 */  false,
  /*  346 */  false,
  /*  347 */  false,
  /*  348 */  false,
  /*  349 */  false,
  /*  350 */  false,
  /*  351 */  false,
  /*  352 */  false,
  /*  353 */  false,
  /*  354 */  false,
  /*  355 */  false,
  /*  356 */  false,
  /*  357 */  false,
  /*  358 */  false,
  /*  359 */  false,
  /*  360 */  false,
  /*  361 */  false,
  /*  362 */  false,
  /*  363 */  false,
  /*  364 */  false,
  /*  365 */  false,
  /*  366 */  false,
  /*  367 */  false,
  /*  368 */  false,
  /*  369 */  false,
  /*  370 */  false,
  /*  371 */  false,
  /*  372 */  false,
  /*  373 */  false,
  /*  374 */  false,
  /*  375 */  false,
  /*  376 */  false,
  /*  377 */  false,
  /*  378 */  false,
  /*  379 */  false,
  /*  380 */  false,
  /*  381 */  false,
  /*  382 */  false,
  /*  383 */  false,
  /*  384 */  false,
  /*  385 */  false,
  /*  386 */  false,
  /*  387 */  false,
  /*  388 */  false,
  /*  389 */  false,
  /*  390 */  false,
  /*  391 */  false,
  /*  392 */  false,
  /*  393 */  false,
  /*  394 */  false,
  /*  395 */  false,
  /*  396 */  false,
  /*  397 */  false,
  /*  398 */  false,
  /*  399 */  false,
  /*  400 */  false,
  /*  401 */  false,
  /*  402 */  false,
  /*  403 */  false,
  /*  404 */  false,
  /*  405 */  false,
  /*  406 */  false,
  /*  407 */  false,
  /*  408 */  false,
  /*  409 */  false,
  /*  410 */  false,
  /*  411 */  false,
  /*  412 */  false,
  /*  413 */  false,
  /*  414 */  false,
  /*  415 */  false,
  /*  416 */  false,
  /*  417 */  false,
  /*  418 */  false,
  /*  419 */  false,
  /*  420 */  false,
  /*  421 */  false,
  /*  422 */  false,
  /*  423 */  false,
  /*  424 */  false,
  /*  425 */  false,
  /*  426 */  false,
  /*  427 */  false,
  /*  428 */  false,
  /*  429 */  false,
  /*  430 */  false,
  /*  431 */  false,
  /*  432 */  false,
  /*  433 */  false,
  /*  434 */  false,
  /*  435 */  false,
  /*  436 */  false,
  /*  437 */  false,
  /*  438 */  false,
  /*  439 */  false,
  /*  440 */  false,
  /*  441 */  false,
  /*  442 */  false,
  /*  443 */  false,
  /*  444 */  false,
  /*  445 */  false,
  /*  446 */  false,
  /*  447 */  false,
  /*  448 */  false,
  /*  449 */  false,
  /*  450 */  false,
  /*  451 */  false,
  /*  452 */  false,
  /*  453 */  false,
  /*  454 */  false,
  /*  455 */  false,
  /*  456 */  false,
  /*  457 */  false,
  /*  458 */  false,
  /*  459 */  false,
  /*  460 */  false,
  /*  461 */  false,
  /*  462 */  false,
  /*  463 */  false,
  /*  464 */  false,
  /*  465 */  false,
  /*  466 */  false,
  /*  467 */  false,
  /*  468 */  false,
  /*  469 */  false,
  /*  470 */  false,
  /*  471 */  false,
  /*  472 */  false,
  /*  473 */  false,
  /*  474 */  false,
  /*  475 */  false,
  /*  476 */  false,
  /*  477 */  false,
  /*  478 */  false,
  /*  479 */  false,
  /*  480 */  false,
  /*  481 */  false,
  /*  482 */  false,
  /*  483 */  false,
  /*  484 */  false,
  /*  485 */  false,
  /*  486 */  false,
  /*  487 */  false,
  /*  488 */  false,
  /*  489 */  false,
  /*  490 */  false,
  /*  491 */  false,
  /*  492 */  false,
  /*  493 */  false,
  /*  494 */  false,
  /*  495 */  false,
  /*  496 */  false,
  /*  497 */  false,
  /*  498 */  false,
  /*  499 */  false,
  /*  500 */  false,
  /*  501 */  false,
  /*  502 */  false,
  /*  503 */  false,
  /*  504 */  false,
  /*  505 */  false,
  /*  506 */  false,
  /*  507 */  false,
  /*  508 */  false,
  /*  509 */  false,
  /*  510 */  false,
  /*  511 */  false,
  /*  512 */  false,
  /*  513 */  false,
  /*  514 */  false,
  /*  515 */  false,
  /*  516 */  false,
  /*  517 */  false,
  /*  518 */  false,
  /*  519 */  false,
  /*  520 */  false,
  /*  521 */  false,
  /*  522 */  false,
  /*  523 */  false,
  /*  524 */  false,
  /*  525 */  false,
  /*  526 */  false,
  /*  527 */  false,
  /*  528 */  false,
  /*  529 */  false,
  /*  530 */  false,
  /*  531 */  false,
  /*  532 */  false,
  /*  533 */  false,
  /*  534 */  false,
  /*  535 */  false,
  /*  536 */  false,
  /*  537 */  false,
  /*  538 */  false,
  /*  539 */  false,
  /*  540 */  false,
  /*  541 */  false,
  /*  542 */  false,
  /*  543 */  false,
  /*  544 */  false,
  /*  545 */  false,
  /*  546 */  false,
  /*  547 */  false,
  /*  548 */  false,
  /*  549 */  false,
  /*  550 */  false,
  /*  551 */  false,
  /*  552 */  false,
  /*  553 */  false,
  /*  554 */  false,
  /*  555 */  false,
  /*  556 */  false,
  /*  557 */  false,
  /*  558 */  false,
  /*  559 */  false,
  /*  560 */  false,
  /*  561 */  false,
  /*  562 */  false,
  /*  563 */  false,
  /*  564 */  false,
  /*  565 */  false,
  /*  566 */  false,
  /*  567 */  false,
  /*  568 */  false,
  /*  569 */  false,
  /*  570 */  false,
  /*  571 */  false,
  /*  572 */  false,
  /*  573 */  false,
  /*  574 */  false,
  /*  575 */  false,
  /*  576 */  false,
  /*  577 */  false,
  /*  578 */  false,
  /*  579 */  false,
  /*  580 */  false,
  /*  581 */  false,
  /*  582 */  false,
  /*  583 */  false,
  /*  584 */  false,
  /*  585 */  false,
  /*  586 */  false,
  /*  587 */  false,
  /*  588 */  false,
  /*  589 */  false,
  /*  590 */  false,
  /*  591 */  false,
  /*  592 */  false,
  /*  593 */  false,
  /*  594 */  false,
  /*  595 */  false,
  /*  596 */  false,
  /*  597 */  false,
  /*  598 */  false,
  /*  599 */  false,
  /*  600 */  false,
  /*  601 */  false,
  /*  602 */  false,
  /*  603 */  false,
  /*  604 */  false,
  /*  605 */  false,
  /*  606 */  false,
  /*  607 */  false,
  /*  608 */  false,
  /*  609 */  false,
  /*  610 */  false,
  /*  611 */  false,
  /*  612 */  false,
  /*  613 */  false,
  /*  614 */  false,
  /*  615 */  false,
  /*  616 */  false,
  /*  617 */  false,
  /*  618 */  false,
  /*  619 */  false,
  /*  620 */  false,
  /*  621 */  false,
  /*  622 */  false,
  /*  623 */  false,
  /*  624 */  false,
  /*  625 */  false,
  /*  626 */  false,
  /*  627 */  false,
  // last instruction
  false // no trailing comma
};

// Mapping from machine-independent opcode to boolean
const        char must_clone[] = {
  0, // Node: 0
  0, // Set: 1
  0, // RegN: 2
  0, // RegI: 3
  0, // RegP: 4
  0, // RegF: 5
  0, // RegD: 6
  0, // RegL: 7
  0, // RegFlags: 8
  0, // VecS: 9
  0, // VecD: 10
  0, // VecX: 11
  0, // VecY: 12
  0, // _last_machine_leaf: 13
  0, // AbsD: 14
  0, // AbsF: 15
  0, // AbsI: 16
  0, // AddD: 17
  0, // AddF: 18
  0, // AddI: 19
  0, // AddL: 20
  0, // AddP: 21
  0, // Allocate: 22
  0, // AllocateArray: 23
  0, // AndI: 24
  0, // AndL: 25
  0, // AryEq: 26
  0, // AtanD: 27
  1, // Binary: 28
  1, // Bool: 29
  0, // BoxLock: 30
  0, // ReverseBytesI: 31
  0, // ReverseBytesL: 32
  0, // ReverseBytesUS: 33
  0, // ReverseBytesS: 34
  0, // CProj: 35
  0, // CallDynamicJava: 36
  0, // CallJava: 37
  0, // CallLeaf: 38
  0, // CallLeafNoFP: 39
  0, // CallRuntime: 40
  0, // CallStaticJava: 41
  0, // CastII: 42
  0, // CastX2P: 43
  0, // CastP2X: 44
  0, // CastPP: 45
  0, // Catch: 46
  0, // CatchProj: 47
  0, // CheckCastPP: 48
  0, // ClearArray: 49
  0, // ConstraintCast: 50
  0, // CMoveD: 51
  0, // CMoveF: 52
  0, // CMoveI: 53
  0, // CMoveL: 54
  0, // CMoveP: 55
  0, // CMoveN: 56
  1, // CmpN: 57
  1, // CmpD: 58
  0, // CmpD3: 59
  1, // CmpF: 60
  0, // CmpF3: 61
  1, // CmpI: 62
  1, // CmpL: 63
  0, // CmpL3: 64
  0, // CmpLTMask: 65
  1, // CmpP: 66
  1, // CmpU: 67
  1, // CmpUL: 68
  0, // CompareAndSwapI: 69
  0, // CompareAndSwapL: 70
  0, // CompareAndSwapP: 71
  0, // CompareAndSwapN: 72
  0, // GetAndAddI: 73
  0, // GetAndAddL: 74
  0, // GetAndSetI: 75
  0, // GetAndSetL: 76
  0, // GetAndSetP: 77
  0, // GetAndSetN: 78
  0, // Con: 79
  0, // ConN: 80
  0, // ConD: 81
  0, // ConF: 82
  0, // ConI: 83
  0, // ConL: 84
  0, // ConP: 85
  0, // Conv2B: 86
  0, // ConvD2F: 87
  0, // ConvD2I: 88
  0, // ConvD2L: 89
  0, // ConvF2D: 90
  0, // ConvF2I: 91
  0, // ConvF2L: 92
  0, // ConvI2D: 93
  0, // ConvI2F: 94
  0, // ConvI2L: 95
  0, // ConvL2D: 96
  0, // ConvL2F: 97
  0, // ConvL2I: 98
  0, // CosD: 99
  0, // CountedLoop: 100
  0, // CountedLoopEnd: 101
  0, // CountLeadingZerosI: 102
  0, // CountLeadingZerosL: 103
  0, // CountTrailingZerosI: 104
  0, // CountTrailingZerosL: 105
  0, // CreateEx: 106
  0, // DecodeN: 107
  0, // DivD: 108
  0, // DivF: 109
  0, // DivI: 110
  0, // DivL: 111
  0, // DivMod: 112
  0, // DivModI: 113
  0, // DivModL: 114
  0, // EncodeP: 115
  0, // ExpD: 116
  1, // FastLock: 117
  1, // FastUnlock: 118
  0, // Goto: 119
  0, // Halt: 120
  0, // If: 121
  0, // IfFalse: 122
  0, // IfTrue: 123
  0, // Initialize: 124
  0, // JProj: 125
  0, // Jump: 126
  0, // JumpProj: 127
  0, // LShiftI: 128
  0, // LShiftL: 129
  0, // LoadB: 130
  0, // LoadUB: 131
  0, // LoadUS: 132
  0, // LoadD: 133
  0, // LoadD_unaligned: 134
  0, // LoadF: 135
  0, // LoadI: 136
  0, // LoadKlass: 137
  0, // LoadNKlass: 138
  0, // LoadL: 139
  0, // LoadL_unaligned: 140
  0, // LoadPLocked: 141
  0, // LoadP: 142
  0, // LoadN: 143
  0, // LoadRange: 144
  0, // LoadS: 145
  0, // Lock: 146
  0, // LogD: 147
  0, // Log10D: 148
  0, // Loop: 149
  0, // LoopLimit: 150
  0, // Mach: 151
  0, // MachProj: 152
  0, // MaxI: 153
  0, // MemBarAcquire: 154
  0, // MemBarAcquireLock: 155
  0, // MemBarCPUOrder: 156
  0, // MemBarRelease: 157
  0, // MemBarReleaseLock: 158
  0, // MemBarVolatile: 159
  0, // MemBarStoreStore: 160
  0, // MergeMem: 161
  0, // MinI: 162
  0, // ModD: 163
  0, // ModF: 164
  0, // ModI: 165
  0, // ModL: 166
  0, // MoveI2F: 167
  0, // MoveF2I: 168
  0, // MoveL2D: 169
  0, // MoveD2L: 170
  0, // MulD: 171
  0, // MulF: 172
  0, // MulHiL: 173
  0, // MulI: 174
  0, // MulL: 175
  0, // Multi: 176
  0, // NegD: 177
  0, // NegF: 178
  0, // NeverBranch: 179
  0, // Opaque1: 180
  0, // Opaque2: 181
  0, // OrI: 182
  0, // OrL: 183
  0, // PCTable: 184
  0, // Parm: 185
  0, // PartialSubtypeCheck: 186
  0, // Phi: 187
  0, // PopCountI: 188
  0, // PopCountL: 189
  0, // PowD: 190
  0, // PrefetchAllocation: 191
  0, // PrefetchRead: 192
  0, // PrefetchWrite: 193
  0, // Proj: 194
  0, // RShiftI: 195
  0, // RShiftL: 196
  0, // Region: 197
  0, // Rethrow: 198
  0, // Return: 199
  0, // Root: 200
  0, // RoundDouble: 201
  0, // RoundFloat: 202
  0, // SafePoint: 203
  0, // SafePointScalarObject: 204
  0, // SCMemProj: 205
  0, // SinD: 206
  0, // SqrtD: 207
  0, // Start: 208
  0, // StartOSR: 209
  0, // StoreB: 210
  0, // StoreC: 211
  0, // StoreCM: 212
  0, // StorePConditional: 213
  0, // StoreIConditional: 214
  0, // StoreLConditional: 215
  0, // StoreD: 216
  0, // StoreF: 217
  0, // StoreI: 218
  0, // StoreL: 219
  0, // StoreP: 220
  0, // StoreN: 221
  0, // StrComp: 222
  0, // StrEquals: 223
  0, // StrIndexOf: 224
  0, // SubD: 225
  0, // SubF: 226
  0, // SubI: 227
  0, // SubL: 228
  0, // TailCall: 229
  0, // TailJump: 230
  0, // TanD: 231
  0, // ThreadLocal: 232
  0, // Unlock: 233
  0, // URShiftI: 234
  0, // URShiftL: 235
  0, // XorI: 236
  0, // XorL: 237
  0, // Vector: 238
  0, // AddVB: 239
  0, // AddVS: 240
  0, // AddVI: 241
  0, // AddVL: 242
  0, // AddVF: 243
  0, // AddVD: 244
  0, // SubVB: 245
  0, // SubVS: 246
  0, // SubVI: 247
  0, // SubVL: 248
  0, // SubVF: 249
  0, // SubVD: 250
  0, // MulVS: 251
  0, // MulVI: 252
  0, // MulVF: 253
  0, // MulVD: 254
  0, // DivVF: 255
  0, // DivVD: 256
  0, // LShiftCntV: 257
  0, // RShiftCntV: 258
  0, // LShiftVB: 259
  0, // LShiftVS: 260
  0, // LShiftVI: 261
  0, // LShiftVL: 262
  0, // RShiftVB: 263
  0, // RShiftVS: 264
  0, // RShiftVI: 265
  0, // RShiftVL: 266
  0, // URShiftVB: 267
  0, // URShiftVS: 268
  0, // URShiftVI: 269
  0, // URShiftVL: 270
  0, // AndV: 271
  0, // OrV: 272
  0, // XorV: 273
  0, // LoadVector: 274
  0, // StoreVector: 275
  0, // Pack: 276
  0, // PackB: 277
  0, // PackS: 278
  0, // PackI: 279
  0, // PackL: 280
  0, // PackF: 281
  0, // PackD: 282
  0, // Pack2L: 283
  0, // Pack2D: 284
  0, // ReplicateB: 285
  0, // ReplicateS: 286
  0, // ReplicateI: 287
  0, // ReplicateL: 288
  0, // ReplicateF: 289
  0, // ReplicateD: 290
  0, // Extract: 291
  0, // ExtractB: 292
  0, // ExtractUB: 293
  0, // ExtractC: 294
  0, // ExtractS: 295
  0, // ExtractI: 296
  0, // ExtractL: 297
  0, // ExtractF: 298
  0 // no trailing comma // ExtractD: 299
};
//  The following instructions can cisc-spill



// An array of character pointers to machine register names.
const char *Matcher::regName[REG_COUNT] = {
  "R14",
  "R14_H",
  "R15",
  "R15_H",
  "R17",
  "R17_H",
  "R18",
  "R18_H",
  "R19",
  "R19_H",
  "R20",
  "R20_H",
  "R21",
  "R21_H",
  "R22",
  "R22_H",
  "R23",
  "R23_H",
  "R24",
  "R24_H",
  "R25",
  "R25_H",
  "R26",
  "R26_H",
  "R27",
  "R27_H",
  "R28",
  "R28_H",
  "R29",
  "R29_H",
  "R30",
  "R30_H",
  "R31",
  "R31_H",
  "R11",
  "R11_H",
  "R12",
  "R12_H",
  "R10",
  "R10_H",
  "R9",
  "R9_H",
  "R8",
  "R8_H",
  "R7",
  "R7_H",
  "R6",
  "R6_H",
  "R5",
  "R5_H",
  "R4",
  "R4_H",
  "R3",
  "R3_H",
  "R16",
  "R16_H",
  "R13",
  "R13_H",
  "R2",
  "R2_H",
  "R1",
  "R1_H",
  "R0",
  "R0_H",
  "F0",
  "F0_H",
  "F13",
  "F13_H",
  "F12",
  "F12_H",
  "F11",
  "F11_H",
  "F10",
  "F10_H",
  "F9",
  "F9_H",
  "F8",
  "F8_H",
  "F7",
  "F7_H",
  "F6",
  "F6_H",
  "F5",
  "F5_H",
  "F4",
  "F4_H",
  "F3",
  "F3_H",
  "F2",
  "F2_H",
  "F1",
  "F1_H",
  "F14",
  "F14_H",
  "F15",
  "F15_H",
  "F16",
  "F16_H",
  "F17",
  "F17_H",
  "F18",
  "F18_H",
  "F19",
  "F19_H",
  "F20",
  "F20_H",
  "F21",
  "F21_H",
  "F22",
  "F22_H",
  "F23",
  "F23_H",
  "F24",
  "F24_H",
  "F25",
  "F25_H",
  "F26",
  "F26_H",
  "F27",
  "F27_H",
  "F28",
  "F28_H",
  "F29",
  "F29_H",
  "F30",
  "F30_H",
  "F31",
  "F31_H",
  "CCR0",
  "CCR1",
  "CCR2",
  "CCR3",
  "CCR4",
  "CCR5",
  "CCR6",
  "CCR7",
  "SR_XER",
  "SR_LR",
  "SR_CTR",
  "SR_VRSAVE",
  "SR_SPEFSCR",
  "SR_PPR" // no trailing comma
};

// An array of character pointers to machine register names.
const VMReg OptoReg::opto2vm[REG_COUNT] = {
	R14->as_VMReg()        ,
	R14->as_VMReg()->next(),
	R15->as_VMReg()        ,
	R15->as_VMReg()->next(),
	R17->as_VMReg()        ,
	R17->as_VMReg()->next(),
	R18->as_VMReg()        ,
	R18->as_VMReg()->next(),
	R19->as_VMReg()        ,
	R19->as_VMReg()->next(),
	R20->as_VMReg()        ,
	R20->as_VMReg()->next(),
	R21->as_VMReg()        ,
	R21->as_VMReg()->next(),
	R22->as_VMReg()        ,
	R22->as_VMReg()->next(),
	R23->as_VMReg()        ,
	R23->as_VMReg()->next(),
	R24->as_VMReg()        ,
	R24->as_VMReg()->next(),
	R25->as_VMReg()        ,
	R25->as_VMReg()->next(),
	R26->as_VMReg()        ,
	R26->as_VMReg()->next(),
	R27->as_VMReg()        ,
	R27->as_VMReg()->next(),
	R28->as_VMReg()        ,
	R28->as_VMReg()->next(),
	R29->as_VMReg()        ,
	R29->as_VMReg()->next(),
	R30->as_VMReg()        ,
	R30->as_VMReg()->next(),
	R31->as_VMReg()        ,
	R31->as_VMReg()->next(),
	R11->as_VMReg()        ,
	R11->as_VMReg()->next(),
	R12->as_VMReg()        ,
	R12->as_VMReg()->next(),
	R10->as_VMReg()        ,
	R10->as_VMReg()->next(),
	R9->as_VMReg()         ,
	R9->as_VMReg()->next() ,
	R8->as_VMReg()         ,
	R8->as_VMReg()->next() ,
	R7->as_VMReg()         ,
	R7->as_VMReg()->next() ,
	R6->as_VMReg()         ,
	R6->as_VMReg()->next() ,
	R5->as_VMReg()         ,
	R5->as_VMReg()->next() ,
	R4->as_VMReg()         ,
	R4->as_VMReg()->next() ,
	R3->as_VMReg()         ,
	R3->as_VMReg()->next() ,
	R16->as_VMReg()        ,
	R16->as_VMReg()->next(),
	R13->as_VMReg()        ,
	R13->as_VMReg()->next(),
	R2->as_VMReg()         ,
	R2->as_VMReg()->next() ,
	R1->as_VMReg()         ,
	R1->as_VMReg()->next() ,
	R0->as_VMReg()         ,
	R0->as_VMReg()->next() ,
	F0->as_VMReg()         ,
	F0->as_VMReg()->next() ,
	F13->as_VMReg()        ,
	F13->as_VMReg()->next(),
	F12->as_VMReg()        ,
	F12->as_VMReg()->next(),
	F11->as_VMReg()        ,
	F11->as_VMReg()->next(),
	F10->as_VMReg()        ,
	F10->as_VMReg()->next(),
	F9->as_VMReg()         ,
	F9->as_VMReg()->next() ,
	F8->as_VMReg()         ,
	F8->as_VMReg()->next() ,
	F7->as_VMReg()         ,
	F7->as_VMReg()->next() ,
	F6->as_VMReg()         ,
	F6->as_VMReg()->next() ,
	F5->as_VMReg()         ,
	F5->as_VMReg()->next() ,
	F4->as_VMReg()         ,
	F4->as_VMReg()->next() ,
	F3->as_VMReg()         ,
	F3->as_VMReg()->next() ,
	F2->as_VMReg()         ,
	F2->as_VMReg()->next() ,
	F1->as_VMReg()         ,
	F1->as_VMReg()->next() ,
	F14->as_VMReg()        ,
	F14->as_VMReg()->next(),
	F15->as_VMReg()        ,
	F15->as_VMReg()->next(),
	F16->as_VMReg()        ,
	F16->as_VMReg()->next(),
	F17->as_VMReg()        ,
	F17->as_VMReg()->next(),
	F18->as_VMReg()        ,
	F18->as_VMReg()->next(),
	F19->as_VMReg()        ,
	F19->as_VMReg()->next(),
	F20->as_VMReg()        ,
	F20->as_VMReg()->next(),
	F21->as_VMReg()        ,
	F21->as_VMReg()->next(),
	F22->as_VMReg()        ,
	F22->as_VMReg()->next(),
	F23->as_VMReg()        ,
	F23->as_VMReg()->next(),
	F24->as_VMReg()        ,
	F24->as_VMReg()->next(),
	F25->as_VMReg()        ,
	F25->as_VMReg()->next(),
	F26->as_VMReg()        ,
	F26->as_VMReg()->next(),
	F27->as_VMReg()        ,
	F27->as_VMReg()->next(),
	F28->as_VMReg()        ,
	F28->as_VMReg()->next(),
	F29->as_VMReg()        ,
	F29->as_VMReg()->next(),
	F30->as_VMReg()        ,
	F30->as_VMReg()->next(),
	F31->as_VMReg()        ,
	F31->as_VMReg()->next(),
	CCR0->as_VMReg(),
	CCR1->as_VMReg(),
	CCR2->as_VMReg(),
	CCR3->as_VMReg(),
	CCR4->as_VMReg(),
	CCR5->as_VMReg(),
	CCR6->as_VMReg(),
	CCR7->as_VMReg(),
	SR_XER->as_VMReg(),
	SR_LR->as_VMReg(),
	SR_CTR->as_VMReg(),
	SR_VRSAVE->as_VMReg(),
	SR_SPEFSCR->as_VMReg(),
	SR_PPR->as_VMReg() // no trailing comma
	};

 OptoReg::Name OptoReg::vm2opto[ConcreteRegisterImpl::number_of_registers];

// An array of the machine register encode values
const unsigned char Matcher::_regEncode[REG_COUNT] = {
  (unsigned char)'\xE',  // R14
  (unsigned char)'\x63',  // R14_H
  (unsigned char)'\xF',  // R15
  (unsigned char)'\x63',  // R15_H
  (unsigned char)'\x11',  // R17
  (unsigned char)'\x63',  // R17_H
  (unsigned char)'\x12',  // R18
  (unsigned char)'\x63',  // R18_H
  (unsigned char)'\x13',  // R19
  (unsigned char)'\x63',  // R19_H
  (unsigned char)'\x14',  // R20
  (unsigned char)'\x63',  // R20_H
  (unsigned char)'\x15',  // R21
  (unsigned char)'\x63',  // R21_H
  (unsigned char)'\x16',  // R22
  (unsigned char)'\x63',  // R22_H
  (unsigned char)'\x17',  // R23
  (unsigned char)'\x63',  // R23_H
  (unsigned char)'\x18',  // R24
  (unsigned char)'\x63',  // R24_H
  (unsigned char)'\x19',  // R25
  (unsigned char)'\x63',  // R25_H
  (unsigned char)'\x1A',  // R26
  (unsigned char)'\x63',  // R26_H
  (unsigned char)'\x1B',  // R27
  (unsigned char)'\x63',  // R27_H
  (unsigned char)'\x1C',  // R28
  (unsigned char)'\x63',  // R28_H
  (unsigned char)'\x1D',  // R29
  (unsigned char)'\x63',  // R29_H
  (unsigned char)'\x1E',  // R30
  (unsigned char)'\x63',  // R30_H
  (unsigned char)'\x1F',  // R31
  (unsigned char)'\x63',  // R31_H
  (unsigned char)'\xB',  // R11
  (unsigned char)'\x63',  // R11_H
  (unsigned char)'\xC',  // R12
  (unsigned char)'\x63',  // R12_H
  (unsigned char)'\xA',  // R10
  (unsigned char)'\x63',  // R10_H
  (unsigned char)'\x9',  // R9
  (unsigned char)'\x63',  // R9_H
  (unsigned char)'\x8',  // R8
  (unsigned char)'\x63',  // R8_H
  (unsigned char)'\x7',  // R7
  (unsigned char)'\x63',  // R7_H
  (unsigned char)'\x6',  // R6
  (unsigned char)'\x63',  // R6_H
  (unsigned char)'\x5',  // R5
  (unsigned char)'\x63',  // R5_H
  (unsigned char)'\x4',  // R4
  (unsigned char)'\x63',  // R4_H
  (unsigned char)'\x3',  // R3
  (unsigned char)'\x63',  // R3_H
  (unsigned char)'\x10',  // R16
  (unsigned char)'\x63',  // R16_H
  (unsigned char)'\xD',  // R13
  (unsigned char)'\x63',  // R13_H
  (unsigned char)'\x2',  // R2
  (unsigned char)'\x63',  // R2_H
  (unsigned char)'\x1',  // R1
  (unsigned char)'\x63',  // R1_H
  (unsigned char)'\x0',  // R0
  (unsigned char)'\x63',  // R0_H
  (unsigned char)'\x0',  // F0
  (unsigned char)'\x63',  // F0_H
  (unsigned char)'\xD',  // F13
  (unsigned char)'\x63',  // F13_H
  (unsigned char)'\xC',  // F12
  (unsigned char)'\x63',  // F12_H
  (unsigned char)'\xB',  // F11
  (unsigned char)'\x63',  // F11_H
  (unsigned char)'\xA',  // F10
  (unsigned char)'\x63',  // F10_H
  (unsigned char)'\x9',  // F9
  (unsigned char)'\x63',  // F9_H
  (unsigned char)'\x8',  // F8
  (unsigned char)'\x63',  // F8_H
  (unsigned char)'\x7',  // F7
  (unsigned char)'\x63',  // F7_H
  (unsigned char)'\x6',  // F6
  (unsigned char)'\x63',  // F6_H
  (unsigned char)'\x5',  // F5
  (unsigned char)'\x63',  // F5_H
  (unsigned char)'\x4',  // F4
  (unsigned char)'\x63',  // F4_H
  (unsigned char)'\x3',  // F3
  (unsigned char)'\x63',  // F3_H
  (unsigned char)'\x2',  // F2
  (unsigned char)'\x63',  // F2_H
  (unsigned char)'\x1',  // F1
  (unsigned char)'\x63',  // F1_H
  (unsigned char)'\xE',  // F14
  (unsigned char)'\x63',  // F14_H
  (unsigned char)'\xF',  // F15
  (unsigned char)'\x63',  // F15_H
  (unsigned char)'\x10',  // F16
  (unsigned char)'\x63',  // F16_H
  (unsigned char)'\x11',  // F17
  (unsigned char)'\x63',  // F17_H
  (unsigned char)'\x12',  // F18
  (unsigned char)'\x63',  // F18_H
  (unsigned char)'\x13',  // F19
  (unsigned char)'\x63',  // F19_H
  (unsigned char)'\x14',  // F20
  (unsigned char)'\x63',  // F20_H
  (unsigned char)'\x15',  // F21
  (unsigned char)'\x63',  // F21_H
  (unsigned char)'\x16',  // F22
  (unsigned char)'\x63',  // F22_H
  (unsigned char)'\x17',  // F23
  (unsigned char)'\x63',  // F23_H
  (unsigned char)'\x18',  // F24
  (unsigned char)'\x63',  // F24_H
  (unsigned char)'\x19',  // F25
  (unsigned char)'\x63',  // F25_H
  (unsigned char)'\x1A',  // F26
  (unsigned char)'\x63',  // F26_H
  (unsigned char)'\x1B',  // F27
  (unsigned char)'\x63',  // F27_H
  (unsigned char)'\x1C',  // F28
  (unsigned char)'\x63',  // F28_H
  (unsigned char)'\x1D',  // F29
  (unsigned char)'\x63',  // F29_H
  (unsigned char)'\x1E',  // F30
  (unsigned char)'\x63',  // F30_H
  (unsigned char)'\x1F',  // F31
  (unsigned char)'\x63',  // F31_H
  (unsigned char)'\x0',  // CCR0
  (unsigned char)'\x1',  // CCR1
  (unsigned char)'\x2',  // CCR2
  (unsigned char)'\x3',  // CCR3
  (unsigned char)'\x4',  // CCR4
  (unsigned char)'\x5',  // CCR5
  (unsigned char)'\x6',  // CCR6
  (unsigned char)'\x7',  // CCR7
  (unsigned char)'\x0',  // SR_XER
  (unsigned char)'\x1',  // SR_LR
  (unsigned char)'\x2',  // SR_CTR
  (unsigned char)'\x3',  // SR_VRSAVE
  (unsigned char)'\x4',  // SR_SPEFSCR
  (unsigned char)'\x5' // no trailing comma  // SR_PPR
};


//------------------Define classes derived from MachOper---------------------
MachOper  *labelOper::clone(Compile* C) const {
  return  new (C) labelOper(_label, _block_num);
}
uint labelOper::opcode() const { return LABEL; }

const RegMask *sRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

MachOper  *methodOper::clone(Compile* C) const {
  return  new (C) methodOper(_method);
}
uint methodOper::opcode() const { return METHOD; }

const RegMask *iRegIdstOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS32_REG_RW_mask();
}

const RegMask *iRegIsrcOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS32_REG_RO_mask();
}

const RegMask *rscratch1RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RSCRATCH1_BITS32_REG_mask();
}

const RegMask *rscratch2RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RSCRATCH2_BITS32_REG_mask();
}

const RegMask *rarg1RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RARG1_BITS32_REG_mask();
}

const RegMask *rarg2RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RARG2_BITS32_REG_mask();
}

const RegMask *rarg3RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RARG3_BITS32_REG_mask();
}

const RegMask *rarg4RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RARG4_BITS32_REG_mask();
}

const RegMask *rarg1RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RARG1_BITS64_REG_mask();
}

const RegMask *rarg2RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RARG2_BITS64_REG_mask();
}

const RegMask *rarg3RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RARG3_BITS64_REG_mask();
}

const RegMask *rarg4RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RARG4_BITS64_REG_mask();
}

const RegMask *iRegPdstOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS64_REG_RW_mask();
}

const RegMask *iRegPdstNoScratchOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS64_REG_LEAF_CALL_mask();
}

const RegMask *iRegPsrcOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS64_REG_RO_mask();
}

const RegMask *threadRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &THREAD_BITS64_REG_mask();
}

const RegMask *rscratch1RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RSCRATCH1_BITS64_REG_mask();
}

const RegMask *rscratch2RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RSCRATCH2_BITS64_REG_mask();
}

const RegMask *rarg1RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RARG1_BITS64_REG_mask();
}

const RegMask *rarg2RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RARG2_BITS64_REG_mask();
}

const RegMask *rarg3RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RARG3_BITS64_REG_mask();
}

const RegMask *rarg4RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RARG4_BITS64_REG_mask();
}

const RegMask *iRegNsrcOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS32_REG_RO_mask();
}

const RegMask *iRegNdstOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS32_REG_RW_mask();
}

const RegMask *iRegLdstOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS64_REG_RW_mask();
}

const RegMask *iRegLsrcOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS64_REG_RO_mask();
}

const RegMask *iRegL2IsrcOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS64_REG_RO_mask();
}

const RegMask *rscratch1RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RSCRATCH1_BITS64_REG_mask();
}

const RegMask *rscratch2RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RSCRATCH2_BITS64_REG_mask();
}

const RegMask *flagsRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsRegCR0Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_CR0_mask();
}

const RegMask *flagsRegCR1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_CR1_mask();
}

const RegMask *flagsRegCR6Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_CR6_mask();
}

const RegMask *regCTROper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &CTR_REG_mask();
}

const RegMask *regDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DBL_REG_mask();
}

const RegMask *regFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLT_REG_mask();
}

const RegMask *inline_cache_regPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R19_BITS64_REG_mask();
}

const RegMask *compiler_method_oop_regPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RSCRATCH1_BITS64_REG_mask();
}

const RegMask *interpreter_method_oop_regPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R19_BITS64_REG_mask();
}

const RegMask *iRegP2NOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS64_REG_RO_mask();
}

const RegMask *iRegN2POper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS32_REG_RO_mask();
}

const RegMask *indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS64_REG_RO_mask();
}

const RegMask *indOffset16Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS64_REG_RO_mask();
}

const RegMask *indOffset16Alg4Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS64_REG_RO_mask();
}

const RegMask *indirectNarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS32_REG_RO_mask();
}

const RegMask *indOffset16NarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS32_REG_RO_mask();
}

const RegMask *indOffset16NarrowAlg4Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &BITS32_REG_RO_mask();
}

const RegMask *stackSlotIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

//------------------Define members for classes derived from MachNode----------

// Copy _idx, inputs and operands to new node
void MachNode::fill_new_machnode( MachNode* node, Compile* C) const {
  // This architecture does not have cisc or short branch instructions
  ShouldNotCallThis();
}

void rangeCheck_iReg_uimm15Node::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void rangeCheck_iReg_uimm15Node::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void rangeCheck_iReg_iRegNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void rangeCheck_iReg_iRegNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void rangeCheck_uimm15_iRegNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void rangeCheck_uimm15_iRegNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void zeroCheckN_iReg_imm0Node::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void zeroCheckN_iReg_imm0Node::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void zeroCheckP_reg_imm0Node::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void zeroCheckP_reg_imm0Node::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchConNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchConNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchConFarNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchConFarNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchConSchedNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchConSchedNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchLoopEndNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchLoopEndNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchLoopEndFarNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchLoopEndFarNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchLoopEndSchedNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchLoopEndSchedNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void CallStaticJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallStaticJavaDirectHandleNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallDynamicJavaDirectSchedNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallDynamicJavaDirectSched_ExNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallDynamicJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallRuntimeDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafDirect_ExNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafNoFPDirect_ExNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

int loadUB_indirectNode::reloc() const {
  return 1;
}

int loadUB_indirect_acNode::reloc() const {
  return 1;
}

int loadB_indirect_ExNode::reloc() const {
  return 1;
}

int loadB_indirect_ac_ExNode::reloc() const {
  return 1;
}

int loadUBNode::reloc() const {
  return 1;
}

int loadUB_acNode::reloc() const {
  return 1;
}

int loadUB2LNode::reloc() const {
  return 1;
}

int loadUB2L_acNode::reloc() const {
  return 1;
}

int loadSNode::reloc() const {
  return 1;
}

int loadS_acNode::reloc() const {
  return 1;
}

int loadUSNode::reloc() const {
  return 1;
}

int loadUS_acNode::reloc() const {
  return 1;
}

int loadUS2LNode::reloc() const {
  return 1;
}

int loadUS2L_acNode::reloc() const {
  return 1;
}

int loadINode::reloc() const {
  return 1;
}

int loadI_acNode::reloc() const {
  return 1;
}

int loadUI2LNode::reloc() const {
  return 1;
}

int loadI2LNode::reloc() const {
  return 1;
}

int loadI2L_acNode::reloc() const {
  return 1;
}

int loadLNode::reloc() const {
  return 1;
}

int loadL_acNode::reloc() const {
  return 1;
}

int loadL_unalignedNode::reloc() const {
  return 1;
}

int loadV8Node::reloc() const {
  return 1;
}

int loadRangeNode::reloc() const {
  return 1;
}

int loadNNode::reloc() const {
  return 1;
}

int loadN_acNode::reloc() const {
  return 1;
}

int loadN2P_unscaledNode::reloc() const {
  return 1;
}

int loadPNode::reloc() const {
  return 1;
}

int loadP_acNode::reloc() const {
  return 1;
}

int loadP2XNode::reloc() const {
  return 1;
}

int loadNKlassNode::reloc() const {
  return 1;
}

int loadKlassNode::reloc() const {
  return 1;
}

int loadFNode::reloc() const {
  return 1;
}

int loadF_acNode::reloc() const {
  return 1;
}

int loadDNode::reloc() const {
  return 1;
}

int loadD_acNode::reloc() const {
  return 1;
}

int loadD_unalignedNode::reloc() const {
  return 1;
}

int loadConP0or1Node::reloc() const {
  return 1;
}

int loadConPNode::reloc() const {
  return 1;
}

int loadConP_loNode::reloc() const {
  return 1;
}

int loadConP_ExNode::reloc() const {
  return 1;
}

int loadConFNode::reloc() const {
  return 1;
}

int loadConFCompNode::reloc() const {
  return 1;
}

int loadConF_ExNode::reloc() const {
  return 2;
}

int loadConDNode::reloc() const {
  return 1;
}

int loadConDCompNode::reloc() const {
  return 1;
}

int loadConD_ExNode::reloc() const {
  return 2;
}

int prefetchrNode::reloc() const {
  return 1;
}

int prefetchr_no_offsetNode::reloc() const {
  return 1;
}

int prefetchwNode::reloc() const {
  return 1;
}

int prefetchw_no_offsetNode::reloc() const {
  return 1;
}

int prefetch_alloc_zeroNode::reloc() const {
  return 1;
}

int prefetch_alloc_zero_no_offsetNode::reloc() const {
  return 1;
}

int prefetch_allocNode::reloc() const {
  return 1;
}

int prefetch_alloc_no_offsetNode::reloc() const {
  return 1;
}

int storeBNode::reloc() const {
  return 2;
}

int storeCNode::reloc() const {
  return 2;
}

int storeINode::reloc() const {
  return 2;
}

int storeI_convL2INode::reloc() const {
  return 2;
}

int storeLNode::reloc() const {
  return 2;
}

int storeA8BNode::reloc() const {
  return 2;
}

int storeNNode::reloc() const {
  return 3;
}

int storePNode::reloc() const {
  return 2;
}

int storeFNode::reloc() const {
  return 2;
}

int storeDNode::reloc() const {
  return 2;
}

int storeCM_CMSNode::reloc() const {
  return 2;
}

int storeCM_CMS_ExExNode::reloc() const {
  return 2;
}

int storeCM_G1Node::reloc() const {
  return 2;
}

int cmovP_regNode::reloc() const {
  return 1;
}

int cmovP_immNode::reloc() const {
  return 1;
}

int loadPLockedNode::reloc() const {
  return 1;
}

int addI_reg_regNode::reloc() const {
  return 2;
}

int addP_reg_regNode::reloc() const {
  return 1;
}

int addP_reg_imm16Node::reloc() const {
  return 1;
}

int addP_reg_immhi16Node::reloc() const {
  return 1;
}

int shrP_convP2X_reg_imm6Node::reloc() const {
  return 1;
}

int castP2XNode::reloc() const {
  return 1;
}

int convP2Bool_reg__cntlz_ExNode::reloc() const {
  return 1;
}

int convP2Bool_reg__cmoveNode::reloc() const {
  return 1;
}

int xorI_convP2Bool_reg__cntlz_ExNode::reloc() const {
  return 1;
}

int xorI_convP2Bool_reg_immIvalue1__cmoveNode::reloc() const {
  return 1;
}

int zeroCheckP_reg_imm0Node::reloc() const {
  return 2;
}

int cmpP_reg_regNode::reloc() const {
  return 2;
}

int partialSubtypeCheckNode::reloc() const {
  return 2;
}

int string_indexOf_imm1_charNode::reloc() const {
  return 1;
}

int repl2F_immF_ExNode::reloc() const {
  return 1;
}

int repl2F_immF0Node::reloc() const {
  return 1;
}

int safePoint_pollNode::reloc() const {
  return 1;
}

int loadConPollAddrNode::reloc() const {
  return 1;
}

int safePoint_poll_conPollAddrNode::reloc() const {
  return 1;
}

int CallStaticJavaDirectNode::reloc() const {
  return 1;
}

int CallStaticJavaDirectHandleNode::reloc() const {
  return 1;
}

int CallDynamicJavaDirectSchedNode::reloc() const {
  return 1;
}

int CallDynamicJavaDirectSched_ExNode::reloc() const {
  return 1;
}

int CallDynamicJavaDirectNode::reloc() const {
  return 1;
}

int CallRuntimeDirectNode::reloc() const {
  return 1;
}

int CallLeafDirectNode::reloc() const {
  return 1;
}

int CallLeafDirect_ExNode::reloc() const {
  return 1;
}

int CallLeafNoFPDirect_ExNode::reloc() const {
  return 1;
}

int TailCalljmpIndNode::reloc() const {
  return 1;
}

int RetNode::reloc() const {
  return 1;
}

int tailjmpIndNode::reloc() const {
  return 1;
}

int RethrowExceptionNode::reloc() const {
  return 1;
}


void convB2I_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2882 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_extsb);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ extsb(Rdst, Rsrc1);
  
#line 7105 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convB2I_reg_2Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadUB_indirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3238 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lbz);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lbz(Rdst, Idisp, Rbase);
  
#line 7133 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadUB_indirectNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadUB_indirect_acNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3251 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lbz(Rdst, Idisp, Rbase);
    __ twi_0(Rdst);
    __ isync();
  
#line 7163 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadUB_indirect_acNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void loadUB_indOffset16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3238 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lbz);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lbz(Rdst, Idisp, Rbase);
  
#line 7191 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadUB_indOffset16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadUB_indOffset16_acNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3251 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lbz(Rdst, Idisp, Rbase);
    __ twi_0(Rdst);
    __ isync();
  
#line 7221 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadUB_indOffset16_acNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void loadUBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3238 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lbz);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lbz(Rdst, Idisp, Rbase);
  
#line 7249 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadUBNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadUB_acNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3251 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lbz(Rdst, Idisp, Rbase);
    __ twi_0(Rdst);
    __ isync();
  
#line 7279 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadUB_acNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void loadUB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3238 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lbz);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lbz(Rdst, Idisp, Rbase);
  
#line 7307 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadUB2LNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadUB2L_acNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3251 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lbz(Rdst, Idisp, Rbase);
    __ twi_0(Rdst);
    __ isync();
  
#line 7337 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadUB2L_acNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void loadSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3211 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lha);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lha(Rdst, Idisp, Rbase);
  
#line 7365 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadSNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadS_acNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3224 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lha(Rdst, Idisp, Rbase);
    __ twi_0(Rdst);
    __ isync();
  
#line 7395 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadS_acNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void loadUSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3182 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lhz);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lhz(Rdst, Idisp, Rbase);
  
#line 7424 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadUSNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadUS_acNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3196 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lhz(Rdst, Idisp, Rbase);
    __ twi_0(Rdst);
    __ isync();
  
#line 7455 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadUS_acNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void loadUS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3182 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lhz);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lhz(Rdst, Idisp, Rbase);
  
#line 7484 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadUS2LNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadUS2L_acNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3196 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lhz(Rdst, Idisp, Rbase);
    __ twi_0(Rdst);
    __ isync();
  
#line 7515 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadUS2L_acNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void loadINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3124 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lwz);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lwz(Rdst, Idisp, Rbase);
  
#line 7544 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadI_acNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3138 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lwz(Rdst, Idisp, Rbase);
    __ twi_0(Rdst); // SAPJVM MD 2011-10-06 replaced cmp-br-isync by twi-isync
    __ isync();
  
#line 7575 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadI_acNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void loadUI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 3124 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lwz);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lwz(Rdst, Idisp, Rbase);
  
#line 7605 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadUI2LNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3154 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lwa);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes() */;
    }
    __ lwa(Rdst, Idisp, Rbase);
  
#line 7634 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadI2LNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadI2L_acNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3167 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lwa);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lwa(Rdst, Idisp, Rbase);
    __ twi_0(Rdst);
    __ isync();
  
#line 7665 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadI2L_acNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void loadLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3265 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ld);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ ld(Rdst, Idisp, Rbase);
  
#line 7695 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadL_acNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3280 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ ld(Rdst, Idisp, Rbase);
    __ twi_0(Rdst);
    __ isync();
  
#line 7727 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadL_acNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void loadL_unalignedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3265 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ld);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ ld(Rdst, Idisp, Rbase);
  
#line 7757 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadL_unalignedNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadV8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3265 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ld);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ ld(Rdst, Idisp, Rbase);
  
#line 7787 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadV8Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadRangeNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3124 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lwz);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lwz(Rdst, Idisp, Rbase);
  
#line 7816 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadRangeNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3124 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lwz);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lwz(Rdst, Idisp, Rbase);
  
#line 7845 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadNNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadN_acNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3138 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lwz(Rdst, Idisp, Rbase);
    __ twi_0(Rdst); // SAPJVM MD 2011-10-06 replaced cmp-br-isync by twi-isync
    __ isync();
  
#line 7876 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadN_acNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void loadN2P_unscaledNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3124 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lwz);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lwz(Rdst, Idisp, Rbase);
  
#line 7905 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadN2P_unscaledNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3265 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ld);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ ld(Rdst, Idisp, Rbase);
  
#line 7935 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadP_acNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3280 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ ld(Rdst, Idisp, Rbase);
    __ twi_0(Rdst);
    __ isync();
  
#line 7967 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadP_acNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void loadP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3265 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ld);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ ld(Rdst, Idisp, Rbase);
  
#line 7997 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadP2XNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadNKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3124 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lwz);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lwz(Rdst, Idisp, Rbase);
  
#line 8026 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadNKlassNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3265 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ld);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ ld(Rdst, Idisp, Rbase);
  
#line 8056 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadKlassNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3296 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lfs);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lfs(Rdst, Idisp, Rbase);
  
#line 8084 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadFNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadF_acNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3309 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    Label next;
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lfs(Rdst, Idisp, Rbase);
    __ fcmpu(CCR0, Rdst, Rdst);
    __ bne(CCR0, next);
    __ bind(next);
    __ isync();
  
#line 8117 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadF_acNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 16, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 16);
}

void loadDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3326 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lfd);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lfd(Rdst, Idisp, Rbase);
  
#line 8145 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadDNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadD_acNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3339 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    Label next;
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lfd(Rdst, Idisp, Rbase);
    __ fcmpu(CCR0, Rdst, Rdst);
    __ bne(CCR0, next);
    __ bind(next);
    __ isync();
  
#line 8178 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadD_acNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 16, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 16);
}

void loadD_unalignedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3326 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lfd);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lfd(Rdst, Idisp, Rbase);
  
#line 8206 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadD_unalignedNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadToc_hiNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 3747 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addis);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);

    __ calculate_address_from_global_toc_hi16only(Rdst, __ method_toc());
  
#line 8231 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadToc_hiNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadToc_loNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3756 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ori);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);

    __ calculate_address_from_global_toc_lo16only(Rdst, __ method_toc());
  
#line 8256 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadToc_loNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConI16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 4042 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    int v = opnd_array(1)->constant();
    __ li(Rdst, (int)((short) (v & 0xFFFF)));
  
#line 8281 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConI16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConIhi16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 4051 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addis);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    int v = opnd_array(1)->constant();
    // Lis sign extends 16-bit src then shifts it 16 bit to the left.
    __ lis(Rdst, (int)((short) ((v & 0xFFFF0000) >> 16)));
  
#line 8307 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConIhi16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConI32_lo16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2759 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ori);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = (opnd_array(2)->constant()) & 0xFFFF;
    __ ori(Rdst, Rsrc1, Isrc2);
  
#line 8334 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConI32_lo16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConL16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 4042 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    int v = opnd_array(1)->constantL();
    __ li(Rdst, (int)((short) (v & 0xFFFF)));
  
#line 8359 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConL16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConL32hi16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 4051 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addis);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    int v = opnd_array(1)->constantL();
    // Lis sign extends 16-bit src then shifts it 16 bit to the left.
    __ lis(Rdst, (int)((short) ((v & 0xFFFF0000) >> 16)));
  
#line 8385 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConL32hi16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConL32_lo16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2759 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ori);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = (opnd_array(2)->constantL()) & 0xFFFF;
    __ ori(Rdst, Rsrc1, Isrc2);
  
#line 8412 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConL32_lo16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// toc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// isoop
  {

#line 4249 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ld);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rtoc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* toc */);
    int toc_offset = 0;
    assert(opnd_array(3)->constant() == 1 || opnd_array(3)->constant() == 0, "must be 1 or 0");

    if (!ra_->C->in_scratch_emit_size()) {
      address const_toc_addr;
      if (opnd_array(3)->constant() == 1) {
#if 0 // TODO: PPC port
        // Create an oop constant and a corresponding relocation.
        AddressLiteral a = __ allocate_oop_address((jobject)opnd_array(1)->constantL());
        const_toc_addr = __ address_constant((address)a.value(), RelocationHolder::none);
        __ relocate(a.rspec());
#endif
        Unimplemented();
      } else {
        // Create a non-oop constant, no relocation needed.
        const_toc_addr = __ long_constant((jlong)opnd_array(1)->constantL());
      }

      // Get the constant's TOC offset.
      toc_offset = __ offset_to_method_toc(const_toc_addr);

      // Keep the current instruction offset in mind.
      ((loadConLNode*)this)->_cbuf_insts_offset = __ offset();
    }

    __ ld(Rdst, toc_offset, Rtoc);
  
#line 8464 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConL_hiNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// toc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// isoop
  {

#line 4176 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addis);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rtoc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* toc */);
    assert(opnd_array(3)->constant() == 1 || opnd_array(3)->constant() == 0, "must be 1 or 0");

    if (!ra_->C->in_scratch_emit_size()) {
      address const_toc_addr;
      if (opnd_array(3)->constant() == 1) {
        // Create an oop constant and a corresponding relocation.
        AddressLiteral a = __ allocate_oop_address((jobject)opnd_array(1)->constantL());
        const_toc_addr = __ address_constant((address)a.value(), RelocationHolder::none);
        __ relocate(a.rspec());
      } else {
        // Create a non-oop constant, no relocation needed.
        const_toc_addr = __ long_constant((jlong)opnd_array(1)->constantL());
      }

      // Get the constant's TOC offset.
      const int toc_offset = __ offset_to_method_toc(const_toc_addr);
      // Store the toc offset of the constant.
      ((loadConL_hiNode*)this)->_const_toc_offset = toc_offset;

      // Also keep the current instruction offset in mind.
      ((loadConL_hiNode*)this)->_cbuf_insts_offset = __ offset();
    }

    __ addis(Rdst, Rtoc, MacroAssembler::largeoffset_si16_si16_hi(_const_toc_offset));
  
#line 8514 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConL_hiNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConL_loNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// isoop
  {

#line 4238 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ld);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* base */);

    int offset = ra_->C->in_scratch_emit_size() ? 0 : this->_const_toc_offset_hi_node->_const_toc_offset;
    __ ld(Rdst, MacroAssembler::largeoffset_si16_si16_lo(offset), Rbase);
  
#line 8543 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConL_loNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void  loadConL_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  // Access to ins and operands for late expand.
  unsigned idx_dst   = idx0; 	// iRegLdst, 	dst
  unsigned idx_src   = idx1; 	// immL, 	src
  unsigned idx_toc   = mach_constant_base_node_input(); 	// iRegPdst, 	constanttablebase
  Node    *n_region  = lookup(0);
  Node    *n_dst     = lookup(idx_dst);
  Node    *n_src     = lookup(idx_src);
  Node    *n_toc     = lookup(idx_toc);
  iRegLdstOper *op_dst = (iRegLdstOper *)opnd_array(0);
  immLOper *op_src = (immLOper *)opnd_array(1);
  Compile *C = ra_->C;
  {
#line 4062 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // Create new nodes.
    loadConLNodesTuple loadConLNodes =
      loadConLNodesTuple_create(C, ra_, n_toc, op_src, false, ra_->get_reg_second(this), ra_->get_reg_first(this));

    // Push new nodes.
    if (loadConLNodes._large_hi) nodes->push(loadConLNodes._large_hi);
    if (loadConLNodes._last)     nodes->push(loadConLNodes._last);

    // some asserts
    assert(nodes->length() >= 1, "must have created at least 1 node");
    assert(loadConLNodes._last->bottom_type()->isa_long(), "must be long");
  
#line 8582 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void loadConL_ExNode::eval_constant(Compile* C) {
  // User did not define an encode section.
}
void loadConN0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 4042 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    int v = opnd_array(1)->constant();
    __ li(Rdst, (int)((short) (v & 0xFFFF)));
  
#line 8605 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConN0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConN1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3766 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addis);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    int      Csrc = opnd_array(1)->constant();
    int xc = (Csrc >> 16) & 0xffff;
    int xd = (Csrc >>  0) & 0xffff;
    int value = (int)(short)(xc + ((xd & 0x8000) != 0 ? 1 : 0)); // Compensate sign extend like in patch_set_narrow_oop.
    __ lis(Rdst, value);
  
#line 8632 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConN1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConN2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3778 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);
    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int      Csrc  = opnd_array(2)->constant();
    assert(__ oop_recorder() != NULL, "this assembler needs an OopRecorder");
    int oop_index = __ oop_recorder()->find_index((jobject)opnd_array(2)->constant());
    RelocationHolder rspec = oop_Relocation::spec(oop_index);

    __ relocate(rspec, 1);
    __ addi(Rdst, Rsrc1, (int)(short)(Csrc & 0xffff));
  
#line 8663 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConN2Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void clearMs32bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3585 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicl);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    int iclearleftbits = (0x20);
    __ clrldi(Rdst, Rsrc, iclearleftbits);
  
#line 8689 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint clearMs32bNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void  loadConN_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  // Access to ins and operands for late expand.
  unsigned idx_dst   = idx0; 	// iRegNdst, 	dst
  unsigned idx_src   = idx1; 	// immN, 	src
  Node    *n_region  = lookup(0);
  Node    *n_dst     = lookup(idx_dst);
  Node    *n_src     = lookup(idx_src);
  iRegNdstOper *op_dst = (iRegNdstOper *)opnd_array(0);
  immNOper *op_src = (immNOper *)opnd_array(1);
  Compile *C = ra_->C;
  {
#line 4095 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    MachNode *m1 = new (C) loadConN1Node();
    MachNode *m2 = new (C) loadConN2Node();
    MachNode *m3 = new (C) clearMs32bNode();
    m1->add_req(NULL);
    m2->add_req(NULL, m1);
    m3->add_req(NULL, m2);
    m1->_opnds[0] = op_dst;
    m1->_opnds[1] = op_src;
    m2->_opnds[0] = op_dst;
    m2->_opnds[1] = op_dst;
    m2->_opnds[2] = op_src;
    m3->_opnds[0] = op_dst;
    m3->_opnds[1] = op_dst;
    ra_->set_pair(m1->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
    ra_->set_pair(m2->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
    ra_->set_pair(m3->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
    nodes->push(m1);
    nodes->push(m2);
    nodes->push(m3);
  
#line 8734 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void loadConP0or1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 4042 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    int v = opnd_array(1)->constant();
    __ li(Rdst, (int)((short) (v & 0xFFFF)));
  
#line 8754 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConP0or1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// toc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// isoop
  {

#line 4283 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ld);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rtoc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* toc */);
    int toc_offset = 0;
    assert(opnd_array(3)->constant() == 1 || opnd_array(3)->constant() == 0, "must be 1 or 0");

    if (!Compile::current()->in_scratch_emit_size()) {
      address const_toc_addr;
      if (opnd_array(3)->constant() == 1) {
#if 0 // TODO: PPC port
        // Create an oop constant and a corresponding relocation.
        AddressLiteral a = __ allocate_oop_address((jobject)opnd_array(1)->constant());
        const_toc_addr = __ address_constant((address)a.value(), RelocationHolder::none);
        __ relocate(a.rspec());
#endif
        Unimplemented();
      } else {
        // Create a non-oop constant, no relocation needed.
        const_toc_addr = __ long_constant((jlong)opnd_array(1)->constant());
      }

      // Get the constant's TOC offset.
      toc_offset = __ offset_to_method_toc(const_toc_addr);
    }

    __ ld(Rdst, toc_offset, Rtoc);
  
#line 8803 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConP_hiNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// toc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// isoop
  {

#line 4208 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addis);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rtoc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* toc */);
    assert(opnd_array(3)->constant() == 1 || opnd_array(3)->constant() == 0, "must be 1 or 0");

    if (!ra_->C->in_scratch_emit_size()) {
      address const_toc_addr;
      if (opnd_array(3)->constant() == 1) {
        // Create an oop constant and a corresponding relocation.
        AddressLiteral a = __ allocate_oop_address((jobject)opnd_array(1)->constant());
        const_toc_addr = __ address_constant((address)a.value(), RelocationHolder::none);
        __ relocate(a.rspec());
      } else {
        // Create a non-oop constant, no relocation needed.
        const_toc_addr = __ long_constant((jlong)opnd_array(1)->constant());
      }

      // Get the constant's TOC offset.
      const int toc_offset = __ offset_to_method_toc(const_toc_addr);

      // Store the toc offset of the constant.
      ((loadConP_hiNode*)this)->_const_toc_offset = toc_offset;
    }

    __ addis(Rdst, Rtoc, MacroAssembler::largeoffset_si16_si16_hi(_const_toc_offset));
  
#line 8851 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConP_hiNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConP_loNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// isoop
  {

#line 4238 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ld);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* base */);

    int offset = ra_->C->in_scratch_emit_size() ? 0 : this->_const_toc_offset_hi_node->_const_toc_offset;
    __ ld(Rdst, MacroAssembler::largeoffset_si16_si16_lo(offset), Rbase);
  
#line 8880 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConP_loNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void  loadConP_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  // Access to ins and operands for late expand.
  unsigned idx_dst   = idx0; 	// iRegPdst, 	dst
  unsigned idx_src   = idx1; 	// immP, 	src
  unsigned idx_toc   = mach_constant_base_node_input(); 	// iRegPdst, 	constanttablebase
  Node    *n_region  = lookup(0);
  Node    *n_dst     = lookup(idx_dst);
  Node    *n_src     = lookup(idx_src);
  Node    *n_toc     = lookup(idx_toc);
  iRegPdstOper *op_dst = (iRegPdstOper *)opnd_array(0);
  immPOper *op_src = (immPOper *)opnd_array(1);
  Compile *C = ra_->C;
  {
#line 4118 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // Create new nodes.
    loadConPNodesTuple loadConPNodes =
      loadConPNodesTuple_create(C, ra_, n_toc, op_src, ra_->get_reg_second(this), ra_->get_reg_first(this));

    // Push new nodes.
    if (loadConPNodes._large_hi) nodes->push(loadConPNodes._large_hi);
    if (loadConPNodes._last)     nodes->push(loadConPNodes._last);

    // some asserts
    assert(nodes->length() >= 1, "must have created at least 1 node");
    assert(loadConPNodes._last->bottom_type()->isa_ptr(), "must be ptr");
  
#line 8919 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void loadConP_ExNode::eval_constant(Compile* C) {
  // User did not define an encode section.
}
void loadConFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// toc
  {

#line 4314 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lfs);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    address float_address = __ float_constant(opnd_array(1)->constantF());
    Register Rtoc;
    Rtoc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* toc */);
    __ lfs(Rdst, __ offset_to_method_toc(float_address), Rtoc);
  
#line 8944 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConFNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConFCompNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// toc
  {

#line 4325 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    address float_address = __ float_constant(opnd_array(1)->constantF());
    int offset = __ offset_to_method_toc(float_address);
    Register Rtoc;
    Rtoc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* toc */);

    int hi = (offset + (1<<15))>>16;
    int lo = offset - hi * (1<<16);
    __ addis(Rtoc, Rtoc, hi);
    __ lfs(Rdst, lo, Rtoc);
    __ addis(Rtoc, Rtoc, -hi);
  
#line 8977 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConFCompNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void  loadConF_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  // Access to ins and operands for late expand.
  unsigned idx_dst   = idx0; 	// regF, 	dst
  unsigned idx_src   = idx1; 	// immF, 	src
  unsigned idx_toc   = mach_constant_base_node_input(); 	// iRegPdst, 	constanttablebase
  Node    *n_region  = lookup(0);
  Node    *n_dst     = lookup(idx_dst);
  Node    *n_src     = lookup(idx_src);
  Node    *n_toc     = lookup(idx_toc);
  regFOper *op_dst = (regFOper *)opnd_array(0);
  immFOper *op_src = (immFOper *)opnd_array(1);
  Compile *C = ra_->C;
  {
#line 4132 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    bool large_constant_pool = true; // TODO: PPC port C->cfg()->_consts_size > 4000;

    MachNode *m2;
    if (large_constant_pool) {
      m2 = new (C) loadConFCompNode();
    } else {
      m2 = new (C) loadConFNode();
    }
    // inputs for new nodes
    m2->add_req(NULL, n_toc);

    // operands for new nodes
    m2->_opnds[0] = op_dst;
    m2->_opnds[1] = op_src;
    m2->_opnds[2] = new (C) iRegPdstOper(); // constanttablebase

    // register allocation for new nodes
    ra_->set_pair(m2->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
    nodes->push(m2);
  
#line 9024 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void loadConF_ExNode::eval_constant(Compile* C) {
  // User did not define an encode section.
}
void loadConDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// toc
  {

#line 4341 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lfd);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    address float_address = __ double_constant(opnd_array(1)->constantD());
    Register Rtoc;
    Rtoc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* toc */);
    __ lfd(Rdst, __ offset_to_method_toc(float_address), Rtoc);
  
#line 9049 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConDNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConDCompNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// toc
  {

#line 4352 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    address float_address = __ double_constant(opnd_array(1)->constantD());
    int offset = __ offset_to_method_toc(float_address);
    Register Rtoc;
    Rtoc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* toc */);

    int hi = (offset + (1<<15))>>16;
    int lo = offset - hi * (1<<16);
    __ addis(Rtoc, Rtoc, hi);
    __ lfd(Rdst, lo, Rtoc);
    __ addis(Rtoc, Rtoc, -hi);
  
#line 9082 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadConDCompNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void  loadConD_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  // Access to ins and operands for late expand.
  unsigned idx_dst   = idx0; 	// regD, 	dst
  unsigned idx_src   = idx1; 	// immD, 	src
  unsigned idx_toc   = mach_constant_base_node_input(); 	// iRegPdst, 	constanttablebase
  Node    *n_region  = lookup(0);
  Node    *n_dst     = lookup(idx_dst);
  Node    *n_src     = lookup(idx_src);
  Node    *n_toc     = lookup(idx_toc);
  regDOper *op_dst = (regDOper *)opnd_array(0);
  immDOper *op_src = (immDOper *)opnd_array(1);
  Compile *C = ra_->C;
  {
#line 4154 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    bool large_constant_pool = true; // TODO: PPC port C->cfg()->_consts_size > 4000;

    MachNode *m2;
    if (large_constant_pool) {
      m2 = new (C) loadConDCompNode();
    } else {
      m2 = new (C) loadConDNode();
    }
    // inputs for new nodes
    m2->add_req(NULL, n_toc);

    // operands for new nodes
    m2->_opnds[0] = op_dst;
    m2->_opnds[1] = op_src;
    m2->_opnds[2] = new (C) iRegPdstOper(); // constanttablebase

    // register allocation for new nodes
    ra_->set_pair(m2->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
    nodes->push(m2);
  
#line 9129 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void loadConD_ExNode::eval_constant(Compile* C) {
  // User did not define an encode section.
}
void prefetchrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 4397 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_dcbt);
    MacroAssembler _masm(&cbuf);
    __ dcbt(reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */), reg_to_register_object(opnd_array(1)->base(ra_,this,idx1)));
  
#line 9150 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint prefetchrNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void prefetchr_no_offsetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {

#line 4403 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_dcbt);
    MacroAssembler _masm(&cbuf);
    __ dcbt(reg_to_register_object(opnd_array(1)->base(ra_,this,idx1)));
  
#line 9172 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint prefetchr_no_offsetNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void prefetchwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 4370 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_dcbtst);
    MacroAssembler _masm(&cbuf);
    __ dcbtst(reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */), reg_to_register_object(opnd_array(1)->base(ra_,this,idx1)));
  
#line 9195 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint prefetchwNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void prefetchw_no_offsetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {

#line 4376 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_dcbtst);
    MacroAssembler _masm(&cbuf);
    __ dcbtst(reg_to_register_object(opnd_array(1)->base(ra_,this,idx1)));
  
#line 9217 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint prefetchw_no_offsetNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void prefetch_alloc_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 4383 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_dcbtst);
    MacroAssembler _masm(&cbuf);
    __ dcbz(reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */), reg_to_register_object(opnd_array(1)->base(ra_,this,idx1)));
  
#line 9240 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint prefetch_alloc_zeroNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void prefetch_alloc_zero_no_offsetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {

#line 4389 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_dcbtst);
    MacroAssembler _masm(&cbuf);
    __ dcbz(reg_to_register_object(opnd_array(1)->base(ra_,this,idx1)));
  
#line 9262 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint prefetch_alloc_zero_no_offsetNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void prefetch_allocNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 4370 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_dcbtst);
    MacroAssembler _masm(&cbuf);
    __ dcbtst(reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */), reg_to_register_object(opnd_array(1)->base(ra_,this,idx1)));
  
#line 9285 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint prefetch_allocNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void prefetch_alloc_no_offsetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {

#line 4376 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_dcbtst);
    MacroAssembler _masm(&cbuf);
    __ dcbtst(reg_to_register_object(opnd_array(1)->base(ra_,this,idx1)));
  
#line 9307 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint prefetch_alloc_no_offsetNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void storeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3392 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_stb);
    MacroAssembler _masm(&cbuf);
    Register Rsrc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ stb(Rsrc, Idisp, Rbase);
  
#line 9336 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint storeBNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void storeCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3417 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_sth);
    MacroAssembler _masm(&cbuf);
    Register Rsrc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ sth(Rsrc, Idisp, Rbase);
  
#line 9365 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint storeCNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void storeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3380 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_stw);
    MacroAssembler _masm(&cbuf);
    Register Rsrc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ stw(Rsrc, Idisp, Rbase);
  
#line 9394 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint storeINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void storeI_convL2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3380 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_stw);
    MacroAssembler _masm(&cbuf);
    Register Rsrc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ stw(Rsrc, Idisp, Rbase);
  
#line 9423 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint storeI_convL2INode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void storeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3429 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_std);
    MacroAssembler _masm(&cbuf);
    Register Rsrc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ std(Rsrc, Idisp, Rbase);
  
#line 9454 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint storeLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void storeA8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3429 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_std);
    MacroAssembler _masm(&cbuf);
    Register Rsrc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ std(Rsrc, Idisp, Rbase);
  
#line 9485 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint storeA8BNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void storeNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3380 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_stw);
    MacroAssembler _masm(&cbuf);
    Register Rsrc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ stw(Rsrc, Idisp, Rbase);
  
#line 9514 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint storeNNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void storePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3429 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_std);
    MacroAssembler _masm(&cbuf);
    Register Rsrc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ std(Rsrc, Idisp, Rbase);
  
#line 9545 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint storePNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void storeFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3356 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_stfs);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rsrc = reg_to_FloatRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ stfs(Rsrc, Idisp, Rbase);
  
#line 9574 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint storeFNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void storeDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3368 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_stfd);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rsrc = reg_to_FloatRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ stfd(Rsrc, Idisp, Rbase);
  
#line 9603 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint storeDNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void storeCM_CMSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// releaseFieldAddr
  {

#line 2700 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    // FIXME: Implement this as a cmove and use a fixed condition code
    // register which is written on every transition to compiled code,
    // e.g. in call-stub and when returning from runtime stubs.
    //
    // Proposed code sequence for the cmove implementation:
    //
    // Label skip_release;
    // __ beq(CCRfixed, skip_release);
    // __ release();
    // __ bind(skip_release);
    // __ stb(card mark);

    MacroAssembler _masm(&cbuf);
    Label skip_release;

#if 0 // TODO: PPC port
    // Check CMSCollectorCardTableModRefBSExt::_requires_release and do the
    // release conditionally.
    __ lwz(R0, 0, reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* releaseFieldAddr */));
    __ cmpwi(CCR0, R0, 0);
    __ beq_predict_taken(CCR0, skip_release);
#endif
    __ li(R0, 0);
    __ release();
#if 0 // TODO: PPC port
    __ bind(skip_release);
#endif

    // Do the store.
    if (opnd_array(1)->index(ra_,this,idx1) == 0) {
      __ stb(R0, opnd_array(1)->disp(ra_,this,idx1), reg_to_register_object(opnd_array(1)->base(ra_,this,idx1)));
    } else {
      assert(0 == opnd_array(1)->disp(ra_,this,idx1), "no displacement possible with indexed load/stores on ppc");
      __ stbx(R0, reg_to_register_object(opnd_array(1)->base(ra_,this,idx1)), reg_to_register_object(opnd_array(1)->index(ra_,this,idx1)));
    }
  
#line 9659 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void storeCM_G1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 3405 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ li(R0, 0);
    __ stb(R0, Idisp, Rbase);
  
#line 9683 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint storeCM_G1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 8);
}

void encodeP_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3655 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicl);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    int isrc = (((Universe::narrow_oop_shift()))) & 0x3f;
    __ srdi(Rdst, Rsrc1, isrc);
  
#line 9709 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint encodeP_shiftNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void encodeP_subNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3000 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_subf);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object((R30_enc));
    Register Rsrc2 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ subf(Rdst, Rsrc1, Rsrc2);
  
#line 9735 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint encodeP_subNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cond_sub_baseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 5753 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmove);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* crx */);
    Label done;

    __ beq(Rcrx, done);
    __ sub(Rdst, Rsrc, R30);
#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required.
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 9773 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cond_sub_baseNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void cond_set_0_oopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 5721 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound); // see above

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* crx */);

    __ isel_0(Rdst, Rcrx, Assembler::equal, Rsrc);
  
#line 9801 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cond_set_0_oopNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void encodeP_32GAlignedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3576 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicl);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ rldicl(Rdst, Rsrc, 64-Universe::narrow_oop_shift(), 32);
  
#line 9826 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint encodeP_32GAlignedNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void  encodeP_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  // Access to ins and operands for late expand.
  unsigned idx_dst   = idx0; 	// iRegNdst, 	dst
  unsigned idx_src   = idx1; 	// iRegPdst, 	src
  unsigned idx_crx   = idx2; 	// flagsReg, 	crx
  Node    *n_region  = lookup(0);
  Node    *n_dst     = lookup(idx_dst);
  Node    *n_src     = lookup(idx_src);
  Node    *n_crx     = lookup(idx_crx);
  iRegNdstOper *op_dst = (iRegNdstOper *)opnd_array(0);
  iRegPdstOper *op_src = (iRegPdstOper *)opnd_array(1);
  flagsRegOper *op_crx = (flagsRegOper *)opnd_array(2);
  Compile *C = ra_->C;
  {
#line 4632 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"


    if (VM_Version::has_isel()) {
      // use isel instruction with Power 7
      cmpP_reg_imm16Node *n_compare  = new (C) cmpP_reg_imm16Node();
      encodeP_subNode    *n_sub_base = new (C) encodeP_subNode();
      encodeP_shiftNode  *n_shift    = new (C) encodeP_shiftNode();
      cond_set_0_oopNode *n_cond_set = new (C) cond_set_0_oopNode();

      n_compare->add_req(n_region, n_src);
      n_compare->_opnds[0] = op_crx;
      n_compare->_opnds[1] = op_src;
      n_compare->_opnds[2] = new (C) immL16Oper(0);

      n_sub_base->add_req(n_region, n_src);
      n_sub_base->_opnds[0] = op_dst;
      n_sub_base->_opnds[1] = op_src;
      n_sub_base->_bottom_type = _bottom_type;

      n_shift->add_req(n_region, n_sub_base);
      n_shift->_opnds[0] = op_dst;
      n_shift->_opnds[1] = op_dst;
      n_shift->_bottom_type = _bottom_type;

      n_cond_set->add_req(n_region, n_compare, n_shift);
      n_cond_set->_opnds[0] = op_dst;
      n_cond_set->_opnds[1] = op_crx;
      n_cond_set->_opnds[2] = op_dst;
      n_cond_set->_bottom_type = _bottom_type;

      ra_->set_pair(n_compare->_idx, ra_->get_reg_second(n_crx), ra_->get_reg_first(n_crx));
      ra_->set_pair(n_sub_base->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
      ra_->set_pair(n_shift->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
      ra_->set_pair(n_cond_set->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));

      nodes->push(n_compare);
      nodes->push(n_sub_base);
      nodes->push(n_shift);
      nodes->push(n_cond_set);

    } else {
      // before Power 7
      moveRegNode        *n_move     = new (C) moveRegNode();
      cmpP_reg_imm16Node *n_compare  = new (C) cmpP_reg_imm16Node();
      encodeP_shiftNode  *n_shift    = new (C) encodeP_shiftNode();
      cond_sub_baseNode  *n_sub_base = new (C) cond_sub_baseNode();

      n_move->add_req(n_region, n_src);
      n_move->_opnds[0] = op_dst;
      n_move->_opnds[1] = op_src;
      ra_->set_oop(n_move, true); // Until here, 'n_move' still produces an oop.

      n_compare->add_req(n_region, n_src);
      n_compare->add_prec(n_move);

      n_compare->_opnds[0] = op_crx;
      n_compare->_opnds[1] = op_src;
      n_compare->_opnds[2] = new (C) immL16Oper(0);

      n_sub_base->add_req(n_region, n_compare, n_src);
      n_sub_base->_opnds[0] = op_dst;
      n_sub_base->_opnds[1] = op_crx;
      n_sub_base->_opnds[2] = op_src;
      n_sub_base->_bottom_type = _bottom_type;

      n_shift->add_req(n_region, n_sub_base);
      n_shift->_opnds[0] = op_dst;
      n_shift->_opnds[1] = op_dst;
      n_shift->_bottom_type = _bottom_type;

      ra_->set_pair(n_shift->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
      ra_->set_pair(n_compare->_idx, ra_->get_reg_second(n_crx), ra_->get_reg_first(n_crx));
      ra_->set_pair(n_sub_base->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
      ra_->set_pair(n_move->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));

      nodes->push(n_move);
      nodes->push(n_compare);
      nodes->push(n_sub_base);
      nodes->push(n_shift);
    }

    assert(!(ra_->is_oop(this)), "sanity"); // This is not supposed to be GC'ed.
  
#line 9937 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void  encodeP_not_null_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  // Access to ins and operands for late expand.
  unsigned idx_dst   = idx0; 	// iRegNdst, 	dst
  unsigned idx_src   = idx1; 	// iRegPdst, 	src
  Node    *n_region  = lookup(0);
  Node    *n_dst     = lookup(idx_dst);
  Node    *n_src     = lookup(idx_src);
  iRegNdstOper *op_dst = (iRegNdstOper *)opnd_array(0);
  iRegPdstOper *op_src = (iRegPdstOper *)opnd_array(1);
  Compile *C = ra_->C;
  {
#line 4611 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"


    encodeP_subNode *n1 = new (C) encodeP_subNode();
    n1->add_req(n_region, n_src);
    n1->_opnds[0] = op_dst;
    n1->_opnds[1] = op_src;
    n1->_bottom_type = _bottom_type;

    encodeP_shiftNode *n2 = new (C) encodeP_shiftNode();
    n2->add_req(n_region, n1);
    n2->_opnds[0] = op_dst;
    n2->_opnds[1] = op_dst;
    n2->_bottom_type = _bottom_type;
    ra_->set_pair(n1->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
    ra_->set_pair(n2->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));

    nodes->push(n1);
    nodes->push(n2);
    assert(!(ra_->is_oop(this)), "sanity"); // This is not supposed to be GC'ed.
  
#line 9976 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void encodeP_not_null_base_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3655 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicl);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    int isrc = (((Universe::narrow_oop_shift()))) & 0x3f;
    __ srdi(Rdst, Rsrc1, isrc);
  
#line 9997 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint encodeP_not_null_base_nullNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void encodeP_narrow_oop_shift_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3890 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_or);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    if (Rsrc != Rdst) {
      __ mr(Rdst, Rsrc);
    }
  
#line 10023 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void decodeN_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3543 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicr);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    int isrc = (((Universe::narrow_oop_shift()))) & 0x3f;
    __ sldi(Rdst, Rsrc1, isrc);
  
#line 10044 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint decodeN_shiftNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void decodeN_addNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2990 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_add);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register Rsrc2 = reg_to_register_object((R30_enc));
    __ add(Rdst, Rsrc1, Rsrc2);
  
#line 10070 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint decodeN_addNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cond_add_baseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 5732 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmove);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* crx */);
    Label done;

    __ beq(Rcrx, done);
    __ add(Rdst, Rsrc, R30);
#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required.
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 10108 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cond_add_baseNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void cond_set_0_ptrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 5721 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound); // see above

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* crx */);

    __ isel_0(Rdst, Rcrx, Assembler::equal, Rsrc);
  
#line 10136 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cond_set_0_ptrNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void  decodeN_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  // Access to ins and operands for late expand.
  unsigned idx_dst   = idx0; 	// iRegPdst, 	dst
  unsigned idx_src   = idx1; 	// iRegNsrc, 	src
  unsigned idx_crx   = idx2; 	// flagsReg, 	crx
  Node    *n_region  = lookup(0);
  Node    *n_dst     = lookup(idx_dst);
  Node    *n_src     = lookup(idx_src);
  Node    *n_crx     = lookup(idx_crx);
  iRegPdstOper *op_dst = (iRegPdstOper *)opnd_array(0);
  iRegNsrcOper *op_src = (iRegNsrcOper *)opnd_array(1);
  flagsRegOper *op_crx = (flagsRegOper *)opnd_array(2);
  Compile *C = ra_->C;
  {
#line 4545 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    decodeN_shiftNode *n_shift    = new (C) decodeN_shiftNode();
    cmpN_reg_imm0Node *n_compare  = new (C) cmpN_reg_imm0Node();

    n_compare->add_req(n_region, n_src);
    n_compare->_opnds[0] = op_crx;
    n_compare->_opnds[1] = op_src;
    n_compare->_opnds[2] = new (C) immN_0Oper(TypeNarrowOop::NULL_PTR);

    n_shift->add_req(n_region, n_src);
    n_shift->_opnds[0] = op_dst;
    n_shift->_opnds[1] = op_src;
    n_shift->_bottom_type = _bottom_type;

    if (VM_Version::has_isel()) {
      // use isel instruction with Power 7

      decodeN_addNode *n_add_base = new (C) decodeN_addNode();
      n_add_base->add_req(n_region, n_shift);
      n_add_base->_opnds[0] = op_dst;
      n_add_base->_opnds[1] = op_dst;
      n_add_base->_bottom_type = _bottom_type;

      cond_set_0_ptrNode *n_cond_set = new (C) cond_set_0_ptrNode();
      n_cond_set->add_req(n_region, n_compare, n_add_base);
      n_cond_set->_opnds[0] = op_dst;
      n_cond_set->_opnds[1] = op_crx;
      n_cond_set->_opnds[2] = op_dst;
      n_cond_set->_bottom_type = _bottom_type;

      assert(ra_->is_oop(this) == true, "A decodeN node must produce an oop!");
      ra_->set_oop(n_cond_set, true);

      ra_->set_pair(n_shift->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
      ra_->set_pair(n_compare->_idx, ra_->get_reg_second(n_crx), ra_->get_reg_first(n_crx));
      ra_->set_pair(n_add_base->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
      ra_->set_pair(n_cond_set->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));

      nodes->push(n_compare);
      nodes->push(n_shift);
      nodes->push(n_add_base);
      nodes->push(n_cond_set);

    } else {
      // before Power 7
      cond_add_baseNode *n_add_base = new (C) cond_add_baseNode();

      n_add_base->add_req(n_region, n_compare, n_shift);
      n_add_base->_opnds[0] = op_dst;
      n_add_base->_opnds[1] = op_crx;
      n_add_base->_opnds[2] = op_dst;
      n_add_base->_bottom_type = _bottom_type;

      assert(ra_->is_oop(this) == true, "A decodeN node must produce an oop!");
      ra_->set_oop(n_add_base, true);

      ra_->set_pair(n_shift->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
      ra_->set_pair(n_compare->_idx, ra_->get_reg_second(n_crx), ra_->get_reg_first(n_crx));
      ra_->set_pair(n_add_base->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));

      nodes->push(n_compare);
      nodes->push(n_shift);
      nodes->push(n_add_base);
    }
  
#line 10229 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void decodeN_nullBaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3543 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicr);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    int isrc = (((Universe::narrow_oop_shift()))) & 0x3f;
    __ sldi(Rdst, Rsrc1, isrc);
  
#line 10250 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint decodeN_nullBaseNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void  decodeN_notNull_addBase_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  // Access to ins and operands for late expand.
  unsigned idx_dst   = idx0; 	// iRegPdst, 	dst
  unsigned idx_src   = idx1; 	// iRegNsrc, 	src
  Node    *n_region  = lookup(0);
  Node    *n_dst     = lookup(idx_dst);
  Node    *n_src     = lookup(idx_src);
  iRegPdstOper *op_dst = (iRegPdstOper *)opnd_array(0);
  iRegNsrcOper *op_src = (iRegNsrcOper *)opnd_array(1);
  Compile *C = ra_->C;
  {
#line 4523 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    decodeN_shiftNode *n1 = new (C) decodeN_shiftNode();
    n1->add_req(n_region, n_src);
    n1->_opnds[0] = op_dst;
    n1->_opnds[1] = op_src;
    n1->_bottom_type = _bottom_type;

    decodeN_addNode *n2 = new (C) decodeN_addNode();
    n2->add_req(n_region, n1);
    n2->_opnds[0] = op_dst;
    n2->_opnds[1] = op_dst;
    n2->_bottom_type = _bottom_type;
    ra_->set_pair(n1->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));
    ra_->set_pair(n2->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this));

    assert(ra_->is_oop(this) == true, "A decodeN node must produce an oop!");
    ra_->set_oop(n2, true);

    nodes->push(n1);
    nodes->push(n2);
  
#line 10295 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void decodeN_unscaledNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3890 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_or);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    if (Rsrc != Rdst) {
      __ mr(Rdst, Rsrc);
    }
  
#line 10316 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void decodeN2I_unscaledNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3890 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_or);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    if (Rsrc != Rdst) {
      __ mr(Rdst, Rsrc);
    }
  
#line 10337 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void unnecessary_membar_acquireNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint unnecessary_membar_acquireNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_acquire_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_acquire_lockNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_releaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 6017 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lwsync);
    MacroAssembler _masm(&cbuf);
    __ release();
  
#line 10380 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint membar_releaseNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void membar_release_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_release_lockNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 6023 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_sync);
    MacroAssembler _masm(&cbuf);
    __ fence();
  
#line 10415 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint membar_volatileNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void membar_CPUOrderNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_CPUOrderNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_storestoreNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 6017 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lwsync);
    MacroAssembler _masm(&cbuf);
    __ release();
  
#line 10450 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint membar_storestoreNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmovI_reg_iselNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 5708 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // This is a Power7 instruction for which no machine description
    // exists. Anyways, the scheduler should be off on Power7.
    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4)/* src */);
    ConditionRegister Rc = reg_to_ConditionRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* crx */);
    int cc        = opnd_array(1)->ccode();
    __ isel(Rdst, Rc, (Assembler::Condition)(cc & 3), /*invert*/((~cc) & 8), Rsrc);
  
#line 10481 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovI_reg_iselNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmovI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 5684 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmove);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4)/* src */);
    int cc        = opnd_array(1)->ccode();
    int flags_reg = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    Label done;
    assert((Assembler::bcondCRbiIs1 & ~Assembler::bcondCRbiIs0) == 8, "check encoding");
    // Branch if not (cmp crx).
    __ bc(cc_to_inverse_boint(cc), cc_to_biint(cc, flags_reg), done);
    __ mr(Rdst, Rsrc);
#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required.
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 10523 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovI_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void cmovI_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 5774 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmove);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* dst */);
    int Csrc      = opnd_array(4)->constant();
    int cc        = opnd_array(1)->ccode();
    int flags_reg = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    Label done;
    assert((Assembler::bcondCRbiIs1 & ~Assembler::bcondCRbiIs0) == 8, "check encoding");
    // Branch if not (cmp crx).
    __ bc(cc_to_inverse_boint(cc), cc_to_biint(cc, flags_reg), done);
    __ li(Rdst, Csrc);
#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required.
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 10565 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovI_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void cmovL_reg_iselNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 5708 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // This is a Power7 instruction for which no machine description
    // exists. Anyways, the scheduler should be off on Power7.
    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4)/* src */);
    ConditionRegister Rc = reg_to_ConditionRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* crx */);
    int cc        = opnd_array(1)->ccode();
    __ isel(Rdst, Rc, (Assembler::Condition)(cc & 3), /*invert*/((~cc) & 8), Rsrc);
  
#line 10596 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovL_reg_iselNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmovL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 5684 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmove);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4)/* src */);
    int cc        = opnd_array(1)->ccode();
    int flags_reg = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    Label done;
    assert((Assembler::bcondCRbiIs1 & ~Assembler::bcondCRbiIs0) == 8, "check encoding");
    // Branch if not (cmp crx).
    __ bc(cc_to_inverse_boint(cc), cc_to_biint(cc, flags_reg), done);
    __ mr(Rdst, Rsrc);
#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required.
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 10638 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovL_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void cmovL_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 5774 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmove);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* dst */);
    int Csrc      = opnd_array(4)->constantL();
    int cc        = opnd_array(1)->ccode();
    int flags_reg = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    Label done;
    assert((Assembler::bcondCRbiIs1 & ~Assembler::bcondCRbiIs0) == 8, "check encoding");
    // Branch if not (cmp crx).
    __ bc(cc_to_inverse_boint(cc), cc_to_biint(cc, flags_reg), done);
    __ li(Rdst, Csrc);
#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required.
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 10680 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovL_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void cmovN_reg_iselNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 5708 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // This is a Power7 instruction for which no machine description
    // exists. Anyways, the scheduler should be off on Power7.
    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4)/* src */);
    ConditionRegister Rc = reg_to_ConditionRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* crx */);
    int cc        = opnd_array(1)->ccode();
    __ isel(Rdst, Rc, (Assembler::Condition)(cc & 3), /*invert*/((~cc) & 8), Rsrc);
  
#line 10711 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovN_reg_iselNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmovN_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 5684 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmove);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4)/* src */);
    int cc        = opnd_array(1)->ccode();
    int flags_reg = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    Label done;
    assert((Assembler::bcondCRbiIs1 & ~Assembler::bcondCRbiIs0) == 8, "check encoding");
    // Branch if not (cmp crx).
    __ bc(cc_to_inverse_boint(cc), cc_to_biint(cc, flags_reg), done);
    __ mr(Rdst, Rsrc);
#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required.
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 10753 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovN_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void cmovN_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 5774 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmove);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* dst */);
    int Csrc      = opnd_array(4)->constant();
    int cc        = opnd_array(1)->ccode();
    int flags_reg = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    Label done;
    assert((Assembler::bcondCRbiIs1 & ~Assembler::bcondCRbiIs0) == 8, "check encoding");
    // Branch if not (cmp crx).
    __ bc(cc_to_inverse_boint(cc), cc_to_biint(cc, flags_reg), done);
    __ li(Rdst, Csrc);
#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required.
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 10795 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovN_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void cmovP_reg_iselNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 5708 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // This is a Power7 instruction for which no machine description
    // exists. Anyways, the scheduler should be off on Power7.
    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4)/* src */);
    ConditionRegister Rc = reg_to_ConditionRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* crx */);
    int cc        = opnd_array(1)->ccode();
    __ isel(Rdst, Rc, (Assembler::Condition)(cc & 3), /*invert*/((~cc) & 8), Rsrc);
  
#line 10826 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovP_reg_iselNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmovP_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 5684 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmove);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4)/* src */);
    int cc        = opnd_array(1)->ccode();
    int flags_reg = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    Label done;
    assert((Assembler::bcondCRbiIs1 & ~Assembler::bcondCRbiIs0) == 8, "check encoding");
    // Branch if not (cmp crx).
    __ bc(cc_to_inverse_boint(cc), cc_to_biint(cc, flags_reg), done);
    __ mr(Rdst, Rsrc);
#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required.
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 10868 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovP_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void cmovP_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 5774 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmove);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* dst */);
    int Csrc      = opnd_array(4)->constant();
    int cc        = opnd_array(1)->ccode();
    int flags_reg = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    Label done;
    assert((Assembler::bcondCRbiIs1 & ~Assembler::bcondCRbiIs0) == 8, "check encoding");
    // Branch if not (cmp crx).
    __ bc(cc_to_inverse_boint(cc), cc_to_biint(cc, flags_reg), done);
    __ li(Rdst, Csrc);
#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required.
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 10910 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovP_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void cmovF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 5797 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmovef);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(3)->reg(ra_,this,idx3)/* dst */);
    FloatRegister Rsrc = reg_to_FloatRegister_object(opnd_array(4)->reg(ra_,this,idx4)/* src */);
    int cc             = opnd_array(1)->ccode();
    int flags_reg      = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    Label done;
    assert((Assembler::bcondCRbiIs1 & ~Assembler::bcondCRbiIs0) == 8, "check encoding");
    // Branch if not (cmp crx).
    __ bc(cc_to_inverse_boint(cc), cc_to_biint(cc, flags_reg), done);
    __ fmr(Rdst, Rsrc);
#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 10952 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovF_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void cmovD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 5797 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmovef);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(3)->reg(ra_,this,idx3)/* dst */);
    FloatRegister Rsrc = reg_to_FloatRegister_object(opnd_array(4)->reg(ra_,this,idx4)/* src */);
    int cc             = opnd_array(1)->ccode();
    int flags_reg      = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    Label done;
    assert((Assembler::bcondCRbiIs1 & ~Assembler::bcondCRbiIs0) == 8, "check encoding");
    // Branch if not (cmp crx).
    __ bc(cc_to_inverse_boint(cc), cc_to_biint(cc, flags_reg), done);
    __ fmr(Rdst, Rsrc);
#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 10994 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovD_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void storeLConditional_regP_regL_regLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldVal
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newVal
  {

#line 5852 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    ConditionRegister flag = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    Register Rdummy = R0;
    Register Rres   = R0;
    Register Rold   = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* oldVal */);
    Register Rnew   = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* newVal */);
    Register Rptr   = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */);
    __ cmpxchgd(flag, Rdummy, Rold, Rnew, Rptr, MacroAssembler::MemBarNone,
                MacroAssembler::cmpxchgx_hint_atomic_update(), noreg, NULL, true);
  
#line 11026 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void storePConditional_regP_regP_regPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldVal
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newVal
  {

#line 5852 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    ConditionRegister flag = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    Register Rdummy = R0;
    Register Rres   = R0;
    Register Rold   = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* oldVal */);
    Register Rnew   = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* newVal */);
    Register Rptr   = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */);
    __ cmpxchgd(flag, Rdummy, Rold, Rnew, Rptr, MacroAssembler::MemBarNone,
                MacroAssembler::cmpxchgx_hint_atomic_update(), noreg, NULL, true);
  
#line 11053 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void loadPLockedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3280 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ ld(Rdst, Idisp, Rbase);
    __ twi_0(Rdst);
    __ isync();
  
#line 11080 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint loadPLockedNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void compareAndSwapI_regP_regI_regINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 5820 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rdummy = R0;
    Register Rres   = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* res */);
    Register Rold   = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
    Register Rnew   = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
    Register Rptr   = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */);

    // CmpxchgX sets CCR0 to cmpX(Rval, Rcomp) and Rres to 'true'/'false'.
    __ cmpxchgw(CCR0, Rdummy, Rold, Rnew, Rptr, MacroAssembler::MemBarNone,
                MacroAssembler::cmpxchgx_hint_atomic_update(), Rres, true);
  
#line 11113 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void compareAndSwapN_regP_regN_regNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 5835 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rdummy = R0;
    Register Rres   = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* res */);
    Register Rold   = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
    Register Rnew   = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
    Register Rptr   = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */);

    // CmpxchgX sets CCR0 to cmpX(Rval, Rcomp) and Rres to 'true'/'false'.
    __ cmpxchgw(CCR0, Rdummy, Rold, Rnew, Rptr, MacroAssembler::MemBarNone,
                MacroAssembler::cmpxchgx_hint_atomic_update(), Rres, true);
  
#line 11141 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void compareAndSwapL_regP_regL_regLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 5866 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rdummy = R0;
    Register Rres   = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* res */);
    Register Rold   = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
    Register Rnew   = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
    Register Rptr   = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */);

    // CmpxchgX sets CCR0 to cmpX(Rval, Rcomp) and Rres to 'true'/'false'.
    __ cmpxchgd(CCR0, Rdummy, Rold, Rnew, Rptr, MacroAssembler::MemBarNone,
                MacroAssembler::cmpxchgx_hint_atomic_update(), Rres, NULL, true);
  
#line 11169 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void compareAndSwapP_regP_regP_regPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 5866 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rdummy = R0;
    Register Rres   = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* res */);
    Register Rold   = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
    Register Rnew   = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
    Register Rptr   = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */);

    // CmpxchgX sets CCR0 to cmpX(Rval, Rcomp) and Rres to 'true'/'false'.
    __ cmpxchgd(CCR0, Rdummy, Rold, Rnew, Rptr, MacroAssembler::MemBarNone,
                MacroAssembler::cmpxchgx_hint_atomic_update(), Rres, NULL, true);
  
#line 11197 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void getAndAddINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 5881 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rtmp   = R0;
    Register Rres   = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* res */);
    Register Rsrc   = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rptr   = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */);
    bool RegCollision = (Rres == Rsrc) || (Rres == Rptr);
    Register Rold   = RegCollision ? Rtmp : Rres;

    Label Lretry;
    __ bind(Lretry);
    __ lwarx(Rold, Rptr, /*hint*/ false);
    __ add(Rtmp, Rsrc, Rold);
    __ stwcx_(Rtmp, Rptr);
    __ bne(CCR0, Lretry);
    if (RegCollision) __ subf(Rres, Rsrc, Rtmp);
  
#line 11229 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void getAndAddLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 5901 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rtmp   = R0;
    Register Rres   = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* res */);
    Register Rsrc   = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rptr   = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */);
    bool RegCollision = (Rres == Rsrc) || (Rres == Rptr);
    Register Rold   = RegCollision ? Rtmp : Rres;

    Label Lretry;
    __ bind(Lretry);
    __ ldarx(Rold, Rptr, /*hint*/ false);
    __ add(Rtmp, Rsrc, Rold);
    __ stdcx_(Rtmp, Rptr);
    __ bne(CCR0, Lretry);
    if (RegCollision) __ subf(Rres, Rsrc, Rtmp);
  
#line 11261 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void getAndSetINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 5921 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rtmp   = R0;
    Register Rres   = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* res */);
    Register Rsrc   = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rptr   = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */);
    bool RegCollision = (Rres == Rsrc) || (Rres == Rptr);
    Register Rold   = RegCollision ? Rtmp : Rres;

    Label Lretry;
    __ bind(Lretry);
    __ lwarx(Rold, Rptr, /*hint*/ false);
    __ stwcx_(Rsrc, Rptr);
    __ bne(CCR0, Lretry);
    if (RegCollision) __ mr(Rres, Rtmp);
  
#line 11292 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void getAndSetLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 5940 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rtmp   = R0;
    Register Rres   = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* res */);
    Register Rsrc   = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rptr   = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */);
    bool RegCollision = (Rres == Rsrc) || (Rres == Rptr);
    Register Rold   = RegCollision ? Rtmp : Rres;

    Label Lretry;
    __ bind(Lretry);
    __ ldarx(Rold, Rptr, /*hint*/ false);
    __ stdcx_(Rsrc, Rptr);
    __ bne(CCR0, Lretry);
    if (RegCollision) __ mr(Rres, Rtmp);
  
#line 11323 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void getAndSetPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 5940 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rtmp   = R0;
    Register Rres   = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* res */);
    Register Rsrc   = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rptr   = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */);
    bool RegCollision = (Rres == Rsrc) || (Rres == Rptr);
    Register Rold   = RegCollision ? Rtmp : Rres;

    Label Lretry;
    __ bind(Lretry);
    __ ldarx(Rold, Rptr, /*hint*/ false);
    __ stdcx_(Rsrc, Rptr);
    __ bne(CCR0, Lretry);
    if (RegCollision) __ mr(Rres, Rtmp);
  
#line 11354 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void getAndSetNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 5921 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rtmp   = R0;
    Register Rres   = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* res */);
    Register Rsrc   = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register Rptr   = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */);
    bool RegCollision = (Rres == Rsrc) || (Rres == Rptr);
    Register Rold   = RegCollision ? Rtmp : Rres;

    Label Lretry;
    __ bind(Lretry);
    __ lwarx(Rold, Rptr, /*hint*/ false);
    __ stwcx_(Rsrc, Rptr);
    __ bne(CCR0, Lretry);
    if (RegCollision) __ mr(Rres, Rtmp);
  
#line 11385 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void addI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2990 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_add);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ add(Rdst, Rsrc1, Rsrc2);
  
#line 11407 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint addI_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addI_reg_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2990 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_add);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ add(Rdst, Rsrc1, Rsrc2);
  
#line 11434 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint addI_reg_reg_2Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addI_reg_imm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2769 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = opnd_array(2)->constant();
    __ addi(Rdst, Rsrc1, Isrc2);
  
#line 11461 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint addI_reg_imm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addI_reg_immhi16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2779 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addis);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = (opnd_array(2)->constant())>>16;
    __ addis(Rdst, Rsrc1, Isrc2);
  
#line 11488 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint addI_reg_immhi16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2990 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_add);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ add(Rdst, Rsrc1, Rsrc2);
  
#line 11515 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint addL_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addL_reg_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2990 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_add);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ add(Rdst, Rsrc1, Rsrc2);
  
#line 11542 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint addL_reg_reg_2Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addI_regL_regLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2990 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_add);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ add(Rdst, Rsrc1, Rsrc2);
  
#line 11569 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint addI_regL_regLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addL_reg_imm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {

#line 2769 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = opnd_array(2)->constantL();
    __ addi(Rdst, Rsrc1, Isrc2);
  
#line 11596 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint addL_reg_imm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addL_reg_immhi16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2779 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addis);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = (opnd_array(2)->constantL())>>16;
    __ addis(Rdst, Rsrc1, Isrc2);
  
#line 11623 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint addL_reg_immhi16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addP_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2990 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_add);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ add(Rdst, Rsrc1, Rsrc2);
  
#line 11650 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint addP_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addP_reg_imm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2769 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = opnd_array(2)->constantL();
    __ addi(Rdst, Rsrc1, Isrc2);
  
#line 11677 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint addP_reg_imm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addP_reg_immhi16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2779 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addis);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = (opnd_array(2)->constantL())>>16;
    __ addis(Rdst, Rsrc1, Isrc2);
  
#line 11704 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint addP_reg_immhi16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void subI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3000 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_subf);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    __ subf(Rdst, Rsrc1, Rsrc2);
  
#line 11731 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint subI_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void subI_reg_imm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2789 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = (opnd_array(2)->constant()) * (-1) ;
    __ addi(Rdst, Rsrc1, Isrc2);
  
#line 11758 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint subI_reg_imm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void subI_imm16_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2799 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_subfic);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    int Isrc2 = opnd_array(1)->constant();
    __ subfic(Rdst, Rsrc1, Isrc2);
  
#line 11785 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint subI_imm16_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void signmask32I_regINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3615 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_srawi);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    int isrc = ((0x1f)) & 0x1f;
    __ srawi(Rdst, Rsrc1, isrc);
  
#line 11811 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint signmask32I_regINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void negI_regINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2819 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_neg);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ neg(Rdst, Rsrc1);
  
#line 11837 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint negI_regINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void subL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3000 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_subf);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    __ subf(Rdst, Rsrc1, Rsrc2);
  
#line 11864 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint subL_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void subI_regL_regLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3000 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_subf);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    __ subf(Rdst, Rsrc1, Rsrc2);
  
#line 11891 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint subI_regL_regLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void subL_reg_imm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2789 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = (opnd_array(2)->constantL()) * (-1) ;
    __ addi(Rdst, Rsrc1, Isrc2);
  
#line 11918 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint subL_reg_imm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void signmask64I_regLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9529 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_sradi);
    __ sradi(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, 0x3f);
  
#line 11940 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint signmask64I_regLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void signmask64L_regLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3635 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_sradi);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    int isrc = ((0x3f)) & 0x3f;
    __ sradi(Rdst, Rsrc1, isrc);
  
#line 11966 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint signmask64L_regLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void negL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2819 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_neg);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ neg(Rdst, Rsrc1);
  
#line 11992 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint negL_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void negI_con0_regLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2819 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_neg);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ neg(Rdst, Rsrc1);
  
#line 12018 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint negI_con0_regLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void mulI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3010 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_mullw);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ mullw(Rdst, Rsrc1, Rsrc2);
  
#line 12045 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint mulI_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void mulI_reg_imm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2809 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_mulli);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = opnd_array(2)->constant();
    __ mulli(Rdst, Rsrc1, Isrc2);
  
#line 12072 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint mulI_reg_imm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void mulL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3020 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_mulld);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ mulld(Rdst, Rsrc1, Rsrc2);
  
#line 12099 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint mulL_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void mulHighL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3030 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_mulhd);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ mulhd(Rdst, Rsrc1, Rsrc2);
  
#line 12126 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint mulHighL_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void mulL_reg_imm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2809 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_mulli);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = opnd_array(2)->constantL();
    __ mulli(Rdst, Rsrc1, Isrc2);
  
#line 12153 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint mulL_reg_imm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void divI_reg_immIvalueMinus1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2819 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_neg);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    __ neg(Rdst, Rsrc1);
  
#line 12179 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint divI_reg_immIvalueMinus1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void divI_reg_regnotMinus1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 5191 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_divw);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ divw(Rdst, Rsrc1, Rsrc2);
  
#line 12206 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint divI_reg_regnotMinus1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmovI_bne_negI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 5168 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmove);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* src1 */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* crx */);

    Label done;
    __ bne(Rcrx, done);
    __ neg(Rdst, Rsrc1);

#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required.
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();

    __ bind(done);
  
#line 12247 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovI_bne_negI_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void divL_reg_immLvalueMinus1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2819 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_neg);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    __ neg(Rdst, Rsrc1);
  
#line 12273 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint divL_reg_immLvalueMinus1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void divL_reg_regnotMinus1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 5201 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_divd);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ divd(Rdst, Rsrc1, Rsrc2);
  
#line 12300 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint divL_reg_regnotMinus1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmovL_bne_negL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 5168 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmove);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* src1 */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* crx */);

    Label done;
    __ bne(Rcrx, done);
    __ neg(Rdst, Rsrc1);

#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required.
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();

    __ bind(done);
  
#line 12341 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovL_bne_negL_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void maskI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 3585 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicl);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    int iclearleftbits = opnd_array(2)->constant();
    __ clrldi(Rdst, Rsrc, iclearleftbits);
  
#line 12368 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint maskI_reg_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void lShiftI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3533 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_slw);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ slw(Rdst, Rsrc1, Rsrc2);
  
#line 12395 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint lShiftI_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void lShiftI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3523 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rlwinm);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int isrc = (opnd_array(2)->constant()) & 0x1f;
    __ slwi(Rdst, Rsrc1, isrc);
  
#line 12422 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint lShiftI_reg_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void lShiftI_andI_immInegpow2_imm5Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {

#line 3824 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rlwinm); // FIXME: assert that rlwinm is equal to addi

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    long src2      = opnd_array(2)->constant();
    long src3      = opnd_array(3)->constant();
    long maskbits  = src3 + log2_long((jlong) (julong) (juint) -src2);
    if (maskbits >= 32) {
      __ li(Rdst, 0); // addi
    } else {
      __ rlwinm(Rdst, Rsrc1, src3 & 0x1f, 0, (31-maskbits) & 0x1f);
    }
  
#line 12456 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint lShiftI_andI_immInegpow2_imm5Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void lShiftI_andI_immInegpow2_rShiftI_imm5Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 3840 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rlwinm); // FIXME: assert that rlwinm is equal to addi

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    long src2      = opnd_array(3)->constant();
    long src3      = opnd_array(2)->constant();
    long maskbits  = src3 + log2_long((jlong) (julong) (juint) -src2);
    if (maskbits >= 32) {
      __ li(Rdst, 0); // addi
    } else {
      __ rlwinm(Rdst, Rsrc1, 0, 0, (31-maskbits) & 0x1f);
    }
  
#line 12490 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint lShiftI_andI_immInegpow2_rShiftI_imm5Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void lShiftL_regL_regINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3553 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_sld);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ sld(Rdst, Rsrc1, Rsrc2);
  
#line 12517 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint lShiftL_regL_regINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void lshiftL_regL_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3543 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicr);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int isrc = (opnd_array(2)->constant()) & 0x3f;
    __ sldi(Rdst, Rsrc1, isrc);
  
#line 12544 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint lshiftL_regL_immINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void lShiftL_regI_immGE32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3543 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicr);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int isrc = (opnd_array(2)->constant()) & 0x3f;
    __ sldi(Rdst, Rsrc1, isrc);
  
#line 12571 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint lShiftL_regI_immGE32Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void scaledPositiveI2L_lShiftL_convI2L_reg_imm6Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3563 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldic);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int iclearleftbits = (0x20);
    int ishiftleftbits = opnd_array(2)->constant();
    __ clrlsldi(Rdst, Rsrc, iclearleftbits, ishiftleftbits);
  
#line 12599 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint scaledPositiveI2L_lShiftL_convI2L_reg_imm6Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void arShiftI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3625 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_sraw);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ sraw(Rdst, Rsrc1, Rsrc2);
  
#line 12626 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint arShiftI_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void arShiftI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3615 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_srawi);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int isrc = (opnd_array(2)->constant()) & 0x1f;
    __ srawi(Rdst, Rsrc1, isrc);
  
#line 12653 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint arShiftI_reg_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void arShiftL_regL_regINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3645 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_srad);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ srad(Rdst, Rsrc1, Rsrc2);
  
#line 12680 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint arShiftL_regL_regINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void arShiftL_regL_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3635 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_sradi);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int isrc = (opnd_array(2)->constant()) & 0x3f;
    __ sradi(Rdst, Rsrc1, isrc);
  
#line 12707 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint arShiftL_regL_immINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void convL2I_arShiftL_regL_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3635 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_sradi);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int isrc = (opnd_array(2)->constant()) & 0x3f;
    __ sradi(Rdst, Rsrc1, isrc);
  
#line 12734 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convL2I_arShiftL_regL_immINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void urShiftI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3605 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_srw);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ srw(Rdst, Rsrc1, Rsrc2);
  
#line 12761 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint urShiftI_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void urShiftI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3595 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rlwinm);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int isrc = (opnd_array(2)->constant()) & 0x1f;
    __ srwi(Rdst, Rsrc1, isrc);
  
#line 12788 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint urShiftI_reg_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void urShiftL_regL_regINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3665 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_srd);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ srd(Rdst, Rsrc1, Rsrc2);
  
#line 12815 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint urShiftL_regL_regINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void urShiftL_regL_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3655 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicl);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int isrc = (opnd_array(2)->constant()) & 0x3f;
    __ srdi(Rdst, Rsrc1, isrc);
  
#line 12842 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint urShiftL_regL_immINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void convL2I_urShiftL_regL_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3655 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicl);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int isrc = (opnd_array(2)->constant()) & 0x3f;
    __ srdi(Rdst, Rsrc1, isrc);
  
#line 12869 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convL2I_urShiftL_regL_immINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void shrP_convP2X_reg_imm6Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3655 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicl);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int isrc = (opnd_array(2)->constant()) & 0x3f;
    __ srdi(Rdst, Rsrc1, isrc);
  
#line 12896 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint shrP_convP2X_reg_imm6Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void sxtI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2855 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_extsw);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ extsw(Rdst, Rsrc1);
  
#line 12921 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint sxtI_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void rotlI_reg_immi8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  {

#line 3676 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rlwinm);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    int ishift = opnd_array(2)->constant();
    __ rotlwi(Rdst, Rsrc, ishift);
  
#line 12949 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint rotlI_reg_immi8Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void rotlI_reg_immi8_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  {

#line 3676 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rlwinm);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    int ishift = opnd_array(3)->constant();
    __ rotlwi(Rdst, Rsrc, ishift);
  
#line 12977 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint rotlI_reg_immi8_0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void rotrI_reg_immi8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  {

#line 3687 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rlwinm);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    int ishift = opnd_array(2)->constant();
    __ rotrwi(Rdst, Rsrc, ishift);
  
#line 13005 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint rotrI_reg_immi8Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void rotrI_reg_immi8_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  {

#line 3687 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rlwinm);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    int ishift = opnd_array(3)->constant();
    __ rotrwi(Rdst, Rsrc, ishift);
  
#line 13033 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint rotrI_reg_immi8_0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2940 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fadds);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    FloatRegister Rsrc2 = reg_to_FloatRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ fadds(Rdst, Rsrc1, Rsrc2);
  
#line 13060 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint addF_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2950 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fadd);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    FloatRegister Rsrc2 = reg_to_FloatRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ fadd(Rdst, Rsrc1, Rsrc2);
  
#line 13087 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint addD_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void subF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2960 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fsubs);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    FloatRegister Rsrc2 = reg_to_FloatRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ fsubs(Rdst, Rsrc1, Rsrc2);
  
#line 13114 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint subF_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void subD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2970 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fsub);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    FloatRegister Rsrc2 = reg_to_FloatRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ fsub(Rdst, Rsrc1, Rsrc2);
  
#line 13141 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint subD_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void mulF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3050 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fmuls);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    FloatRegister Rsrc2 = reg_to_FloatRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ fmuls(Rdst, Rsrc1, Rsrc2);
  
#line 13168 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint mulF_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void mulD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3060 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fmul);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    FloatRegister Rsrc2 = reg_to_FloatRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ fmul(Rdst, Rsrc1, Rsrc2);
  
#line 13195 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint mulD_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void divF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2980 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fdivs);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    FloatRegister Rsrc2 = reg_to_FloatRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ fdivs(Rdst, Rsrc1, Rsrc2);
  
#line 13222 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint divF_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void divD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3040 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fdiv);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    FloatRegister Rsrc2 = reg_to_FloatRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ fdiv(Rdst, Rsrc1, Rsrc2);
  
#line 13249 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint divD_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void absF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3070 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fabs);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fabs(Rdst, Rsrc1);
  
#line 13274 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint absF_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void absD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3070 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fabs);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fabs(Rdst, Rsrc1);
  
#line 13299 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint absD_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void negF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3115 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fneg);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fneg(Rdst, Rsrc1);
  
#line 13324 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint negF_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void negD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3115 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fneg);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fneg(Rdst, Rsrc1);
  
#line 13349 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint negD_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void negF_absF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3079 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fnabs);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fnabs(Rdst, Rsrc1);
  
#line 13374 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint negF_absF_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void negD_absD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3079 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fnabs);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fnabs(Rdst, Rsrc1);
  
#line 13399 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint negD_absD_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void sqrtD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3088 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fsqrt);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fsqrt(Rdst, Rsrc1);
  
#line 13424 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint sqrtD_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void sqrtF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3106 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fsqrts);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fsqrts(Rdst, Rsrc1);
  
#line 13449 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint sqrtF_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void roundDouble_nopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint roundDouble_nopNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void roundFloat_nopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint roundFloat_nopNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void andI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2910 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_and);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ andr(Rdst, Rsrc1, Rsrc2);
  
#line 13502 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint andI_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void andI_reg_uimm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2739 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_andi_);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = opnd_array(2)->constant();
    // FIXME: avoid andi_ ?
    __ andi_(Rdst, Rsrc1, Isrc2);
  
#line 13530 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint andI_reg_uimm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void andI_reg_immInegpow2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3697 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicr);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int src2       = opnd_array(2)->constant();
    __ clrrdi(Rdst, Rsrc1, log2_long((jlong) (julong) (juint) -src2));
  
#line 13557 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint andI_reg_immInegpow2Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void andI_reg_immIpow2minus1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3717 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicl);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    long src2      = opnd_array(2)->constant();
    __ clrldi(Rdst, Rsrc1, 64-log2_long((((jlong) src2)+1)));
  
#line 13584 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint andI_reg_immIpow2minus1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void andI_reg_immIpowerOf2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3727 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rlwinm);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    long src2      = opnd_array(2)->constant();
    __ rlwinm(Rdst, Rsrc1, 0, (31-log2_long((jlong) src2)) & 0x1f, (31-log2_long((jlong) src2)) & 0x1f);
  
#line 13611 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint andI_reg_immIpowerOf2Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void andL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2910 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_and);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ andr(Rdst, Rsrc1, Rsrc2);
  
#line 13638 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint andL_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void andL_reg_uimm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2739 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_andi_);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = opnd_array(2)->constantL();
    // FIXME: avoid andi_ ?
    __ andi_(Rdst, Rsrc1, Isrc2);
  
#line 13666 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint andL_reg_uimm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void andL_reg_immLnegpow2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3707 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicr);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    long src2      = opnd_array(2)->constantL();
    __ clrrdi(Rdst, Rsrc1, log2_long((jlong)-src2));
  
#line 13693 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint andL_reg_immLnegpow2Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void andL_reg_immLpow2minus1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3717 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicl);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    long src2      = opnd_array(2)->constantL();
    __ clrldi(Rdst, Rsrc1, 64-log2_long((((jlong) src2)+1)));
  
#line 13720 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint andL_reg_immLpow2minus1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void convL2I_andL_reg_immLpow2minus1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3717 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicl);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    long src2      = opnd_array(2)->constantL();
    __ clrldi(Rdst, Rsrc1, 64-log2_long((((jlong) src2)+1)));
  
#line 13747 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convL2I_andL_reg_immLpow2minus1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void orI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2920 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_or);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ or_unchecked(Rdst, Rsrc1, Rsrc2);
  
#line 13774 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint orI_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void orI_reg_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2920 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_or);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ or_unchecked(Rdst, Rsrc1, Rsrc2);
  
#line 13801 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint orI_reg_reg_2Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void orI_reg_uimm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2759 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ori);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = (opnd_array(2)->constant()) & 0xFFFF;
    __ ori(Rdst, Rsrc1, Isrc2);
  
#line 13828 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint orI_reg_uimm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void orL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2920 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_or);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ or_unchecked(Rdst, Rsrc1, Rsrc2);
  
#line 13855 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint orL_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void orI_regL_regLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2920 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_or);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ or_unchecked(Rdst, Rsrc1, Rsrc2);
  
#line 13882 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint orI_regL_regLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void orL_reg_uimm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {

#line 2759 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ori);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = (opnd_array(2)->constantL()) & 0xFFFF;
    __ ori(Rdst, Rsrc1, Isrc2);
  
#line 13909 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint orL_reg_uimm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void xorI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2930 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_xor);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ xorr(Rdst, Rsrc1, Rsrc2);
  
#line 13936 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint xorI_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void xorI_reg_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2930 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_xor);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ xorr(Rdst, Rsrc1, Rsrc2);
  
#line 13963 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint xorI_reg_reg_2Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void xorI_reg_uimm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2750 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_xori);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);   int Isrc2 = opnd_array(2)->constant();
    __ xori(Rdst, Rsrc1, Isrc2);
  
#line 13989 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint xorI_reg_uimm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void xorL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2930 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_xor);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ xorr(Rdst, Rsrc1, Rsrc2);
  
#line 14016 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint xorL_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void xorI_regL_regLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2930 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_xor);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ xorr(Rdst, Rsrc1, Rsrc2);
  
#line 14043 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint xorI_regL_regLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void xorL_reg_uimm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {

#line 2750 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_xori);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);   int Isrc2 = opnd_array(2)->constantL();
    __ xori(Rdst, Rsrc1, Isrc2);
  
#line 14069 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint xorL_reg_uimm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void notI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2891 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_nor);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    __ nor(Rdst, Rsrc1, Rsrc1);
  
#line 14095 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint notI_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void notL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2891 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_nor);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    __ nor(Rdst, Rsrc1, Rsrc1);
  
#line 14121 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint notL_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void andcI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {

#line 2900 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_andc);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* src3 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    __ andc(Rdst, Rsrc1, Rsrc2);
  
#line 14149 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint andcI_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void andcI_reg_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 2900 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_andc);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src3 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
    __ andc(Rdst, Rsrc1, Rsrc2);
  
#line 14177 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint andcI_reg_reg_0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void andcL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2900 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_andc);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ andc(Rdst, Rsrc1, Rsrc2);
  
#line 14204 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint andcL_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void stkI_to_regINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3124 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lwz);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lwz(Rdst, Idisp, Rbase);
  
#line 14233 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint stkI_to_regINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void regI_to_stkINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3380 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_stw);
    MacroAssembler _masm(&cbuf);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(0)->base(ra_,this,idx0));
    int Idisp = opnd_array(0)->disp(ra_,this,0);
    if (opnd_array(0)->base(ra_,this,idx0) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ stw(Rsrc, Idisp, Rbase);
  
#line 14261 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint regI_to_stkINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void stkL_to_regLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3265 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ld);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ ld(Rdst, Idisp, Rbase);
  
#line 14291 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint stkL_to_regLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void regL_to_stkLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3429 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_std);
    MacroAssembler _masm(&cbuf);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(0)->base(ra_,this,idx0));
    int Idisp = opnd_array(0)->disp(ra_,this,0);
    if (opnd_array(0)->base(ra_,this,idx0) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ std(Rsrc, Idisp, Rbase);
  
#line 14321 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint regL_to_stkLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void moveF2I_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3124 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lwz);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lwz(Rdst, Idisp, Rbase);
  
#line 14350 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint moveF2I_stack_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void moveF2I_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3356 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_stfs);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rsrc = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(0)->base(ra_,this,idx0));
    int Idisp = opnd_array(0)->disp(ra_,this,0);
    if (opnd_array(0)->base(ra_,this,idx0) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ stfs(Rsrc, Idisp, Rbase);
  
#line 14378 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint moveF2I_reg_stackNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void moveI2F_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3296 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lfs);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lfs(Rdst, Idisp, Rbase);
  
#line 14406 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint moveI2F_stack_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void moveI2F_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3380 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_stw);
    MacroAssembler _masm(&cbuf);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(0)->base(ra_,this,idx0));
    int Idisp = opnd_array(0)->disp(ra_,this,0);
    if (opnd_array(0)->base(ra_,this,idx0) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ stw(Rsrc, Idisp, Rbase);
  
#line 14434 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint moveI2F_reg_stackNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void moveF2L_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3368 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_stfd);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rsrc = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(0)->base(ra_,this,idx0));
    int Idisp = opnd_array(0)->disp(ra_,this,0);
    if (opnd_array(0)->base(ra_,this,idx0) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ stfd(Rsrc, Idisp, Rbase);
  
#line 14462 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint moveF2L_reg_stackNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void moveD2L_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3265 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ld);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ ld(Rdst, Idisp, Rbase);
  
#line 14492 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint moveD2L_stack_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void moveD2L_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3368 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_stfd);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rsrc = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(0)->base(ra_,this,idx0));
    int Idisp = opnd_array(0)->disp(ra_,this,0);
    if (opnd_array(0)->base(ra_,this,idx0) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ stfd(Rsrc, Idisp, Rbase);
  
#line 14520 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint moveD2L_reg_stackNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void moveL2D_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3326 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_lfd);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rbase = reg_to_register_object(opnd_array(1)->base(ra_,this,idx1));
    int Idisp = opnd_array(1)->disp(ra_,this,idx1);
    if (opnd_array(1)->base(ra_,this,idx1) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    __ lfd(Rdst, Idisp, Rbase);
  
#line 14548 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint moveL2D_stack_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void moveL2D_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3429 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_std);
    MacroAssembler _masm(&cbuf);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register Rbase = reg_to_register_object(opnd_array(0)->base(ra_,this,idx0));
    int Idisp = opnd_array(0)->disp(ra_,this,0);
    if (opnd_array(0)->base(ra_,this,idx0) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    // Operand 'ds' requires 4-alignment.
    assert((Idisp & 0x3) == 0, "unaligned offset");
    __ std(Rsrc, Idisp, Rbase);
  
#line 14578 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint moveL2D_reg_stackNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void moveRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3890 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_or);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    if (Rsrc != Rdst) {
      __ mr(Rdst, Rsrc);
    }
  
#line 14604 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void castX2PNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3890 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_or);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    if (Rsrc != Rdst) {
      __ mr(Rdst, Rsrc);
    }
  
#line 14625 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void castP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3890 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_or);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    if (Rsrc != Rdst) {
      __ mr(Rdst, Rsrc);
    }
  
#line 14646 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void castPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castPPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castIINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castIINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void checkCastPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint checkCastPPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void convI2Bool_reg__cmoveNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  {

#line 3792 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* crx */);

    Label done;
    __ cmpwi(Rcrx, Rsrc, 0);
    __ li(Rdst, (0x0));
    __ beq(Rcrx, done);
    __ li(Rdst, (0x1));
    __ bind(done);
  
#line 14713 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convI2Bool_reg__cmoveNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 16, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 16);
}

void xorI_convI2Bool_reg_immIvalue1__cmoveNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// crx
  {

#line 3792 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(3)->reg(ra_,this,idx3)/* crx */);

    Label done;
    __ cmpwi(Rcrx, Rsrc, 0);
    __ li(Rdst, (0x1));
    __ beq(Rcrx, done);
    __ li(Rdst, (0x0));
    __ bind(done);
  
#line 14747 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint xorI_convI2Bool_reg_immIvalue1__cmoveNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 16, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 16);
}

void convI2Bool_andI_reg_immIpowerOf2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 3737 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rlwinm);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    long src2      = opnd_array(2)->constant();
    __ rlwinm(Rdst, Rsrc1, (32-log2_long((jlong) src2)) & 0x1f, 31, 31);
  
#line 14774 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convI2Bool_andI_reg_immIpowerOf2Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void convP2Bool_reg__cmoveNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// crx
  {

#line 3808 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* crx */);

    Label done;
    __ cmpdi(Rcrx, Rsrc, 0);
    __ li(Rdst, (0x0));
    __ beq(Rcrx, done);
    __ li(Rdst, (0x1));
    __ bind(done);
  
#line 14807 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convP2Bool_reg__cmoveNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 16, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 16);
}

void xorI_convP2Bool_reg_immIvalue1__cmoveNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// crx
  {

#line 3808 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(3)->reg(ra_,this,idx3)/* crx */);

    Label done;
    __ cmpdi(Rcrx, Rsrc, 0);
    __ li(Rdst, (0x1));
    __ beq(Rcrx, done);
    __ li(Rdst, (0x0));
    __ bind(done);
  
#line 14841 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint xorI_convP2Bool_reg_immIvalue1__cmoveNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 16, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 16);
}

void cmpLTMask_reg_immI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3615 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_srawi);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int isrc = ((0x1f)) & 0x1f;
    __ srawi(Rdst, Rsrc1, isrc);
  
#line 14868 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmpLTMask_reg_immI0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void convB2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// amount
  {

#line 2882 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_extsb);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ extsb(Rdst, Rsrc1);
  
#line 14894 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convB2I_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void convS2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// amount
  {

#line 2873 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_extsh);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ extsh(Rdst, Rsrc1);
  
#line 14920 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convS2I_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void sxtI_L2L_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2864 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_extsw);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ extsw(Rdst, Rsrc1);
  
#line 14945 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint sxtI_L2L_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void convL2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3890 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_or);
    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    if (Rsrc != Rdst) {
      __ mr(Rdst, Rsrc);
    }
  
#line 14971 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void convD2IRaw_regDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3874 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fctiwz);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fctiwz(Rdst, Rsrc1);
  
#line 14990 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convD2IRaw_regDNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmovI_bso_stackSlotLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 5566 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmove);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* crx */);
    Register Rbase= reg_to_register_object(opnd_array(2)->base(ra_,this,idx2));
    int Idisp     = opnd_array(2)->disp(ra_,this,idx2);
    if (opnd_array(2)->base(ra_,this,idx2) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    Label done;
    __ bso(Rcrx, done);
    __ ld(Rdst, Idisp, Rbase);
#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required.
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 15031 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovI_bso_stackSlotLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void  cmovI_bso_stackSlotL_conLvalue0_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  // Access to ins and operands for late expand.
  unsigned idx_dst   = idx0; 	// iRegIdst, 	dst
  unsigned idx_crx   = idx1; 	// flagsReg, 	crx
  unsigned idx_mem   = idx2; 	// stackSlotL, 	src
  Node    *n_region  = lookup(0);
  Node    *n_dst     = lookup(idx_dst);
  Node    *n_crx     = lookup(idx_crx);
  Node    *n_mem     = lookup(idx_mem);
  iRegIdstOper *op_dst = (iRegIdstOper *)opnd_array(0);
  flagsRegOper *op_crx = (flagsRegOper *)opnd_array(1);
  stackSlotLOper *op_mem = (stackSlotLOper *)opnd_array(2);
  Compile *C = ra_->C;
  {
#line 5590 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    //
    // replaces
    //
    //   region  dst  crx  mem
    //    \       |    |   /
    //     dst=cmovI_bso_stackSlotL_conLvalue0
    //
    // with
    //
    //   region  dst
    //    \       /
    //     dst=loadConI16(0)
    //      |
    //      ^  region  dst  crx  mem
    //      |   \       |    |    /
    //      dst=cmovI_bso_stackSlotL
    //

    // Create new nodes.
    MachNode *m1 = new (C) loadConI16Node();
    MachNode *m2 = new (C) cmovI_bso_stackSlotLNode();

    // inputs for new nodes
    m1->add_req(n_region);
    m2->add_req(n_region, n_crx, n_mem);

    // precedences for new nodes
    m2->add_prec(m1);

    // operands for new nodes
    m1->_opnds[0] = op_dst;
    m1->_opnds[1] = new (C) immI16Oper(0);

    m2->_opnds[0] = op_dst;
    m2->_opnds[1] = op_crx;
    m2->_opnds[2] = op_mem;

    // registers for new nodes
    ra_->set_pair(m1->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this)); // dst
    ra_->set_pair(m2->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this)); // dst

    // Insert new nodes.
    nodes->push(m1);
    nodes->push(m2);
  
#line 15105 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void convF2IRaw_regFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3874 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fctiwz);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fctiwz(Rdst, Rsrc1);
  
#line 15124 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convF2IRaw_regFNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void convI2L_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2846 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_extsw);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ extsw(Rdst, Rsrc1);
  
#line 15149 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convI2L_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void zeroExtendL_regINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 3856 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicl);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ clrldi(Rdst, Rsrc, 32);
  
#line 15175 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint zeroExtendL_regINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void zeroExtendL_regLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 3856 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicl);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ clrldi(Rdst, Rsrc, 32);
  
#line 15201 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint zeroExtendL_regLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void convF2LRaw_regFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3882 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fctiwz);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fctidz(Rdst, Rsrc1);
  
#line 15225 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convF2LRaw_regFNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmovL_bso_stackSlotLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 5566 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmove);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* crx */);
    Register Rbase= reg_to_register_object(opnd_array(2)->base(ra_,this,idx2));
    int Idisp     = opnd_array(2)->disp(ra_,this,idx2);
    if (opnd_array(2)->base(ra_,this,idx2) == 1) {
      Idisp += 0 /* TODO: PPC port ra_->C->frame_slots_sp_bias_in_bytes()*/;
    }
    Label done;
    __ bso(Rcrx, done);
    __ ld(Rdst, Idisp, Rbase);
#if 0 // TODO: PPC port
    if (_size == 12) {
      // Bundler will have changed our size to 8 if stop is not required.
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 15266 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovL_bso_stackSlotLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 12 : 8);
}

void  cmovL_bso_stackSlotL_conLvalue0_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  // Access to ins and operands for late expand.
  unsigned idx_dst   = idx0; 	// iRegLdst, 	dst
  unsigned idx_crx   = idx1; 	// flagsReg, 	crx
  unsigned idx_mem   = idx2; 	// stackSlotL, 	src
  Node    *n_region  = lookup(0);
  Node    *n_dst     = lookup(idx_dst);
  Node    *n_crx     = lookup(idx_crx);
  Node    *n_mem     = lookup(idx_mem);
  iRegLdstOper *op_dst = (iRegLdstOper *)opnd_array(0);
  flagsRegOper *op_crx = (flagsRegOper *)opnd_array(1);
  stackSlotLOper *op_mem = (stackSlotLOper *)opnd_array(2);
  Compile *C = ra_->C;
  {
#line 5637 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    //
    // replaces
    //
    //   region  dst  crx  mem
    //    \       |    |   /
    //     dst=cmovL_bso_stackSlotL_conLvalue0
    //
    // with
    //
    //   region  dst
    //    \       /
    //     dst=loadConL16(0)
    //      |
    //      ^  region  dst  crx  mem
    //      |   \       |    |    /
    //      dst=cmovL_bso_stackSlotL
    //

    // Create new nodes.
    MachNode *m1 = new (C) loadConL16Node();
    MachNode *m2 = new (C) cmovL_bso_stackSlotLNode();

    // inputs for new nodes
    m1->add_req(n_region);
    m2->add_req(n_region, n_crx, n_mem);

    // precedences for new nodes
    m2->add_prec(m1);

    // operands for new nodes
    m1->_opnds[0] = op_dst;
    m1->_opnds[1] = new (C) immL16Oper(0);

    m2->_opnds[0] = op_dst;
    m2->_opnds[1] = op_crx;
    m2->_opnds[2] = op_mem;

    // registers for new nodes
    ra_->set_pair(m1->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this)); // dst
    ra_->set_pair(m2->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this)); // dst

    // Insert new nodes.
    nodes->push(m1);
    nodes->push(m2);
  
#line 15340 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void convD2LRaw_regDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3882 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fctiwz);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fctidz(Rdst, Rsrc1);
  
#line 15359 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convD2LRaw_regDNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void convL2DRaw_regDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2828 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fcfid);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rtmp = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fcfid(Rdst, Rtmp);
  
#line 15384 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convL2DRaw_regDNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void convD2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3097 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_frsp);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst  = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ frsp(Rdst, Rsrc1);
  
#line 15409 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convD2F_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void convL2FRaw_regFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2837 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fcfid);

    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rtmp = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fcfids(Rdst, Rtmp);
  
#line 15434 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint convL2FRaw_regFNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void convF2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3900 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fmr);
    MacroAssembler _masm(&cbuf);
    FloatRegister Rdst = reg_to_FloatRegister_object(opnd_array(0)->reg(ra_,this)/* dst */);
    FloatRegister Rsrc = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    if (Rsrc != Rdst) {
      __ fmr(Rdst, Rsrc);
    }
  
#line 15460 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void cmpI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3443 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmp);
    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    __ cmpw(Rcrx, Rsrc1, Rsrc2);
  
#line 15481 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmpI_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmpI_reg_imm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3488 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmpi);
    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = opnd_array(2)->constant();
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    __ cmpwi(Rcrx, Rsrc1, Isrc2);
  
#line 15507 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmpI_reg_imm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void testI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 2739 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_andi_);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object((R0_enc));
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = opnd_array(2)->constant();
    // FIXME: avoid andi_ ?
    __ andi_(Rdst, Rsrc1, Isrc2);
  
#line 15536 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint testI_reg_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmpL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3470 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmp);
    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    __ cmpd(Rcrx, Rsrc1, Rsrc2);
  
#line 15562 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmpL_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmpL_reg_imm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {

#line 3479 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmpi);
    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = opnd_array(2)->constantL();
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    __ cmpdi(Rcrx, Rsrc1, Isrc2);
  
#line 15588 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmpL_reg_imm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmpUL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3452 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmpl);
    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    __ cmpld(Rcrx, Rsrc1, Rsrc2);
  
#line 15614 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmpUL_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmpUL_reg_imm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3506 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmpli);
    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    __ cmpldi(Rcrx, Rsrc1, opnd_array(2)->constantL());
  
#line 15639 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmpUL_reg_imm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void testL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 3514 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_and_);

    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ and_(R0, Rsrc1, Rsrc2);
  
#line 15666 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint testL_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void testL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 2739 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_andi_);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object((R0_enc));
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = opnd_array(2)->constantL();
    // FIXME: avoid andi_ ?
    __ andi_(Rdst, Rsrc1, Isrc2);
  
#line 15695 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint testL_reg_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmovI_conIvalueMinus1_conIvalue1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  {

#line 5406 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmove);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* crx */);
    Label done;
    // li(Rdst, 0);              // equal -> 0
    __ beq(Rcrx, done);
    __ li(Rdst, 1);              // greater -> +1
    __ bgt(Rcrx, done);
    __ li(Rdst, -1);             // unordered or less -> -1
#if 0 // TODO: PPC port
    if (_size == 20) {
      // Bundler will have changed our size to 16 if stop is not required.
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 15733 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmovI_conIvalueMinus1_conIvalue1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 20 : 16, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 20 : 16);
}

void  cmovI_conIvalueMinus1_conIvalue0_conIvalue1_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  // Access to ins and operands for late expand.
  unsigned idx_dst   = idx0; 	// iRegIdst, 	dst
  unsigned idx_crx   = idx1; 	// flagsReg, 	crx
  Node    *n_region  = lookup(0);
  Node    *n_dst     = lookup(idx_dst);
  Node    *n_crx     = lookup(idx_crx);
  iRegIdstOper *op_dst = (iRegIdstOper *)opnd_array(0);
  flagsRegOper *op_crx = (flagsRegOper *)opnd_array(1);
  Compile *C = ra_->C;
  {
#line 5360 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    //
    // replaces
    //
    //   region  crx
    //    \       |
    //     dst=cmovI_conIvalueMinus1_conIvalue0_conIvalue1
    //
    // with
    //
    //   region
    //    \
    //     dst=loadConI16(0)
    //      |
    //      ^  region  crx
    //      |   \       |
    //      dst=cmovI_conIvalueMinus1_conIvalue1
    //

    // Create new nodes.
    MachNode *m1 = new (C) loadConI16Node();
    MachNode *m2 = new (C) cmovI_conIvalueMinus1_conIvalue1Node();

    // inputs for new nodes
    m1->add_req(n_region);
    m2->add_req(n_region, n_crx);

    // precedences for new nodes
    m2->add_prec(m1);

    // operands for new nodes
    m1->_opnds[0] = op_dst;
    m1->_opnds[1] = new (C) immI16Oper(0);

    m2->_opnds[0] = op_dst;
    m2->_opnds[1] = op_crx;

    // registers for new nodes
    ra_->set_pair(m1->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this)); // dst
    ra_->set_pair(m2->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this)); // dst

    // Insert new nodes.
    nodes->push(m1);
    nodes->push(m2);
  
#line 15802 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void rangeCheck_iReg_uimm15Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src_length
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// index
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {

#line 4410 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_twi);

    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src_length */);
    int      Isrc2 = opnd_array(3)->constant();
    if (opnd_array(1)->ccode() == 0x1 /* less_equal */) {
      __ trap_range_check_le(Rsrc1, Isrc2);
    } else {
      // Both successors are uncommon traps, probability is 0.
      // Node got flipped during fixup flow.
      assert(opnd_array(1)->ccode() == 0x9, "must be greater");
      __ trap_range_check_g(Rsrc1, Isrc2);
    }
  
#line 15832 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint rangeCheck_iReg_uimm15Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void rangeCheck_iReg_iRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src_index
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src_length
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {

#line 4427 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_tw);

    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src_index */);
    Register Rsrc2 = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* src_length */);
    if (opnd_array(1)->ccode() == 0x0 /* greater_equal */) {
      __ trap_range_check_ge(Rsrc1, Rsrc2);
    } else {
      // Both successors are uncommon traps, probability is 0.
      // Node got flipped during fixup flow.
      assert(opnd_array(1)->ccode() == 0x8, "must be less");
      __ trap_range_check_l(Rsrc1, Rsrc2);
    }
  
#line 15867 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint rangeCheck_iReg_iRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void rangeCheck_uimm15_iRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src_index
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// length
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {

#line 4444 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_twi);

    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src_index */);
    int      Isrc2 = opnd_array(3)->constant();
    if (opnd_array(1)->ccode() == 0x0 /* greater_equal */) {
      __ trap_range_check_ge(Rsrc1, Isrc2);
    } else {
      // Both successors are uncommon traps, probability is 0.
      // Node got flipped during fixup flow.
      assert(opnd_array(1)->ccode() == 0x8, "must be less");
      __ trap_range_check_l(Rsrc1, Isrc2);
    }
  
#line 15902 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint rangeCheck_uimm15_iRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void compU_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3461 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmpl);
    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    __ cmplw(Rcrx, Rsrc1, Rsrc2);
  
#line 15928 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint compU_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void compU_reg_uimm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3497 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmpli);
    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = opnd_array(2)->constant();
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    __ cmplwi(Rcrx, Rsrc1, Isrc2);
  
#line 15954 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint compU_reg_uimm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void zeroCheckN_iReg_imm0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// value
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {

#line 4461 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_tdi);
    MacroAssembler _masm(&cbuf);
    Register Rvalue = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* value */);
    if (opnd_array(1)->ccode() == 0xA) {
      __ trap_null_check(Rvalue);
    } else {
      // Both successors are uncommon traps, probability is 0.
      // Node got flipped during fixup flow.
      assert(opnd_array(1)->ccode() == 0x2 , "must be equal(0xA) or notEqual(0x2)");
      __ trap_null_check(Rvalue, Assembler::traptoGreaterThanUnsigned);
    }
  
#line 15987 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint zeroCheckN_iReg_imm0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmpN_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3461 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmpl);
    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    __ cmplw(Rcrx, Rsrc1, Rsrc2);
  
#line 16013 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmpN_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmpN_reg_imm0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3497 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmpli);
    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = opnd_array(2)->constant();
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    __ cmplwi(Rcrx, Rsrc1, Isrc2);
  
#line 16039 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmpN_reg_imm0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void zeroCheckP_reg_imm0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// value
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {

#line 4461 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_tdi);
    MacroAssembler _masm(&cbuf);
    Register Rvalue = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* value */);
    if (opnd_array(1)->ccode() == 0xA) {
      __ trap_null_check(Rvalue);
    } else {
      // Both successors are uncommon traps, probability is 0.
      // Node got flipped during fixup flow.
      assert(opnd_array(1)->ccode() == 0x2 , "must be equal(0xA) or notEqual(0x2)");
      __ trap_null_check(Rvalue, Assembler::traptoGreaterThanUnsigned);
    }
  
#line 16072 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint zeroCheckP_reg_imm0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmpP_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3452 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmpl);
    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register Rsrc2 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    __ cmpld(Rcrx, Rsrc1, Rsrc2);
  
#line 16098 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmpP_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmpP_reg_imm16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {

#line 3479 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmpi);
    MacroAssembler _masm(&cbuf);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int Isrc2 = opnd_array(2)->constantL();
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    __ cmpdi(Rcrx, Rsrc1, Isrc2);
  
#line 16124 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmpP_reg_imm16Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmpFUnordered_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3865 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fcmpu);
    MacroAssembler _masm(&cbuf);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    FloatRegister Rsrc2 = reg_to_FloatRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ fcmpu(Rcrx, Rsrc1, Rsrc2);
  
#line 16150 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmpFUnordered_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmov_bns_lessNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 5249 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cmovecr);

    MacroAssembler _masm(&cbuf);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    Label done;
    __ bns(Rcrx, done);        // not unordered -> keep crx
    __ li(R0, 0);
    __ cmpwi(Rcrx, R0, 1);     // unordered -> set crx to 'less'
#if 0 // TODO: PPC port
    if (_size == 16) {
      // Bundler will have changed our size if stop is not required
      __ endgroup();
    }
#endif
    if (false /* TODO: PPC PORT ra_->C->do_hb_scheduling()*/) Unimplemented();
    __ bind(done);
  
#line 16185 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmov_bns_lessNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 16 : 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : false /* TODO: PPC PORT Compile::current()->do_hb_scheduling()*/ ? 16 : 12);
}

void  cmpF_reg_reg_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  // Access to ins and operands for late expand.
  unsigned idx_crx   = idx0; 	// flagsReg, 	crx
  unsigned idx_src1  = idx1; 	// regF, 	src1
  unsigned idx_src2  = idx2; 	// regF, 	src2
  Node    *n_region  = lookup(0);
  Node    *n_crx     = lookup(idx_crx);
  Node    *n_src1    = lookup(idx_src1);
  Node    *n_src2    = lookup(idx_src2);
  flagsRegOper *op_crx = (flagsRegOper *)opnd_array(0);
  regFOper *op_src1 = (regFOper *)opnd_array(1);
  regFOper *op_src2 = (regFOper *)opnd_array(2);
  Compile *C = ra_->C;
  {
#line 5268 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    //
    // replaces
    //
    //   region  src1  src2
    //    \       |     |
    //     crx=cmpF_reg_reg
    //
    // with
    //
    //   region  src1  src2
    //    \       |     |
    //     crx=cmpFUnordered_reg_reg
    //      |
    //      ^  region
    //      |   \
    //      crx=cmov_bns_less
    //

    // Create new nodes.
    MachNode *m1 = new (C) cmpFUnordered_reg_regNode();
    MachNode *m2 = new (C) cmov_bns_lessNode();

    // inputs for new nodes
    m1->add_req(n_region, n_src1, n_src2);
    m2->add_req(n_region);

    // precedences for new nodes
    m2->add_prec(m1);

    // operands for new nodes
    m1->_opnds[0] = op_crx;
    m1->_opnds[1] = op_src1;
    m1->_opnds[2] = op_src2;

    m2->_opnds[0] = op_crx;

    // registers for new nodes
    ra_->set_pair(m1->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this)); // crx
    ra_->set_pair(m2->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this)); // crx

    // Insert new nodes.
    nodes->push(m1);
    nodes->push(m2);
  
#line 16258 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void cmpDUnordered_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3865 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fcmpu);
    MacroAssembler _masm(&cbuf);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* crx */);
    FloatRegister Rsrc1 = reg_to_FloatRegister_object(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    FloatRegister Rsrc2 = reg_to_FloatRegister_object(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ fcmpu(Rcrx, Rsrc1, Rsrc2);
  
#line 16279 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint cmpDUnordered_reg_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void  cmpD_reg_reg_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  // Access to ins and operands for late expand.
  unsigned idx_crx   = idx0; 	// flagsReg, 	crx
  unsigned idx_src1  = idx1; 	// regD, 	src1
  unsigned idx_src2  = idx2; 	// regD, 	src2
  Node    *n_region  = lookup(0);
  Node    *n_crx     = lookup(idx_crx);
  Node    *n_src1    = lookup(idx_src1);
  Node    *n_src2    = lookup(idx_src2);
  flagsRegOper *op_crx = (flagsRegOper *)opnd_array(0);
  regDOper *op_src1 = (regDOper *)opnd_array(1);
  regDOper *op_src2 = (regDOper *)opnd_array(2);
  Compile *C = ra_->C;
  {
#line 5314 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    //
    // replaces
    //
    //   region  src1  src2
    //    \       |     |
    //     crx=cmpD_reg_reg
    //
    // with
    //
    //   region  src1  src2
    //    \       |     |
    //     crx=cmpDUnordered_reg_reg
    //      |
    //      ^  region
    //      |   \
    //      crx=cmov_bns_less
    //

    // create new nodes
    MachNode *m1 = new (C) cmpDUnordered_reg_regNode();
    MachNode *m2 = new (C) cmov_bns_lessNode();

    // inputs for new nodes
    m1->add_req(n_region, n_src1, n_src2);
    m2->add_req(n_region);

    // precedences for new nodes
    m2->add_prec(m1);

    // operands for new nodes
    m1->_opnds[0] = op_crx;
    m1->_opnds[1] = op_src1;
    m1->_opnds[2] = op_src2;

    m2->_opnds[0] = op_crx;

    // registers for new nodes
    ra_->set_pair(m1->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this)); // crx
    ra_->set_pair(m2->_idx, ra_->get_reg_second(this), ra_->get_reg_first(this)); // crx

    // Insert new nodes.
    nodes->push(m1);
    nodes->push(m2);
  
#line 16352 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 3911 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_b);

     MacroAssembler _masm(&cbuf);
     Label d;    // dummy
     __ bind(d);
     Label* p = (opnd_array(1)->label());
     // `p' is `NULL' when this encoding class is used only to
     // determine the size of the encoded instruction.
     Label& l = (NULL == p)? d : *(p);
     __ b(l);
  
#line 16376 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint branchNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void branchConNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lbl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 3924 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_bc);

    MacroAssembler _masm(&cbuf);
    Label d;   // dummy
    __ bind(d);
    Label* p = (opnd_array(3)->label());
    // `p' is `NULL' when this encoding class is used only to
    // determine the size of the encoded instruction.
    Label& l = (NULL == p)? d : *(p);
    int cc = opnd_array(1)->ccode();
    int flags_reg = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    assert((Assembler::bcondCRbiIs1 & ~Assembler::bcondCRbiIs0) == 8, "check encoding");
    int bhint = Assembler::bhintNoHint;

    if (UseStaticBranchPredictionForUncommonPathsPPC64) {
      if (_prob <= PROB_NEVER) {
        bhint = Assembler::bhintIsNotTaken;
      } else if (_prob >= PROB_ALWAYS) {
        bhint = Assembler::bhintIsTaken;
      }
    }

    __ bc(Assembler::add_bhint_to_boint(bhint, cc_to_boint(cc)),
          cc_to_biint(cc, flags_reg),
          l);
  
#line 16422 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint branchConNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void branchConFarNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lbl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 3952 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // The scheduler doesn't know about branch shortening, so we set the opcode
    // to ppc64Opcode_bc in order to hide this detail from the scheduler.
    // TODO: PPC port $archOpcode(ppc64Opcode_bc);

    MacroAssembler _masm(&cbuf);
    Label d;    // dummy
    __ bind(d);
    Label* p = (opnd_array(3)->label());
    // `p' is `NULL' when this encoding class is used only to
    // determine the size of the encoded instruction.
    Label& l = (NULL == p)? d : *(p);
    int cc = opnd_array(1)->ccode();
    int flags_reg = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    int bhint = Assembler::bhintNoHint;

    if (UseStaticBranchPredictionForUncommonPathsPPC64) {
      if (_prob <= PROB_NEVER) {
        bhint = Assembler::bhintIsNotTaken;
      } else if (_prob >= PROB_ALWAYS) {
        bhint = Assembler::bhintIsTaken;
      }
    }

    // Tell the conditional far branch to optimize itself when being relocated.
    __ bc_far(Assembler::add_bhint_to_boint(bhint, cc_to_boint(cc)),
                  cc_to_biint(cc, flags_reg),
                  l,
                  MacroAssembler::bc_far_optimize_on_relocate);
  
#line 16471 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint branchConFarNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 8);
}

void branchConSchedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lbl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 3984 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // The scheduler doesn't know about branch shortening, so we set the opcode
    // to ppc64Opcode_bc in order to hide this detail from the scheduler.
    // TODO: PPC port $archOpcode(ppc64Opcode_bc);

    MacroAssembler _masm(&cbuf);
    Label d;   // dummy
    __ bind(d);
    Label* p = (opnd_array(3)->label());
    // `p' is `NULL' when this encoding class is used only to
    // determine the size of the encoded instruction.
    Label& l = (NULL == p)? d : *(p);
    int cc = opnd_array(1)->ccode();
    int flags_reg = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    int bhint = Assembler::bhintNoHint;

    if (UseStaticBranchPredictionForUncommonPathsPPC64) {
      if (_prob <= PROB_NEVER) {
        bhint = Assembler::bhintIsNotTaken;
      } else if (_prob >= PROB_ALWAYS) {
        bhint = Assembler::bhintIsTaken;
      }
    }

#if 0 // TODO: PPC port
    if (_size == 8) {
      // Tell the conditional far branch to optimize itself when being relocated.
      __ bc_far(Assembler::add_bhint_to_boint(bhint, cc_to_boint(cc)),
                    cc_to_biint(cc, flags_reg),
                    l,
                    MacroAssembler::bc_far_optimize_on_relocate);
    } else {
      __ bc    (Assembler::add_bhint_to_boint(bhint, cc_to_boint(cc)),
                    cc_to_biint(cc, flags_reg),
                    l);
    }
#endif
    Unimplemented();
  
#line 16529 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint branchConSchedNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 8);
}

void branchLoopEndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 3924 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_bc);

    MacroAssembler _masm(&cbuf);
    Label d;   // dummy
    __ bind(d);
    Label* p = (opnd_array(3)->label());
    // `p' is `NULL' when this encoding class is used only to
    // determine the size of the encoded instruction.
    Label& l = (NULL == p)? d : *(p);
    int cc = opnd_array(1)->ccode();
    int flags_reg = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    assert((Assembler::bcondCRbiIs1 & ~Assembler::bcondCRbiIs0) == 8, "check encoding");
    int bhint = Assembler::bhintNoHint;

    if (UseStaticBranchPredictionForUncommonPathsPPC64) {
      if (_prob <= PROB_NEVER) {
        bhint = Assembler::bhintIsNotTaken;
      } else if (_prob >= PROB_ALWAYS) {
        bhint = Assembler::bhintIsTaken;
      }
    }

    __ bc(Assembler::add_bhint_to_boint(bhint, cc_to_boint(cc)),
          cc_to_biint(cc, flags_reg),
          l);
  
#line 16575 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint branchLoopEndNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void branchLoopEndFarNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 3952 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // The scheduler doesn't know about branch shortening, so we set the opcode
    // to ppc64Opcode_bc in order to hide this detail from the scheduler.
    // TODO: PPC port $archOpcode(ppc64Opcode_bc);

    MacroAssembler _masm(&cbuf);
    Label d;    // dummy
    __ bind(d);
    Label* p = (opnd_array(3)->label());
    // `p' is `NULL' when this encoding class is used only to
    // determine the size of the encoded instruction.
    Label& l = (NULL == p)? d : *(p);
    int cc = opnd_array(1)->ccode();
    int flags_reg = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    int bhint = Assembler::bhintNoHint;

    if (UseStaticBranchPredictionForUncommonPathsPPC64) {
      if (_prob <= PROB_NEVER) {
        bhint = Assembler::bhintIsNotTaken;
      } else if (_prob >= PROB_ALWAYS) {
        bhint = Assembler::bhintIsTaken;
      }
    }

    // Tell the conditional far branch to optimize itself when being relocated.
    __ bc_far(Assembler::add_bhint_to_boint(bhint, cc_to_boint(cc)),
                  cc_to_biint(cc, flags_reg),
                  l,
                  MacroAssembler::bc_far_optimize_on_relocate);
  
#line 16624 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint branchLoopEndFarNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 8);
}

void branchLoopEndSchedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// crx
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 3984 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // The scheduler doesn't know about branch shortening, so we set the opcode
    // to ppc64Opcode_bc in order to hide this detail from the scheduler.
    // TODO: PPC port $archOpcode(ppc64Opcode_bc);

    MacroAssembler _masm(&cbuf);
    Label d;   // dummy
    __ bind(d);
    Label* p = (opnd_array(3)->label());
    // `p' is `NULL' when this encoding class is used only to
    // determine the size of the encoded instruction.
    Label& l = (NULL == p)? d : *(p);
    int cc = opnd_array(1)->ccode();
    int flags_reg = opnd_array(2)->reg(ra_,this,idx2)/* crx */;
    int bhint = Assembler::bhintNoHint;

    if (UseStaticBranchPredictionForUncommonPathsPPC64) {
      if (_prob <= PROB_NEVER) {
        bhint = Assembler::bhintIsNotTaken;
      } else if (_prob >= PROB_ALWAYS) {
        bhint = Assembler::bhintIsTaken;
      }
    }

#if 0 // TODO: PPC port
    if (_size == 8) {
      // Tell the conditional far branch to optimize itself when being relocated.
      __ bc_far(Assembler::add_bhint_to_boint(bhint, cc_to_boint(cc)),
                    cc_to_biint(cc, flags_reg),
                    l,
                    MacroAssembler::bc_far_optimize_on_relocate);
    } else {
      __ bc    (Assembler::add_bhint_to_boint(bhint, cc_to_boint(cc)),
                    cc_to_biint(cc, flags_reg),
                    l);
    }
#endif
    Unimplemented();
  
#line 16682 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint branchLoopEndSchedNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 8);
}

void partialSubtypeCheckNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// subklass
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// superklass
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// result
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp_klass
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp_arrayptr
  {

#line 4510 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);

    const Register reg_result     = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* result */);
    const Register reg_subklass   = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* subklass */);
    const Register reg_superklass = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* superklass */);
    const Register reg_klass      = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4)/* tmp_klass */);    // index into cache array
    const Register reg_arrayptr   = reg_to_register_object(opnd_array(5)->reg(ra_,this,idx5)/* tmp_arrayptr */); // current value from cache array

    __ check_klass_subtype_slow_path(reg_subklass, reg_superklass, reg_arrayptr, reg_klass, NULL, reg_result);
  
#line 16715 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void cmpFastLockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  {

#line 5212 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);

    Register displaced_header = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* tmp1 */);
    Register current_header = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4)/* tmp2 */);
    Register temp = reg_to_register_object(opnd_array(5)->reg(ra_,this,idx5)/* tmp3 */);
    Register box = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* box */);
    Register oop = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* oop */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* pcc */);

    __ compiler_fast_lock_object(Rcrx, oop, box, temp, displaced_header, current_header);

    // if locking was successfull, Rcrx should indicate 'EQ'.
    // the compiler generates a branch to the runtime call to
    // _complete_monitor_locking_Java for the case where Rcrx is 'NE'.
  
#line 16748 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void cmpFastUnlockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  {

#line 5231 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);

    Register displaced_header = reg_to_register_object(opnd_array(3)->reg(ra_,this,idx3)/* tmp1 */);
    Register current_header = reg_to_register_object(opnd_array(4)->reg(ra_,this,idx4)/* tmp2 */);
    Register temp = reg_to_register_object(opnd_array(5)->reg(ra_,this,idx5)/* tmp3 */);
    Register box = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* box */);
    Register oop = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* oop */);
    ConditionRegister Rcrx = reg_to_ConditionRegister_object(opnd_array(0)->reg(ra_,this)/* pcc */);

    __ compiler_fast_unlock_object(Rcrx, oop, box, temp, displaced_header, current_header);

    // If unlocking was successfull, Rcrx should indicate 'EQ'.
    // the compiler generates a branch to the runtime call to
    // _complete_monitor_unlocking_Java for the case where Rcrx is 'NE'.
  
#line 16781 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void align_addrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 3707 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldicr);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    long src2      = opnd_array(2)->constantL();
    __ clrrdi(Rdst, Rsrc1, log2_long((jlong)-src2));
  
#line 16803 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint align_addrNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void array_sizeNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// end
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// start
  {

#line 3000 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_subf);

    MacroAssembler _masm(&cbuf);
    Register Rdst  = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc1 = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* start */);
    Register Rsrc2 = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* end */);
    __ subf(Rdst, Rsrc1, Rsrc2);
  
#line 16830 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint array_sizeNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void inlineCallClearArrayNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  {

#line 5435 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register cnt_dwords_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* cnt */);
    Register base_ptr_reg   = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* base */);

    __ clear_memory_doubleword(base_ptr_reg, cnt_dwords_reg); // kills cnt, base, R0
  
#line 16857 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void string_indexOf_imm1_charNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// haystack
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// haycnt
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// needleImm
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// offsetImm
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// needlecntImm
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// result
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp1
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp2
  {

#line 5476 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);

    immPOper *needleOper = (immPOper *)opnd_array(3);
    const TypeOopPtr *t = needleOper->type()->isa_oopptr();
    ciTypeArray* needle_values = t->const_oop()->as_type_array();  // Pointer to live char *
    assert(needle_values, "");

    immLOper *offsetOper = (immLOper *)opnd_array(4);
    long off = offsetOper->constant();
    guarantee(!UseCompressedOops || off == 16, "not beginning of char[]");

    __ string_indexof_1(opnd_array(6)->as_Register(ra_,this,idx6)/* result */,
                        opnd_array(1)->as_Register(ra_,this,idx1)/* haystack */, opnd_array(2)->as_Register(ra_,this,idx2)/* haycnt */,
                        R0, needle_values->char_at(0),
                        opnd_array(7)->as_Register(ra_,this,idx7)/* tmp1 */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp2 */);
  
#line 16894 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void string_indexOf_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// haystack
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// haycnt
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// needle
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// needlecntImm
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp1
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp2
  {

#line 5497 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);

    Node *ndl = in(operand_index(opnd_array(3)));  // The node that defines needle.
    ciTypeArray* needle_values = ndl->bottom_type()->is_aryptr()->const_oop()->as_type_array();
    guarantee(needle_values, "sanity");
    if (needle_values != NULL) {
      __ string_indexof_1(opnd_array(5)->as_Register(ra_,this,idx5)/* result */,
                          opnd_array(1)->as_Register(ra_,this,idx1)/* haystack */, opnd_array(2)->as_Register(ra_,this,idx2)/* haycnt */,
                          R0, needle_values->char_at(0),
                          opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */);
    } else {
      __ string_indexof_1(opnd_array(5)->as_Register(ra_,this,idx5)/* result */,
                          opnd_array(1)->as_Register(ra_,this,idx1)/* haystack */, opnd_array(2)->as_Register(ra_,this,idx2)/* haycnt */,
                          opnd_array(3)->as_Register(ra_,this,idx3)/* needle */, 0,
                          opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */);
    }
  
#line 16931 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void string_indexOf_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// haystack
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// haycnt
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// needle
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// needlecntImm
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp1
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp2
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp3
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp4
  unsigned idx10 = idx9 + opnd_array(9)->num_edges(); 	// tmp5
  {

#line 5518 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    int needlecnt = opnd_array(4)->constant();
    Node *ndl = in(operand_index(opnd_array(3)));  // The node that defines needle.
    ciTypeArray* needle_values = ndl->bottom_type()->is_aryptr()->const_oop()->as_type_array();

    __ string_indexof(opnd_array(5)->as_Register(ra_,this,idx5)/* result */,
                      opnd_array(1)->as_Register(ra_,this,idx1)/* haystack */, opnd_array(2)->as_Register(ra_,this,idx2)/* haycnt */,
                      opnd_array(3)->as_Register(ra_,this,idx3)/* needle */, needle_values, opnd_array(10)->as_Register(ra_,this,idx10)/* tmp5 */, needlecnt,
                      opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp3 */, opnd_array(9)->as_Register(ra_,this,idx9)/* tmp4 */);
  
#line 16964 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void string_indexOfNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// haystack
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// haycnt
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// needle
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// needlecnt
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp1
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp2
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp3
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp4
  {

#line 5532 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    __ string_indexof(opnd_array(5)->as_Register(ra_,this,idx5)/* result */,
                      opnd_array(1)->as_Register(ra_,this,idx1)/* haystack */, opnd_array(2)->as_Register(ra_,this,idx2)/* haycnt */,
                      opnd_array(3)->as_Register(ra_,this,idx3)/* needle */, NULL, opnd_array(4)->as_Register(ra_,this,idx4)/* needlecnt */, 0,  // needlecnt not constant.
                      opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp3 */, opnd_array(9)->as_Register(ra_,this,idx9)/* tmp4 */);
  
#line 16992 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void string_equals_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cntImm
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// result
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  {

#line 5542 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    int cntval = opnd_array(3)->constant();
    MacroAssembler _masm(&cbuf);
    __ char_arrays_equalsImm(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */, cntval, opnd_array(4)->as_Register(ra_,this,idx4)/* result */,
                             opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */);
  
#line 17016 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void string_equalsNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// result
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp4
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp5
  {

#line 5551 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    __ char_arrays_equals(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */, opnd_array(3)->as_Register(ra_,this,idx3)/* cnt */, opnd_array(4)->as_Register(ra_,this,idx4)/* result */,
                          opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp4 */, opnd_array(9)->as_Register(ra_,this,idx9)/* tmp5 */);
  
#line 17042 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void string_compareNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp
  {

#line 5559 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      opnd_array(5)->as_Register(ra_,this,idx5)/* result */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp */);
  
#line 17065 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void popCountINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 6067 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_popcntb);
    MacroAssembler _masm(&cbuf);
    Register src_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register dst_reg = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    __ popcntw(dst_reg, src_reg);
  
#line 17084 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint popCountINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void popCountLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 6075 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_popcntb);
    MacroAssembler _masm(&cbuf);
    Register src_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register dst_reg = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    __ popcntd(dst_reg, src_reg);
  
#line 17108 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint popCountLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void countLeadingZerosINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 4024 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cntlzw);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ cntlzw(Rdst, Rsrc);
  
#line 17133 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint countLeadingZerosINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void countLeadingZerosLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 4033 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cntlzd);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ cntlzd(Rdst, Rsrc);
  
#line 17158 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint countLeadingZerosLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void countLeadingZerosPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 4033 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_cntlzd);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ cntlzd(Rdst, Rsrc);
  
#line 17183 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint countLeadingZerosPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void insrwi_aNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// imm2
  {

#line 6058 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rlwimi);
    MacroAssembler _masm(&cbuf);
    Register src_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register dst_reg = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);

    __ insrwi(dst_reg, src_reg, opnd_array(3)->constant(), opnd_array(2)->constant());
  
#line 17210 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint insrwi_aNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void insrwiNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// imm2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {

#line 6058 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rlwimi);
    MacroAssembler _masm(&cbuf);
    Register src_reg = reg_to_register_object(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    Register dst_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* dst */);

    __ insrwi(dst_reg, src_reg, opnd_array(4)->constant(), opnd_array(3)->constant());
  
#line 17238 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint insrwiNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void repl32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 6049 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldimi);
    MacroAssembler _masm(&cbuf);
    Register src_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    Register dst_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* dst */);

    __ insrdi(dst_reg, src_reg, ((32)), ((0)));
  
#line 17263 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint repl32Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void repl48Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 6049 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldimi);
    MacroAssembler _masm(&cbuf);
    Register src_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    Register dst_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* dst */);

    __ insrdi(dst_reg, src_reg, ((48)), ((0)));
  
#line 17288 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint repl48Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void repl56Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 6049 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_rldimi);
    MacroAssembler _masm(&cbuf);
    Register src_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    Register dst_reg = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* dst */);

    __ insrdi(dst_reg, src_reg, ((56)), ((0)));
  
#line 17313 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint repl56Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void repl8B_immI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {

#line 4042 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    int v = opnd_array(1)->constant();
    __ li(Rdst, (int)((short) (v & 0xFFFF)));
  
#line 17338 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint repl8B_immI0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void repl8B_immIminus1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 4042 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    int v = opnd_array(1)->constant();
    __ li(Rdst, (int)((short) (v & 0xFFFF)));
  
#line 17363 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint repl8B_immIminus1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void repl4S_immI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {

#line 4042 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    int v = opnd_array(1)->constant();
    __ li(Rdst, (int)((short) (v & 0xFFFF)));
  
#line 17388 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint repl4S_immI0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void repl4S_immIminus1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 4042 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    int v = opnd_array(1)->constant();
    __ li(Rdst, (int)((short) (v & 0xFFFF)));
  
#line 17413 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint repl4S_immIminus1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void repl2I_immI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {

#line 4042 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    int v = opnd_array(1)->constant();
    __ li(Rdst, (int)((short) (v & 0xFFFF)));
  
#line 17438 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint repl2I_immI0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void repl2I_immIminus1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 4042 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    int v = opnd_array(1)->constant();
    __ li(Rdst, (int)((short) (v & 0xFFFF)));
  
#line 17463 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint repl2I_immIminus1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void  repl2F_immF_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  // Access to ins and operands for late expand.
  unsigned idx_dst   = idx0; 	// iRegLdst, 	dst
  unsigned idx_src   = idx1; 	// immF, 	src
  unsigned idx_toc   = mach_constant_base_node_input(); 	// iRegPdst, 	constanttablebase
  Node    *n_region  = lookup(0);
  Node    *n_dst     = lookup(idx_dst);
  Node    *n_src     = lookup(idx_src);
  Node    *n_toc     = lookup(idx_toc);
  iRegLdstOper *op_dst = (iRegLdstOper *)opnd_array(0);
  immFOper *op_src = (immFOper *)opnd_array(1);
  Compile *C = ra_->C;
  {
#line 4077 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // Create new nodes.

    // Make an operand with the bit pattern to load as float.
    immLOper *op_repl = new (C) immLOper((jlong)replicate_immF(op_src->constantF()));

    loadConLNodesTuple loadConLNodes =
      loadConLNodesTuple_create(C, ra_, n_toc, op_repl, false, ra_->get_reg_second(this), ra_->get_reg_first(this));

    // Push new nodes.
    if (loadConLNodes._large_hi) nodes->push(loadConLNodes._large_hi);
    if (loadConLNodes._last)     nodes->push(loadConLNodes._last);

    assert(nodes->length() >= 1, "must have created at least 1 node");
    assert(loadConLNodes._last->bottom_type()->isa_long(), "must be long");
  
#line 17505 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void repl2F_immF_ExNode::eval_constant(Compile* C) {
  // User did not define an encode section.
}
void repl2F_immF0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {

#line 4042 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_addi);

    MacroAssembler _masm(&cbuf);
    Register Rdst = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* dst */);
    int v = (0x0);
    __ li(Rdst, (int)((short) (v & 0xFFFF)));
  
#line 17528 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void safePoint_pollNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 6029 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ld);
    // Fake operand dst needed for PPC scheduler.
    assert((0x0) == 0x0, "dst must be 0x0");

    MacroAssembler _masm(&cbuf);
    // Mark the code position where the load from the safepoint
    // polling page was emitted as relocInfo::poll_type.
    __ relocate(relocInfo::poll_type);
    __ load_from_polling_page(opnd_array(1)->as_Register(ra_,this,idx1)/* poll */);
  
#line 17551 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint safePoint_pollNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void loadConPollAddrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 6041 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);

    Register Rpoll = reg_to_register_object(opnd_array(0)->reg(ra_,this)/* poll */);
    __ load_const_optimized(Rpoll, (long)(address) os::get_polling_page()); // TODO: PPC port get_standard_polling_page());
  
#line 17575 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void safePoint_poll_conPollAddrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 6029 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_ld);
    // Fake operand dst needed for PPC scheduler.
    assert((0x0) == 0x0, "dst must be 0x0");

    MacroAssembler _masm(&cbuf);
    // Mark the code position where the load from the safepoint
    // polling page was emitted as relocInfo::poll_type.
    __ relocate(relocInfo::poll_type);
    __ load_from_polling_page(opnd_array(1)->as_Register(ra_,this,idx1)/* poll */);
  
#line 17598 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void CallStaticJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 4926 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_bl);

    MacroAssembler _masm(&cbuf);
    relocInfo::relocType reloc;

    if (!_method) {
      // A call to a runtime wrapper, e.g. new, new_typeArray_Java, uncommon_trap.
      reloc = relocInfo::runtime_call_type;
    } else if (_optimized_virtual) {
      reloc = relocInfo::opt_virtual_call_type;
    } else {
      reloc = relocInfo::static_call_type;
    }

    // Puts 1 constant into the constant pool (call target).
    const EmitCallOffsets offsets = emit_call_with_trampoline_stub(_masm, (address)opnd_array(1)->method(), reloc);

    if (ra_->C->env()->failing())
      return;

    assert(offsets.insts_call_instruction_offset != -1, "must be initialized");

    if (_method) {
      // Create stub for static call. Puts 1 or 2 constants into constant
      // pool (inline cache, call target).
      emit_java_to_interp_stub(_masm, offsets.insts_call_instruction_offset);
    }
  
#line 17639 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint CallStaticJavaDirectNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void CallStaticJavaDirectHandleNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 4963 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    relocInfo::relocType reloc;
    assert(_optimized_virtual, "methodHandle call should be a virtual call");
    reloc = relocInfo::opt_virtual_call_type;

    const EmitCallOffsets offsets = emit_call_with_trampoline_stub(_masm, (address)opnd_array(1)->method(), reloc);
    // restore original sp
    __ ld(R11_scratch1, 0, R1_SP); // load caller sp
    Compile* C = ra_->C;
    const long framesize = C->frame_slots() << LogBytesPerInt;
    unsigned int bytes = (unsigned int)framesize;
    long offset = Assembler::align_addr(bytes, frame::alignment_in_bytes);
    if (Assembler::is_simm(-offset, 16)) {
      __ addi(R1_SP, R11_scratch1, -offset);
    } else {
      __ load_const_optimized(R12_scratch2, -offset);
      __ add(R1_SP, R11_scratch1, R12_scratch2);
    }
#ifdef ASSERT
  __ ld(R12_scratch2, 0, R1_SP); // Load from unextended_sp.
  __ cmpd(CCR0, R11_scratch1, R12_scratch2);
  __ asm_assert_eq("backlink changed", 0x8000);
#endif
    // If fails should store backlink before unextending.

    if (ra_->C->env()->failing())
      return;

    assert(offsets.insts_call_instruction_offset != -1, "must be initialized");

    if (_method) {
      // Create stub for static call.
      emit_java_to_interp_stub(_masm, offsets.insts_call_instruction_offset);
    }
  
#line 17694 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void CallDynamicJavaDirectSchedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 5065 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_bl);

    MacroAssembler _masm(&cbuf);

    if (!ra_->C->in_scratch_emit_size()) {
      // Create a call trampoline stub for the given method.
      const address entry_point = !(opnd_array(1)->method()) ? 0 : (address)opnd_array(1)->method();
      const address entry_point_const = __ address_constant(entry_point, RelocationHolder::none);
      const int entry_point_const_toc_offset = __ offset_to_method_toc(entry_point_const);
      emit_trampoline_stub(_masm, entry_point_const_toc_offset, __ offset());

      if (ra_->C->env()->failing())
        return;

      // Build relocation at call site with ic position as data.
      assert((_load_ic_hi_node != NULL && _load_ic_node == NULL) ||
             (_load_ic_hi_node == NULL && _load_ic_node != NULL),
             "must have one, but can't have both");
      assert((_load_ic_hi_node != NULL && _load_ic_hi_node->_cbuf_insts_offset != -1) ||
             (_load_ic_node != NULL    && _load_ic_node->_cbuf_insts_offset != -1),
             "must contain instruction offset");
      const int virtual_call_oop_addr_offset = _load_ic_hi_node != NULL
        ? _load_ic_hi_node->_cbuf_insts_offset
        : _load_ic_node->_cbuf_insts_offset;
      const address virtual_call_oop_addr = __ addr_at(virtual_call_oop_addr_offset);
      assert(MacroAssembler::is_load_const_from_method_toc_at(virtual_call_oop_addr),
             "should be load from TOC");

      __ relocate(virtual_call_Relocation::spec(virtual_call_oop_addr));
    }

    // At this point I do not have the address of the trampoline stub,
    // and the entry point might be too far away for bl. Pc() serves
    // as dummy and bl will be patched later.
    __ bl((address) __ pc());
  
#line 17743 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint CallDynamicJavaDirectSchedNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void  CallDynamicJavaDirectSched_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // Access to ins and operands for late expand.
  unsigned idx_meth  = idx1; 	// method, 	meth
  Node    *n_region  = lookup(0);
  Node    *n_meth    = lookup(idx_meth);
  methodOper *op_meth = (methodOper *)opnd_array(1);
  Compile *C = ra_->C;
  {
#line 5003 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // Toc is in return address field, though not accessible via lateExpand
    // functionaliy.
    Node *toc = in(TypeFunc::ReturnAdr);

    // Create the nodes for loading the IC from the TOC.
    loadConLNodesTuple loadConLNodes_IC =
      loadConLNodesTuple_create(C, ra_, toc, new (C) immLOper((jlong)Universe::non_oop_word()),
                                true, OptoReg::Name(R19_H_num), OptoReg::Name(R19_num));

    // Create the call node.
    CallDynamicJavaDirectSchedNode *call = new (C) CallDynamicJavaDirectSchedNode();
    call->_method_handle_invoke = _method_handle_invoke;
    call->_vtable_index      = _vtable_index;
    call->_method            = _method;
    call->_bci               = _bci;
    call->_optimized_virtual = _optimized_virtual;
    call->_tf                = _tf;
    call->_entry_point       = _entry_point;
    call->_cnt               = _cnt;
    call->_argsize           = _argsize;
    call->_oop_map           = _oop_map;
    call->_jvms              = _jvms;
    call->_jvmadj            = _jvmadj;
    call->_in_rms            = _in_rms;
    call->_nesting           = _nesting;

    // New call needs all inputs of old call.
    // Req...
    for (uint i = 0; i < req(); ++i) {
      if (i != TypeFunc::ReturnAdr) {
        call->add_req(in(i));
      } else {
        // The expanded node does not need toc any more.
        call->add_req(C->top());
      }
    }
    // ...as well as prec
    for (uint i = req(); i < len() ; ++i) {
      call->add_prec(in(i));
    }

    // The cache must come before the call, but it's not a req edge.
    call->add_prec(loadConLNodes_IC._last);
    // Remember nodes loading the inline cache into r19.
    call->_load_ic_hi_node = loadConLNodes_IC._large_hi;
    call->_load_ic_node    = loadConLNodes_IC._small;

    // Operands for new nodes.
    call->_opnds[0] = _opnds[0];
    call->_opnds[1] = _opnds[1];

    // Only the inline cache is associated with a register.
    assert(Matcher::inline_cache_reg() == OptoReg::Name(R19_num), "ic reg should be R19");

    // Push new nodes.
    if (loadConLNodes_IC._large_hi) nodes->push(loadConLNodes_IC._large_hi);
    if (loadConLNodes_IC._last)     nodes->push(loadConLNodes_IC._last);
    nodes->push(call);
  
#line 17824 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void CallDynamicJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 5104 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    int start_offset = __ offset();

    Register Rtoc = (ra_)
                    ? as_Register(Matcher::_regEncode[ra_->get_reg_first(in(4))])
                    : R2_TOC;

    int vtable_index = this->_vtable_index;
#if 0  // TODO: PPC port
    if (!methodOopDesc::is_valid_virtual_vtable_index(vtable_index)) {
      // Must be invalid_vtable_index, not nonvirtual_vtable_index.
      assert(vtable_index == methodOopDesc::invalid_vtable_index, "correct sentinel value");
      Register ic_reg = reg_to_register_object(Matcher::inline_cache_reg_encode());
      AddressLiteral oop = __ allocate_oop_address((jobject)Universe::non_oop_word());

      address virtual_call_oop_addr = __ pc();
      __ load_const_from_method_toc(ic_reg, oop, Rtoc);
      // CALL to fixup routine.  Fixup routine uses ScopeDesc info
      // to determine who we intended to call.
      __ relocate(virtual_call_Relocation::spec(virtual_call_oop_addr));

      EmitCallOffsets offsets = { -1, -1 };
      int ret_addr_offset = __ offset();
      offsets = emit_call_with_trampoline_stub(_masm, (address)opnd_array(1)->method(), relocInfo::none);
      ret_addr_offset += offsets.ret_addr_offset;
      set_lazy_constant(MachCallDynamicJavaNode, ret_addr_offset_no_vtable,
                        ret_addr_offset - start_offset);
      assert(((MachCallDynamicJavaNode*)this)->ret_addr_offset() == ret_addr_offset - start_offset,
             "Fix constant in ret_addr_offset()");
    } else {
      assert(!UseInlineCaches, "expect vtable calls only if not using ICs");
      // Go thru the vtable. Get receiver klass. Receiver already
      // checked for non-null. If we'll go thru a C2I adapter, the
      // interpreter expects method in R19_method.

      __ load_heap_oop_not_null(R11_scratch1, oopDesc::klass_offset_in_bytes(), R3);

      int entry_offset = instanceKlass::vtable_start_offset()
        + vtable_index * vtableEntry::size();
      int v_off = entry_offset * wordSize
        + vtableEntry::method_offset_in_bytes();
      __ li(R19_method, v_off);
      __ ldx(R19_method/*method oop*/,
             R19_method/*method offset*/,
             R11_scratch1/*class*/);
      // NOTE: for vtable dispatches, the vtable entry will never be
      // null. However it may very well end up in handle_wrong_method
      // if the method is abstract for the particular class.
      __ ld(R11_scratch1,
            in_bytes(methodOopDesc::from_compiled_offset()),
            R19_method);
      // Call target. Either compiled code or C2I adapter.
      __ mtctr(R11_scratch1);
      __ bctrl();
      int return_pc_offset = __ offset();
      assert(((MachCallDynamicJavaNode*)this)->ret_addr_offset() == return_pc_offset - start_offset,
             "Fix constant in ret_addr_offset()");
    }
#endif
    Unimplemented();
  
#line 17899 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void CallRuntimeDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 4853 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    const address start_pc = __ pc();

#if defined(ABI_ELFv2)
    address entry= !(opnd_array(1)->method()) ? NULL : (address)opnd_array(1)->method();
    __ call_c(entry, relocInfo::runtime_call_type);
#else
    // The function we're going to call.
    FunctionDescriptor fdtemp;
    const FunctionDescriptor* fd = !(opnd_array(1)->method()) ? &fdtemp : (FunctionDescriptor*)opnd_array(1)->method();

    Register Rtoc = R12_scratch2;
    // Calculate the method's TOC.
    __ calculate_address_from_global_toc(Rtoc, __ method_toc());
    // Put entry, env, toc into the constant pool, this needs up to 3 constant
    // pool entries; call_c_using_toc will optimize the call.
    __ call_c_using_toc(fd, relocInfo::runtime_call_type, Rtoc);
#endif

    // Check the ret_addr_offset.
    assert(((MachCallRuntimeNode*)this)->ret_addr_offset() == __ last_calls_return_pc() - start_pc,
           "Fix constant in ret_addr_offset()");
  
#line 17937 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void CallLeafDirect_mtctrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 4838 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_mtctr);
    MacroAssembler _masm(&cbuf);
    Register Rsrc = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ mtctr(Rsrc);
  
#line 17955 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint CallLeafDirect_mtctrNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void CallLeafDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 4846 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_bctrl);
    MacroAssembler _masm(&cbuf);
    __ bctrl();
  
#line 17977 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint CallLeafDirectNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void  CallLeafDirect_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // Access to ins and operands for late expand.
  unsigned idx_meth  = idx1; 	// method, 	meth
  Node    *n_region  = lookup(0);
  Node    *n_meth    = lookup(idx_meth);
  methodOper *op_meth = (methodOper *)opnd_array(1);
  Compile *C = ra_->C;
  {
#line 4733 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    Node *toc = in(TypeFunc::ReturnAdr);
    loadConLNodesTuple loadConLNodes_Entry;
#if defined(ABI_ELFv2)
    jlong entry_address = (jlong) this->entry_point();
    assert(entry_address, "need address here");
    loadConLNodes_Entry = loadConLNodesTuple_create(C, ra_, toc, new (C) immLOper(entry_address),
						     false, OptoReg::Name(R12_H_num), OptoReg::Name(R12_num));
#else
    // Get the struct that describes the function we are about to call.
    FunctionDescriptor* fd = (FunctionDescriptor*) this->entry_point();
    assert(fd, "need fd here");
    jlong entry_address = (jlong) fd->entry();

    // new nodes
    loadConLNodesTuple loadConLNodes_Env;
    loadConLNodesTuple loadConLNodes_Toc;

    // Create nodes and operands for loading the entry point.
    loadConLNodes_Entry = loadConLNodesTuple_create(C, ra_, toc, new (C) immLOper(entry_address),
                                                    false, OptoReg::Name(R12_H_num), OptoReg::Name(R12_num));

    // Create nodes and operands for loading the env pointer.
    if (fd->env() != NULL) {
      loadConLNodes_Env = loadConLNodesTuple_create(C, ra_, toc, new (C) immLOper((jlong) fd->env()),
                                                    false, OptoReg::Name(R11_H_num), OptoReg::Name(R11_num));
    } else {
      loadConLNodes_Env._large_hi = NULL;
      loadConLNodes_Env._large_lo = NULL;
      loadConLNodes_Env._small    = NULL;
      loadConLNodes_Env._last = new (C) loadConL16Node();
      loadConLNodes_Env._last->_opnds[0] = new (C) iRegLdstOper();
      loadConLNodes_Env._last->_opnds[1] = new (C) immL16Oper(0);
      ra_->set_pair(loadConLNodes_Env._last->_idx, OptoReg::Name(R11_H_num), OptoReg::Name(R11_num));
    }

    // Create nodes and operands for loading the Toc point.
    loadConLNodes_Toc = loadConLNodesTuple_create(C, ra_, toc, new (C) immLOper((jlong) fd->toc()),
                                                  false, OptoReg::Name(R2_H_num), OptoReg::Name(R2_num));
#endif // ABI_ELFv2
    // mtctr node
    MachNode *mtctr = new (C) CallLeafDirect_mtctrNode();
    assert(loadConLNodes_Entry._last != NULL, "entry must exist");
    mtctr->add_req(0, loadConLNodes_Entry._last);

    mtctr->_opnds[0] = new (C) iRegLdstOper();
    mtctr->_opnds[1] = new (C) iRegLdstOper();

    // call node
    MachCallLeafNode *call = new (C) CallLeafDirectNode();

    call->_opnds[0] = _opnds[0];
    call->_opnds[1] = new (C) methodOper((intptr_t) entry_address); // May get set later.

    // Make the new call node look like the old one.
    call->_name        = _name;
    call->_tf          = _tf;
    call->_entry_point = _entry_point;
    call->_cnt         = _cnt;
    call->_argsize     = _argsize;
    call->_oop_map     = _oop_map;
    call->_jvms        = _jvms;
    call->_jvmadj      = _jvmadj;
    call->_in_rms      = _in_rms;
    call->_nesting     = _nesting;

    // New call needs all inputs of old call.
    // Req...
    for (uint i = 0; i < req(); ++i) {
      if (i != TypeFunc::ReturnAdr) {
        call->add_req(in(i));
      } else {
        // put the mtctr where ReturnAdr would be
        call->add_req(mtctr);
      }
    }

    // ...as well as prec
    for (uint i = req(); i < len(); ++i) {
      call->add_prec(in(i));
    }

#if !defined(ABI_ELFv2)
    // ... and more prec.
    if (loadConLNodes_Env._last) call->add_prec(loadConLNodes_Env._last);
    if (loadConLNodes_Toc._last) call->add_prec(loadConLNodes_Toc._last);
#endif

    // registers
    ra_->set1(mtctr->_idx, OptoReg::Name(SR_CTR_num));

    // Insert the new nodes.
    if (loadConLNodes_Entry._large_hi) nodes->push(loadConLNodes_Entry._large_hi);
    if (loadConLNodes_Entry._last)     nodes->push(loadConLNodes_Entry._last);
#if !defined(ABI_ELFv2)
    if (loadConLNodes_Env._large_hi)   nodes->push(loadConLNodes_Env._large_hi);
    if (loadConLNodes_Env._last)       nodes->push(loadConLNodes_Env._last);
    if (loadConLNodes_Toc._large_hi)   nodes->push(loadConLNodes_Toc._large_hi);
    if (loadConLNodes_Toc._last)       nodes->push(loadConLNodes_Toc._last);
#endif
    nodes->push(mtctr);
    nodes->push(call);
  
#line 18101 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void  CallLeafNoFPDirect_ExNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // Access to ins and operands for late expand.
  unsigned idx_meth  = idx1; 	// method, 	meth
  Node    *n_region  = lookup(0);
  Node    *n_meth    = lookup(idx_meth);
  methodOper *op_meth = (methodOper *)opnd_array(1);
  Compile *C = ra_->C;
  {
#line 4733 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    Node *toc = in(TypeFunc::ReturnAdr);
    loadConLNodesTuple loadConLNodes_Entry;
#if defined(ABI_ELFv2)
    jlong entry_address = (jlong) this->entry_point();
    assert(entry_address, "need address here");
    loadConLNodes_Entry = loadConLNodesTuple_create(C, ra_, toc, new (C) immLOper(entry_address),
						     false, OptoReg::Name(R12_H_num), OptoReg::Name(R12_num));
#else
    // Get the struct that describes the function we are about to call.
    FunctionDescriptor* fd = (FunctionDescriptor*) this->entry_point();
    assert(fd, "need fd here");
    jlong entry_address = (jlong) fd->entry();

    // new nodes
    loadConLNodesTuple loadConLNodes_Env;
    loadConLNodesTuple loadConLNodes_Toc;

    // Create nodes and operands for loading the entry point.
    loadConLNodes_Entry = loadConLNodesTuple_create(C, ra_, toc, new (C) immLOper(entry_address),
                                                    false, OptoReg::Name(R12_H_num), OptoReg::Name(R12_num));

    // Create nodes and operands for loading the env pointer.
    if (fd->env() != NULL) {
      loadConLNodes_Env = loadConLNodesTuple_create(C, ra_, toc, new (C) immLOper((jlong) fd->env()),
                                                    false, OptoReg::Name(R11_H_num), OptoReg::Name(R11_num));
    } else {
      loadConLNodes_Env._large_hi = NULL;
      loadConLNodes_Env._large_lo = NULL;
      loadConLNodes_Env._small    = NULL;
      loadConLNodes_Env._last = new (C) loadConL16Node();
      loadConLNodes_Env._last->_opnds[0] = new (C) iRegLdstOper();
      loadConLNodes_Env._last->_opnds[1] = new (C) immL16Oper(0);
      ra_->set_pair(loadConLNodes_Env._last->_idx, OptoReg::Name(R11_H_num), OptoReg::Name(R11_num));
    }

    // Create nodes and operands for loading the Toc point.
    loadConLNodes_Toc = loadConLNodesTuple_create(C, ra_, toc, new (C) immLOper((jlong) fd->toc()),
                                                  false, OptoReg::Name(R2_H_num), OptoReg::Name(R2_num));
#endif // ABI_ELFv2
    // mtctr node
    MachNode *mtctr = new (C) CallLeafDirect_mtctrNode();
    assert(loadConLNodes_Entry._last != NULL, "entry must exist");
    mtctr->add_req(0, loadConLNodes_Entry._last);

    mtctr->_opnds[0] = new (C) iRegLdstOper();
    mtctr->_opnds[1] = new (C) iRegLdstOper();

    // call node
    MachCallLeafNode *call = new (C) CallLeafDirectNode();

    call->_opnds[0] = _opnds[0];
    call->_opnds[1] = new (C) methodOper((intptr_t) entry_address); // May get set later.

    // Make the new call node look like the old one.
    call->_name        = _name;
    call->_tf          = _tf;
    call->_entry_point = _entry_point;
    call->_cnt         = _cnt;
    call->_argsize     = _argsize;
    call->_oop_map     = _oop_map;
    call->_jvms        = _jvms;
    call->_jvmadj      = _jvmadj;
    call->_in_rms      = _in_rms;
    call->_nesting     = _nesting;

    // New call needs all inputs of old call.
    // Req...
    for (uint i = 0; i < req(); ++i) {
      if (i != TypeFunc::ReturnAdr) {
        call->add_req(in(i));
      } else {
        // put the mtctr where ReturnAdr would be
        call->add_req(mtctr);
      }
    }

    // ...as well as prec
    for (uint i = req(); i < len(); ++i) {
      call->add_prec(in(i));
    }

#if !defined(ABI_ELFv2)
    // ... and more prec.
    if (loadConLNodes_Env._last) call->add_prec(loadConLNodes_Env._last);
    if (loadConLNodes_Toc._last) call->add_prec(loadConLNodes_Toc._last);
#endif

    // registers
    ra_->set1(mtctr->_idx, OptoReg::Name(SR_CTR_num));

    // Insert the new nodes.
    if (loadConLNodes_Entry._large_hi) nodes->push(loadConLNodes_Entry._large_hi);
    if (loadConLNodes_Entry._last)     nodes->push(loadConLNodes_Entry._last);
#if !defined(ABI_ELFv2)
    if (loadConLNodes_Env._large_hi)   nodes->push(loadConLNodes_Env._large_hi);
    if (loadConLNodes_Env._last)       nodes->push(loadConLNodes_Env._last);
    if (loadConLNodes_Toc._large_hi)   nodes->push(loadConLNodes_Toc._large_hi);
    if (loadConLNodes_Toc._last)       nodes->push(loadConLNodes_Toc._last);
#endif
    nodes->push(mtctr);
    nodes->push(call);
  
#line 18220 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void TailCalljmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// method_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 4483 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    __ mtctr(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* jump_target */));
    __ bctr();
  
#line 18240 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint TailCalljmpIndNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 8);
}

void RetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 4475 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_blr);
    MacroAssembler _masm(&cbuf);
    // LR is restored in MachEpilogNode. Just do the RET here.
    __ blr();
  
#line 18263 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint RetNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void tailjmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// ex_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 4491 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);

    MacroAssembler _masm(&cbuf);
    Register Rdest = reg_to_register_object(opnd_array(1)->reg(ra_,this,idx1)/* jump_target */);
    __ ld(R4_ARG2/* issuing pc */, _abi(lr), R1_SP);
    __ mtctr(Rdest);
    __ bctr();
  
#line 18290 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint tailjmpIndNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void CreateExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint CreateExceptionNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void RethrowExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 6003 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_compound);
    MacroAssembler _masm(&cbuf);
    cbuf.set_insts_mark();
    __ b64_patchable((address)OptoRuntime::rethrow_stub(),
                         relocInfo::runtime_call_type);
  
#line 18327 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

void ShouldNotReachHereNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 4501 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_tdi);

    MacroAssembler _masm(&cbuf);
    __ trap_should_not_reach_here();
  
#line 18345 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint ShouldNotReachHereNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void tlsLoadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint tlsLoadPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void endGroupNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 5960 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_endgroup);
    MacroAssembler _masm(&cbuf);
    __ endgroup();
  
#line 18380 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint endGroupNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void fxNopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 5967 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fmr);
    MacroAssembler _masm(&cbuf);
    __ nop();
  
#line 18402 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint fxNopNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void fpNop0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 5973 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fmr);
    MacroAssembler _masm(&cbuf);
    __ fpnop0();
  
#line 18424 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint fpNop0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void fpNop1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 5979 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_fmr);
    MacroAssembler _masm(&cbuf);
    __ fpnop1();
  
#line 18446 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint fpNop1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void brNop0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 5985 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_mcrf);
    MacroAssembler _masm(&cbuf);
    __ brnop0();
  
#line 18468 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint brNop0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void brNop1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 5991 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_mcrf);
    MacroAssembler _masm(&cbuf);
    __ brnop1();
  
#line 18490 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint brNop1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void brNop2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 5997 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // TODO: PPC port $archOpcode(ppc64Opcode_mcrf);
    MacroAssembler _masm(&cbuf);
    __ brnop2();
  
#line 18512 "../generated/adfiles/ad_ppc_64.cpp"
  }
}

uint brNop2Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

const MachOper* loadUB_indirectNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB_indirect_acNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB_indirect_ExNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB_indirect_ac_ExNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB_indOffset16Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB_indOffset16_acNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB_indOffset16_ExNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB_indOffset16_ac_ExNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB_acNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2L_acNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS_acNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS_acNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2L_acNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI_acNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_acNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadL_acNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadL_unalignedNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadV8Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadRangeNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadN_acNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadN2P_unscaledNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadP_acNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadP2XNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadF_acNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadD_acNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadD_unalignedNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchr_no_offsetNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchw_no_offsetNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetch_alloc_zeroNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetch_alloc_zero_no_offsetNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetch_allocNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetch_alloc_no_offsetNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeCNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeI_convL2INode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeA8BNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeNNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeCM_CMSNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeCM_CMS_ExExNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeCM_G1Node::memory_operand() const { return _opnds[1]; }
const TypePtr *unnecessary_membar_acquireNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* unnecessary_membar_acquireNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_acquire_lockNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_acquire_lockNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_releaseNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_releaseNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_release_lockNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_release_lockNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_storestoreNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_storestoreNode::memory_operand() const { return (MachOper*)-1; }
const MachOper* storeLConditional_regP_regL_regLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePConditional_regP_regP_regPNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPLockedNode::memory_operand() const { return _opnds[1]; }


const bool Matcher::has_match_rule(int opcode) {
  assert(_last_machine_leaf < opcode && opcode < _last_opcode, "opcode in range");
  return _hasMatchRule[opcode];
}

const bool Matcher::_hasMatchRule[_last_opcode] = {
    false,  // Node
    false,  // Set
    false,  // RegN
    false,  // RegI
    false,  // RegP
    false,  // RegF
    false,  // RegD
    false,  // RegL
    false,  // RegFlags
    false,  // VecS
    false,  // VecD
    false,  // VecX
    false,  // VecY
    false,  // _last_machine_leaf
    true ,  // AbsD
    true ,  // AbsF
    true ,  // AbsI
    true ,  // AddD
    true ,  // AddF
    true ,  // AddI
    true ,  // AddL
    true ,  // AddP
    false,  // Allocate
    false,  // AllocateArray
    true ,  // AndI
    true ,  // AndL
    false,  // AryEq
    false,  // AtanD
    true ,  // Binary
    true ,  // Bool
    false,  // BoxLock
    true ,  // ReverseBytesI
    false,  // ReverseBytesL
    false,  // ReverseBytesUS
    false,  // ReverseBytesS
    false,  // CProj
    true ,  // CallDynamicJava
    false,  // CallJava
    true ,  // CallLeaf
    true ,  // CallLeafNoFP
    true ,  // CallRuntime
    true ,  // CallStaticJava
    true ,  // CastII
    true ,  // CastX2P
    true ,  // CastP2X
    true ,  // CastPP
    false,  // Catch
    false,  // CatchProj
    true ,  // CheckCastPP
    true ,  // ClearArray
    false,  // ConstraintCast
    true ,  // CMoveD
    true ,  // CMoveF
    true ,  // CMoveI
    true ,  // CMoveL
    true ,  // CMoveP
    true ,  // CMoveN
    true ,  // CmpN
    true ,  // CmpD
    true ,  // CmpD3
    true ,  // CmpF
    true ,  // CmpF3
    true ,  // CmpI
    true ,  // CmpL
    true ,  // CmpL3
    true ,  // CmpLTMask
    true ,  // CmpP
    true ,  // CmpU
    true ,  // CmpUL
    true ,  // CompareAndSwapI
    true ,  // CompareAndSwapL
    true ,  // CompareAndSwapP
    true ,  // CompareAndSwapN
    true ,  // GetAndAddI
    true ,  // GetAndAddL
    true ,  // GetAndSetI
    true ,  // GetAndSetL
    true ,  // GetAndSetP
    true ,  // GetAndSetN
    false,  // Con
    true ,  // ConN
    true ,  // ConD
    true ,  // ConF
    true ,  // ConI
    true ,  // ConL
    true ,  // ConP
    true ,  // Conv2B
    true ,  // ConvD2F
    true ,  // ConvD2I
    true ,  // ConvD2L
    true ,  // ConvF2D
    true ,  // ConvF2I
    true ,  // ConvF2L
    true ,  // ConvI2D
    true ,  // ConvI2F
    true ,  // ConvI2L
    true ,  // ConvL2D
    true ,  // ConvL2F
    true ,  // ConvL2I
    false,  // CosD
    false,  // CountedLoop
    true ,  // CountedLoopEnd
    true ,  // CountLeadingZerosI
    true ,  // CountLeadingZerosL
    true ,  // CountTrailingZerosI
    true ,  // CountTrailingZerosL
    true ,  // CreateEx
    true ,  // DecodeN
    true ,  // DivD
    true ,  // DivF
    true ,  // DivI
    true ,  // DivL
    false,  // DivMod
    false,  // DivModI
    false,  // DivModL
    true ,  // EncodeP
    false,  // ExpD
    true ,  // FastLock
    true ,  // FastUnlock
    true ,  // Goto
    true ,  // Halt
    true ,  // If
    false,  // IfFalse
    false,  // IfTrue
    false,  // Initialize
    false,  // JProj
    false,  // Jump
    false,  // JumpProj
    true ,  // LShiftI
    true ,  // LShiftL
    true ,  // LoadB
    true ,  // LoadUB
    true ,  // LoadUS
    true ,  // LoadD
    true ,  // LoadD_unaligned
    true ,  // LoadF
    true ,  // LoadI
    true ,  // LoadKlass
    true ,  // LoadNKlass
    true ,  // LoadL
    true ,  // LoadL_unaligned
    true ,  // LoadPLocked
    true ,  // LoadP
    true ,  // LoadN
    true ,  // LoadRange
    true ,  // LoadS
    false,  // Lock
    false,  // LogD
    false,  // Log10D
    false,  // Loop
    false,  // LoopLimit
    false,  // Mach
    false,  // MachProj
    true ,  // MaxI
    true ,  // MemBarAcquire
    true ,  // MemBarAcquireLock
    true ,  // MemBarCPUOrder
    true ,  // MemBarRelease
    true ,  // MemBarReleaseLock
    true ,  // MemBarVolatile
    true ,  // MemBarStoreStore
    false,  // MergeMem
    true ,  // MinI
    false,  // ModD
    false,  // ModF
    true ,  // ModI
    true ,  // ModL
    true ,  // MoveI2F
    true ,  // MoveF2I
    true ,  // MoveL2D
    true ,  // MoveD2L
    true ,  // MulD
    true ,  // MulF
    true ,  // MulHiL
    true ,  // MulI
    true ,  // MulL
    false,  // Multi
    true ,  // NegD
    true ,  // NegF
    false,  // NeverBranch
    false,  // Opaque1
    false,  // Opaque2
    true ,  // OrI
    true ,  // OrL
    false,  // PCTable
    false,  // Parm
    true ,  // PartialSubtypeCheck
    false,  // Phi
    true ,  // PopCountI
    true ,  // PopCountL
    false,  // PowD
    true ,  // PrefetchAllocation
    true ,  // PrefetchRead
    true ,  // PrefetchWrite
    false,  // Proj
    true ,  // RShiftI
    true ,  // RShiftL
    false,  // Region
    true ,  // Rethrow
    true ,  // Return
    false,  // Root
    true ,  // RoundDouble
    true ,  // RoundFloat
    true ,  // SafePoint
    false,  // SafePointScalarObject
    false,  // SCMemProj
    false,  // SinD
    true ,  // SqrtD
    false,  // Start
    false,  // StartOSR
    true ,  // StoreB
    true ,  // StoreC
    true ,  // StoreCM
    true ,  // StorePConditional
    false,  // StoreIConditional
    true ,  // StoreLConditional
    true ,  // StoreD
    true ,  // StoreF
    true ,  // StoreI
    true ,  // StoreL
    true ,  // StoreP
    true ,  // StoreN
    true ,  // StrComp
    true ,  // StrEquals
    true ,  // StrIndexOf
    true ,  // SubD
    true ,  // SubF
    true ,  // SubI
    true ,  // SubL
    true ,  // TailCall
    true ,  // TailJump
    false,  // TanD
    true ,  // ThreadLocal
    false,  // Unlock
    true ,  // URShiftI
    true ,  // URShiftL
    true ,  // XorI
    true ,  // XorL
    false,  // Vector
    false,  // AddVB
    false,  // AddVS
    false,  // AddVI
    false,  // AddVL
    false,  // AddVF
    false,  // AddVD
    false,  // SubVB
    false,  // SubVS
    false,  // SubVI
    false,  // SubVL
    false,  // SubVF
    false,  // SubVD
    false,  // MulVS
    false,  // MulVI
    false,  // MulVF
    false,  // MulVD
    false,  // DivVF
    false,  // DivVD
    false,  // LShiftCntV
    false,  // RShiftCntV
    false,  // LShiftVB
    false,  // LShiftVS
    false,  // LShiftVI
    false,  // LShiftVL
    false,  // RShiftVB
    false,  // RShiftVS
    false,  // RShiftVI
    false,  // RShiftVL
    false,  // URShiftVB
    false,  // URShiftVS
    false,  // URShiftVI
    false,  // URShiftVL
    false,  // AndV
    false,  // OrV
    false,  // XorV
    true ,  // LoadVector
    true ,  // StoreVector
    false,  // Pack
    false,  // PackB
    false,  // PackS
    false,  // PackI
    false,  // PackL
    false,  // PackF
    false,  // PackD
    false,  // Pack2L
    false,  // Pack2D
    true ,  // ReplicateB
    true ,  // ReplicateS
    true ,  // ReplicateI
    false,  // ReplicateL
    true ,  // ReplicateF
    false,  // ReplicateD
    false,  // Extract
    false,  // ExtractB
    false,  // ExtractUB
    false,  // ExtractC
    false,  // ExtractS
    false,  // ExtractI
    false,  // ExtractL
    false,  // ExtractF
    false   // ExtractD
};


bool Matcher::stack_direction() const { return false; }

int Compile::sync_stack_slots() const { return (frame::jit_monitor_size / VMRegImpl::stack_slot_size); }

uint Matcher::stack_alignment_in_bytes() { return frame::alignment_in_bytes; }

OptoReg::Name Matcher::return_addr() const { return OptoReg::stack2reg(4); }

uint Compile::in_preserve_stack_slots() { return (frame::jit_in_preserve_size / VMRegImpl::stack_slot_size); }

uint Compile::out_preserve_stack_slots() { return SharedRuntime::out_preserve_stack_slots(); }

uint Compile::varargs_C_out_slots_killed() const { return ((frame::abi_reg_args_size - frame::jit_out_preserve_size) / VMRegImpl::stack_slot_size); }

void Matcher::calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length, bool is_outgoing) {

#line 6162 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // No difference between ingoing/outgoing. Just pass false.
    SharedRuntime::java_calling_convention(sig_bt, regs, length, false);
  
#line 18928 "../generated/adfiles/ad_ppc_64.cpp"

}

void Matcher::c_calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length) {

#line 6174 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    // This is obviously always outgoing.
    // C argument in register AND stack slot.
    (void) SharedRuntime::c_calling_convention(sig_bt, regs, /*regs2=*/NULL, length);
  
#line 18940 "../generated/adfiles/ad_ppc_64.cpp"

}

OptoRegPair Matcher::return_value(int ideal_reg, bool is_outgoing) {

#line 6197 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    assert((ideal_reg >= Op_RegI && ideal_reg <= Op_RegL) ||
            (ideal_reg == Op_RegN && Universe::narrow_oop_base() == NULL && Universe::narrow_oop_shift() == 0),
            "only return normal values");
    // enum names from opcodes.hpp:    Op_Node Op_Set Op_RegN       Op_RegI       Op_RegP       Op_RegF       Op_RegD       Op_RegL
    static int typeToRegLo[Op_RegL+1] = { 0,   0,     R3_num,   R3_num,   R3_num,   F1_num,   F1_num,   R3_num };
    static int typeToRegHi[Op_RegL+1] = { 0,   0,     OptoReg::Bad, R3_H_num, R3_H_num, OptoReg::Bad, F1_H_num, R3_H_num };
    return OptoRegPair(typeToRegHi[ideal_reg], typeToRegLo[ideal_reg]);
  
#line 18956 "../generated/adfiles/ad_ppc_64.cpp"

}

OptoRegPair Matcher::c_return_value(int ideal_reg, bool is_outgoing) {

#line 6186 "/builddir/build/BUILD/java-1.7.0-openjdk-1.7.0.171-2.6.13.0.el7_4.ppc64/openjdk/hotspot/src/cpu/ppc/vm/ppc.ad"

    assert((ideal_reg >= Op_RegI && ideal_reg <= Op_RegL) ||
            (ideal_reg == Op_RegN && Universe::narrow_oop_base() == NULL && Universe::narrow_oop_shift() == 0),
            "only return normal values");
    // enum names from opcodes.hpp:    Op_Node Op_Set Op_RegN       Op_RegI       Op_RegP       Op_RegF       Op_RegD       Op_RegL
    static int typeToRegLo[Op_RegL+1] = { 0,   0,     R3_num,   R3_num,   R3_num,   F1_num,   F1_num,   R3_num };
    static int typeToRegHi[Op_RegL+1] = { 0,   0,     OptoReg::Bad, R3_H_num, R3_H_num, OptoReg::Bad, F1_H_num, R3_H_num };
    return OptoRegPair(typeToRegHi[ideal_reg], typeToRegLo[ideal_reg]);
  
#line 18972 "../generated/adfiles/ad_ppc_64.cpp"

}

OptoReg::Name Matcher::inline_cache_reg() { return OptoReg::Name(R19_num); }

int Matcher::inline_cache_reg_encode() { return _regEncode[inline_cache_reg()]; }

OptoReg::Name Matcher::interpreter_method_oop_reg() { return OptoReg::Name(R19_num); }

int Matcher::interpreter_method_oop_reg_encode() { return _regEncode[interpreter_method_oop_reg()]; }

OptoReg::Name Matcher::interpreter_frame_pointer_reg() { return OptoReg::Name(R14_num); }

OptoReg::Name Matcher::c_frame_pointer() const { return OptoReg::Name(R1_num); }

// Number of callee-save + always-save registers
int  Matcher::number_of_saved_registers() {
  return 0;
};

// Check consistency of C++ compilation with ADLC options:
// Check adlc -DLINUX=1
#ifndef LINUX
#  error "LINUX must be defined"
#endif // LINUX
// Check adlc -D_GNU_SOURCE=1
#ifndef _GNU_SOURCE
#  error "_GNU_SOURCE must be defined"
#endif // _GNU_SOURCE
// Check adlc -DPPC64=1
#ifndef PPC64
#  error "PPC64 must be defined"
#endif // PPC64
// Check adlc -D_LP64=1
#ifndef _LP64
#  error "_LP64 must be defined"
#endif // _LP64
